@@ -118,7 +118,7 @@ static void on_complete_process (w_and_interface_t *wi)
       if ( wi->w->source_interface->autoview ) {
 	vik_trw_layer_auto_set_view ( wi->vtl, vik_layers_panel_get_viewport(wi->w->vlp) );
       }
-      vik_layers_panel_emit_update ( wi->w->vlp );
+      vik_layers_panel_emit_update ( wi->w->vlp, TRUE );
     }
   } else {
     /* cancelled */
@@ -239,10 +239,7 @@ static void acquire ( VikWindow *vw,
     dialog = gtk_dialog_new_with_buttons ( "", GTK_WINDOW(vw), 0, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL );
 
     gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
-    GtkWidget *response_w = NULL;
-#if GTK_CHECK_VERSION (2, 20, 0)
-    response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
-#endif
+    GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 
     source_interface->add_setup_widgets_func(dialog, vvp, user_data);
     gtk_window_set_title ( GTK_WINDOW(dialog), _(source_interface->window_title) );
@@ -271,27 +268,36 @@ static void acquire ( VikWindow *vw,
   /* CREATE INPUT DATA & GET OPTIONS */
   ProcessOptions *po = g_malloc0 ( sizeof(ProcessOptions) );
 
+
   if ( source_interface->inputtype == VIK_DATASOURCE_INPUTTYPE_TRWLAYER ) {
-    gchar *name_src = a_gpx_write_tmp_file ( vtl, NULL );
 
-    source_interface->get_process_options_func ( pass_along_data, po, NULL, name_src, NULL );
+    if ( vtl ) {
+      GpxWritingOptions gpx_options = { FALSE, FALSE, FALSE, FALSE, vik_trw_layer_get_gpx_version(vtl) };
+      gchar *name_src = a_gpx_write_tmp_file ( vtl, &gpx_options );
+      source_interface->get_process_options_func ( pass_along_data, po, NULL, name_src, NULL );
+      util_add_to_deletion_list ( name_src );
+      g_free ( name_src );
+    } else
+      g_critical ( "%s: vtl not defined", __FUNCTION__ );
 
-    util_add_to_deletion_list ( name_src );
-
-    g_free ( name_src );
   } else if ( source_interface->inputtype == VIK_DATASOURCE_INPUTTYPE_TRWLAYER_TRACK ) {
-    gchar *name_src = a_gpx_write_tmp_file ( vtl, NULL );
-    gchar *name_src_track = a_gpx_write_track_tmp_file ( track, NULL );
 
-    source_interface->get_process_options_func ( pass_along_data, po, NULL, name_src, name_src_track );
+    if ( vtl && track ) {
+      GpxWritingOptions gpx_options = { FALSE, FALSE, FALSE, FALSE, vik_trw_layer_get_gpx_version(vtl) };
+      gchar *name_src = a_gpx_write_tmp_file ( vtl, &gpx_options );
+      gchar *name_src_track = a_gpx_write_track_tmp_file ( vtl, track, &gpx_options );
+      source_interface->get_process_options_func ( pass_along_data, po, NULL, name_src, name_src_track );
+      util_add_to_deletion_list ( name_src );
+      util_add_to_deletion_list ( name_src_track );
+      g_free ( name_src );
+      g_free ( name_src_track );
+    } else
+      g_critical ( "%s: vtl and/or track not defined", __FUNCTION__ );
 
-    util_add_to_deletion_list ( name_src );
-    util_add_to_deletion_list ( name_src_track );
-
-    g_free ( name_src );
-    g_free ( name_src_track );
   } else if ( source_interface->inputtype == VIK_DATASOURCE_INPUTTYPE_TRACK ) {
-    gchar *name_src_track = a_gpx_write_track_tmp_file ( track, NULL );
+
+    GpxWritingOptions gpx_options = { FALSE, FALSE, FALSE, FALSE, vtl ? vik_trw_layer_get_gpx_version(vtl) : GPX_V1_0 };
+    gchar *name_src_track = a_gpx_write_track_tmp_file ( vtl, track, &gpx_options );
 
     source_interface->get_process_options_func ( pass_along_data, po, NULL, NULL, name_src_track );
 
@@ -367,11 +373,7 @@ static void acquire ( VikWindow *vw,
 
   if ( source_interface->is_thread ) {
     if ( po->babelargs || po->url || po->shell_command ) {
-#if GLIB_CHECK_VERSION (2, 32, 0)
       g_thread_try_new ( "get_from_anything", (GThreadFunc)get_from_anything, wi, NULL );
-#else
-      g_thread_create ( (GThreadFunc)get_from_anything, wi, FALSE, NULL );
-#endif
       gtk_dialog_run ( GTK_DIALOG(dialog) );
       if (w->running) {
         // Cancel and mark for thread to finish
@@ -467,6 +469,10 @@ static GtkWidget *acquire_build_menu ( VikWindow *vw, VikLayersPanel *vlp, VikVi
   GtkWidget *item=NULL;
   int i;
 
+  // ATM All the filter options invoke GPSBabel
+  if ( !a_babel_available() )
+    return NULL;
+
   pass_along[0] = vw;
   pass_along[1] = vlp;
   pass_along[2] = vvp;
@@ -477,7 +483,8 @@ static GtkWidget *acquire_build_menu ( VikWindow *vw, VikLayersPanel *vlp, VikVi
     if ( filters[i]->inputtype == inputtype ) {
       if ( ! menu_item ) { /* do this just once, but return NULL if no filters */
         menu = gtk_menu_new();
-        menu_item = gtk_menu_item_new_with_mnemonic ( menu_title );
+        menu_item = gtk_image_menu_item_new_with_mnemonic ( menu_title );
+        gtk_image_menu_item_set_image ( (GtkImageMenuItem*)menu_item, gtk_image_new_from_stock(VIK_ICON_FILTER, GTK_ICON_SIZE_MENU) );
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu );
       }
 
