@@ -32,8 +32,9 @@
 
 #include "viking.h"
 #include "gpx.h"
+#include "kml.h"
+#include "geojson.h"
 #include "babel.h"
-#include "preferences.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -54,12 +55,12 @@ static gchar *unbuffer_loc = NULL;
 /**
  * List of file formats supported by gpsbabel.
  */
-GList *a_babel_file_list;
+static GList *a_babel_file_list = NULL;
 
 /**
- * List of device supported by gpsbabel.
+ * List of devices supported by gpsbabel.
  */
-GList *a_babel_device_list;
+static GList *a_babel_device_list = NULL;
 
 /**
  * Run a function on all file formats supporting a given mode.
@@ -171,10 +172,12 @@ static gboolean babel_general_convert( BabelStatusFunc cb, gchar **args, gpointe
   gint babel_stdout;
 
   if ( vik_debug ) {
-    (void)g_printf ( "%s:", __FUNCTION__ );
+    GString *gstr = g_string_new ( NULL );
+    g_string_append_printf ( gstr, "%s:", __FUNCTION__ );
     for ( guint i=0; args[i]; i++ )
-      (void)g_printf ( " %s", args[i] );
-    (void)g_printf ( "\n" );
+      g_string_append_printf ( gstr, " %s", args[i] );
+    g_message ( "%s", gstr->str );
+    g_string_free ( gstr, TRUE );
   }
 
   if (!g_spawn_async_with_pipes (NULL, args, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL, &babel_stdout, NULL, &error)) {
@@ -240,7 +243,7 @@ static gboolean babel_general_convert_from( VikTrwLayer *vt, BabelStatusFunc cb,
     f = g_fopen(name_dst, "r");
     if (f) {
       gchar *dirpath = g_path_get_dirname ( name_dst );
-      ret = a_gpx_read_file ( vt, f, dirpath );
+      ret = a_gpx_read_file ( vt, f, dirpath, FALSE );
       g_free ( dirpath );
       fclose(f);
     }
@@ -383,14 +386,17 @@ gboolean a_babel_convert_from_shellcommand ( VikTrwLayer *vt, const char *input_
  *
  * Download the file pointed by the URL and optionally uses GPSBabel to convert from input_type.
  * If input_type and babelfilters are %NULL, gpsbabel is not used.
+ * If input_type is 'gpx' or 'kml' or 'geojson-osrm' then we will use our own native parsers.
+ * NB TCX and indeed Vikings own files aren't available via this method since they can only be
+ * loaded into an aggregate layer (i.e. not into a %VikTrwLayer which is all that is available here)
  *
- * Returns: %TRUE on successful invocation of GPSBabel or read of the GPX
+ * Returns: %TRUE on successful invocation of GPSBabel or read of the downloaded file
  *
  */
 gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, const char *input_type, const char *babelfilters, BabelStatusFunc cb, gpointer user_data, DownloadFileOptions *options )
 {
   // If no download options specified, use defaults:
-  DownloadFileOptions myoptions = { FALSE, FALSE, NULL, 2, NULL, NULL, NULL };
+  DownloadFileOptions myoptions = { FALSE, FALSE, NULL, 2, NULL, NULL, NULL, FALSE, FALSE, NULL };
   if ( options )
     myoptions = *options;
   gint fd_src;
@@ -408,18 +414,43 @@ gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, con
 
     fetch_ret = a_http_download_get_url(url, "", name_src, &myoptions, NULL);
     if (fetch_ret == DOWNLOAD_SUCCESS) {
-      if (input_type != NULL || babelfilters != NULL) {
-        babelargs = (input_type) ? g_strdup_printf(" -i %s", input_type) : g_strdup("");
-        ret = a_babel_convert_from_filter( vt, babelargs, name_src, babelfilters, NULL, NULL, NULL );
-      } else {
+      gboolean do_gpx = FALSE;
+      gboolean do_kml = FALSE;
+      gboolean do_gjo = FALSE;
+      if ( g_strcmp0(input_type, "gpx") == 0 )
+        do_gpx = TRUE;
+      if ( g_strcmp0(input_type, "kml") == 0 )
+        do_kml = TRUE;
+      if ( g_strcmp0(input_type, "viking-geojson-osrm") == 0 )
+        do_gjo = TRUE;
+
+      if ( !do_gpx && !do_kml && !do_gjo ) {
+        if (input_type != NULL || babelfilters != NULL) {
+          babelargs = (input_type) ? g_strdup_printf(" -i %s", input_type) : g_strdup("");
+          ret = a_babel_convert_from_filter( vt, babelargs, name_src, babelfilters, NULL, NULL, NULL );
+        } else
+          // No input_type specified - resort to GPX
+          do_gpx = TRUE;
+      }
+      if ( do_gpx || do_kml || do_gjo ) {
         /* Process directly the retrieved file */
-        g_debug("%s: directly read GPX file %s", __FUNCTION__, name_src);
+        g_debug ( "%s: directly read file %s", __FUNCTION__, name_src );
         FILE *f = g_fopen(name_src, "r");
         if (f) {
           gchar *dirpath = g_path_get_dirname ( name_src );
-          ret = a_gpx_read_file ( vt, f, dirpath );
+          if ( do_kml ) {
+            ret = a_kml_read_file ( vt, f );
+            fclose(f);
+          }
+          else if ( do_gjo ) {
+            fclose(f);
+            ret = a_geojson_read_file_OSRM ( vt, name_src );
+          }
+          else {
+            ret = a_gpx_read_file ( vt, f, dirpath, FALSE );
+            fclose(f);
+          }
           g_free ( dirpath );
-          fclose(f);
         }
         // Try to avoid adding the description if URL is OAuth signed
         if ( !g_ascii_strncasecmp(url, "?oauth_consumer_key=", 20) ) {
@@ -564,7 +595,8 @@ static void load_feature_parse_line (gchar *line)
         device->name = g_strdup (tokens[2]);
         device->label = g_strndup (tokens[4], 50); // Limit really long label text
         a_babel_device_list = g_list_append (a_babel_device_list, device);
-        g_debug ("New gpsbabel device: %s, %d%d%d%d%d%d(%s)",
+        if ( vik_verbose )
+          g_debug ("New gpsbabel device: %s, %d%d%d%d%d%d(%s)",
         		device->name,
         		device->mode.waypointsRead, device->mode.waypointsWrite,
         		device->mode.tracksRead, device->mode.tracksWrite,
@@ -584,7 +616,8 @@ static void load_feature_parse_line (gchar *line)
         file->ext = g_strdup (tokens[3]);
         file->label = g_strdup (tokens[4]);
         a_babel_file_list = g_list_append (a_babel_file_list, file);
-        g_debug ("New gpsbabel file: %s, %d%d%d%d%d%d(%s)",
+        if ( vik_verbose )
+          g_debug ("New gpsbabel file: %s, %d%d%d%d%d%d(%s)",
 			file->name,
 			file->mode.waypointsRead, file->mode.waypointsWrite,
 			file->mode.tracksRead, file->mode.tracksWrite,
@@ -626,9 +659,24 @@ static gboolean load_feature ()
   return ret;
 }
 
+static VikLayerParamData gb_default ( void ) {
+  VikLayerParamData vlpd;
+#ifdef WINDOWS
+  // Basic guesses - could use %ProgramFiles% but this is simpler:
+  if ( g_file_test ( "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe", G_FILE_TEST_EXISTS ) )
+    // 32 bit location on a 64 bit system
+    vlpd.s = g_strdup ( "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe" );
+  else
+    vlpd.s = g_strdup ( "C:\\Program Files\\GPSBabel\\gpsbabel.exe" );
+#else
+  vlpd.s = g_strdup ( "gpsbabel" );
+#endif
+  return vlpd;
+}
+
 static VikLayerParam prefs[] = {
   { VIK_LAYER_NUM_TYPES, VIKING_PREFERENCES_IO_NAMESPACE "gpsbabel", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("GPSBabel:"), VIK_LAYER_WIDGET_FILEENTRY, NULL, NULL,
-      N_("Allow setting the specific instance of GPSBabel. You must restart Viking for this value to take effect."), NULL, NULL, NULL },
+      N_("Allow setting the specific instance of GPSBabel. You must restart Viking for this value to take effect."), gb_default, NULL, NULL },
 };
 
 /**
@@ -638,19 +686,7 @@ static VikLayerParam prefs[] = {
  */
 void a_babel_init ()
 {
-  // Set the defaults
-  VikLayerParamData vlpd;
-#ifdef WINDOWS
-  // Basic guesses - could use %ProgramFiles% but this is simpler:
-  if ( g_file_test ( "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe", G_FILE_TEST_EXISTS ) )
-    // 32 bit location on a 64 bit system
-    vlpd.s = "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe";
-  else
-    vlpd.s = "C:\\Program Files\\GPSBabel\\gpsbabel.exe";
-#else
-  vlpd.s = "gpsbabel";
-#endif
-  a_preferences_register(&prefs[0], vlpd, VIKING_PREFERENCES_IO_GROUP_KEY);
+  a_preferences_register ( &prefs[0], (VikLayerParamData){0}, VIKING_PREFERENCES_IO_GROUP_KEY );
 }
 
 /**
@@ -734,3 +770,23 @@ gboolean a_babel_available ()
 {
   return a_babel_device_list != NULL;
 }
+
+/**
+ * a_babel_file_list_get:
+ *
+ * Returns: String list of babel file formats. Maybe NULL.
+ */
+GList *a_babel_file_list_get ()
+{
+  return a_babel_file_list;
+}
+
+/**
+ * a_babel_device_list_get:
+ *
+ * Returns: String list of babel devices. Maybe NULL.
+ */
+GList *a_babel_device_list_get ()
+{
+  return a_babel_device_list;
+}
