@@ -34,23 +34,16 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_MATH_H
 #include <math.h>
-#endif
-
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
-#include <gdk-pixbuf/gdk-pixdata.h>
 #include "globals.h"
 #include "bingmapsource.h"
 #include "maputils.h"
 #include "bbox.h"
 #include "background.h"
-#include "icons/icons.h"
 
 /* Format for URL */
 #define URL_ATTR_FMT "http://dev.virtualearth.net/REST/v1/Imagery/Metadata/Aerial/0,0?zl=1&mapVersion=v1&key=%s&include=ImageryProviders&output=xml"
@@ -85,7 +78,8 @@ struct _BingMapSourcePrivate
 /* The pixbuf to store the logo */
 static GdkPixbuf *pixbuf = NULL;
 
-#define BING_MAP_SOURCE_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BING_TYPE_MAP_SOURCE, BingMapSourcePrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (BingMapSource, bing_map_source, VIK_TYPE_SLIPPY_MAP_SOURCE)
+#define BING_MAP_SOURCE_GET_PRIVATE(o)  (bing_map_source_get_instance_private (BING_MAP_SOURCE(o)))
 
 /* properties */
 enum
@@ -97,13 +91,11 @@ enum
 	PROP_API_KEY,
 };
 
-G_DEFINE_TYPE (BingMapSource, bing_map_source, VIK_TYPE_SLIPPY_MAP_SOURCE);
-
 static void
 bing_map_source_init (BingMapSource *self)
 {
 	/* initialize the object here */
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = bing_map_source_get_instance_private (self);
 
 	priv->hostname = NULL;
 	priv->url = NULL;
@@ -117,7 +109,7 @@ static void
 bing_map_source_finalize (GObject *object)
 {
 	BingMapSource *self = BING_MAP_SOURCE (object);
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = bing_map_source_get_instance_private (self);
 
 	g_free (priv->hostname);
 	priv->hostname = NULL;
@@ -136,7 +128,7 @@ _set_property (GObject      *object,
                GParamSpec   *pspec)
 {
 	BingMapSource *self = BING_MAP_SOURCE (object);
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = bing_map_source_get_instance_private (self);
 
 	switch (property_id)
 	{
@@ -168,7 +160,7 @@ _get_property (GObject    *object,
                GParamSpec *pspec)
 {
 	BingMapSource *self = BING_MAP_SOURCE (object);
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = bing_map_source_get_instance_private (self);
 
 	switch (property_id)
 	{
@@ -229,11 +221,14 @@ bing_map_source_class_init (BingMapSourceClass *klass)
                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_API_KEY, pspec);
 
-	g_type_class_add_private (klass, sizeof (BingMapSourcePrivate));
-	
 	object_class->finalize = bing_map_source_finalize;
 
-	pixbuf = gdk_pixbuf_from_pixdata ( &bing_maps_pixbuf, TRUE, NULL );
+	GError *error = NULL;
+	pixbuf = gdk_pixbuf_new_from_resource ( "/org/viking-gps/viking/assets/bing_maps.png", &error );
+	if ( error ) {
+		g_critical ( "%s: %s", __FUNCTION__, error->message );
+		g_error_free ( error );
+	}
 }
 
 static gchar *
@@ -290,7 +285,8 @@ static void
 _get_copyright(VikMapSource * self, LatLonBBox bbox, gdouble zoom, void (*fct)(VikViewport*,const gchar*), void *data)
 {
 	g_return_if_fail (BING_IS_MAP_SOURCE(self));
-	g_debug("%s: looking for %g %g %g %g at %g", __FUNCTION__, bbox.south, bbox.north, bbox.east, bbox.west, zoom);
+	if ( vik_verbose )
+		g_debug("%s: looking for %g %g %g %g at %g", __FUNCTION__, bbox.south, bbox.north, bbox.east, bbox.west, zoom);
 
 	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE(self);
 
@@ -327,8 +323,7 @@ bstart_element (GMarkupParseContext *context,
                 gpointer             user_data,
                 GError             **error)
 {
-	BingMapSource *self = BING_MAP_SOURCE (user_data);
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (user_data);
 	const gchar *element = g_markup_parse_context_get_element (context);
 	if (strcmp (element, "CoverageArea") == 0) {
 		/* New Attribution */
@@ -347,8 +342,7 @@ btext (GMarkupParseContext *context,
        gpointer             user_data,
        GError             **error)
 {
-	BingMapSource *self = BING_MAP_SOURCE (user_data);
-	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (self);
+	BingMapSourcePrivate *priv = BING_MAP_SOURCE_GET_PRIVATE (user_data);
 
 	struct _Attribution *attribution = priv->attributions == NULL ? NULL : g_list_last (priv->attributions)->data;
 	const gchar *element = g_markup_parse_context_get_element (context);
@@ -463,7 +457,7 @@ _load_attributions ( BingMapSource *self )
 	priv->loading_attributions = TRUE;
 	gchar *uri = g_strdup_printf(URL_ATTR_FMT, priv->api_key);
 
-	gchar *tmpname = a_download_uri_to_tmp_file ( uri, vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self)) );
+	gchar *tmpname = a_download_uri_to_tmp_file ( uri, vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self),NULL) );
 	if ( !tmpname ) {
 		ret = -1;
 		goto done;
