@@ -381,11 +381,8 @@ static GList *select_from_list (GtkWindow *parent, GList *list, const gchar *tit
 													 NULL);
 	/* When something is selected then OK */
 	gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
-	GtkWidget *response_w = NULL;
-#if GTK_CHECK_VERSION (2, 20, 0)
 	/* Default to not apply - as initially nothing is selected! */
-	response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
-#endif
+	GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
 	GtkWidget *label = gtk_label_new ( msg );
 	GtkTreeStore *store = gtk_tree_store_new ( 6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN );
 	GList *list_runner = list;
@@ -565,7 +562,7 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOpti
 #endif
 
 	// Support .zip + bzip2 files directly
-	DownloadFileOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass, a_try_decompress_file }; // Allow a couple of redirects
+	DownloadFileOptions options = { FALSE, FALSE, NULL, 2, NULL, NULL, user_pass, FALSE, FALSE, a_try_decompress_file }; // Allow a couple of redirects
 
 	gchar *tmpname = a_download_uri_to_tmp_file ( uri, &options );
 	g_free ( uri );
@@ -691,7 +688,7 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOpti
 	// Would prefer to keep the update in acquire.c,
 	//  however since we may create the layer - need to do the update here
 	if ( got_something )
-		vik_layer_emit_update ( VIK_LAYER(vtl_last) );
+		vik_layer_emit_update ( VIK_LAYER(vtl_last), TRUE );
 
 	// ATM The user is only informed if all getting *all* of the traces failed
 	if ( selected )
