@@ -23,7 +23,7 @@
 /*
  * This uses EXIF information from images to create waypoints at those positions
  *
- * The intial implementation uses libexif, which keeps Viking a pure C program.
+ * The initial implementation uses libexif, which keeps Viking a pure C program.
  * Now libgexiv2 is available (in C as a wrapper around the more powerful libexiv2 [C++]) so this is the preferred build.
  *  The attentative reader will notice the use of gexiv2 is a lot simpler as well.
  * For the time being the libexif code + build is still made available.
@@ -35,7 +35,9 @@
 #include "file.h"
 
 #include <sys/stat.h>
+#ifdef HAVE_UTIME_H
 #include <utime.h>
+#endif
 #include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
@@ -49,7 +51,20 @@
 #include "libjpeg/jpeg-data.h"
 #endif
 
+#define EXIF_GPS_IMGDIR "Exif.GPSInfo.GPSImgDirection"
+#define EXIF_GPS_IMGDIR_REF "Exif.GPSInfo.GPSImgDirectionRef"
+
 #ifdef HAVE_LIBGEXIV2
+// Compatibility
+static void metadata_free ( GExiv2Metadata *gemd )
+{
+#if GEXIV2_CHECK_VERSION (0,10,4)
+	g_object_unref ( gemd );
+#else
+	gexiv2_metadata_free  ( gemd );
+#endif
+}
+
 /**
  * Attempt to get a single comment from the various exif fields
  */
@@ -220,7 +235,7 @@ struct LatLon a_geotag_get_position ( const gchar *filename )
 			ll.lon = lon;
 		}
 	}
-	gexiv2_metadata_free  ( gemd );
+	metadata_free  ( gemd );
 #else
 #ifdef HAVE_LIBEXIF
 	// open image with libexif
@@ -276,7 +291,6 @@ VikWaypoint* a_geotag_create_waypoint_from_file ( const gchar *filename, VikCoor
 			// Now create Waypoint with acquired information
 			//
 			wp = vik_waypoint_new();
-			wp->visible = TRUE;
 			// Set info from exif values
 			// Location
 			vik_coord_load_from_latlon ( &(wp->coord), vcmode, &ll );
@@ -287,10 +301,30 @@ VikWaypoint* a_geotag_create_waypoint_from_file ( const gchar *filename, VikCoor
 				*name = g_strdup ( gexiv2_metadata_get_tag_interpreted_string ( gemd, "Exif.Image.XPTitle" ) );
 			wp->comment = geotag_get_exif_comment ( gemd );
 
+			// Direction
+			VikWaypointImageDirectionRef ref = WP_IMAGE_DIRECTION_REF_TRUE;
+			if ( gexiv2_metadata_has_tag ( gemd, EXIF_GPS_IMGDIR_REF ) ) {
+				gchar* ref_str = gexiv2_metadata_get_tag_interpreted_string(gemd, EXIF_GPS_IMGDIR_REF);
+				if ( ref_str && g_ascii_strncasecmp ("M", ref_str, 1) == 0 )
+					ref = WP_IMAGE_DIRECTION_REF_MAGNETIC;
+				g_free ( ref_str );
+			}
+			if ( gexiv2_metadata_has_tag ( gemd, EXIF_GPS_IMGDIR ) ) {
+				gint nom;
+				gint den;
+				gdouble direction = NAN;
+				if ( gexiv2_metadata_get_exif_tag_rational (gemd, EXIF_GPS_IMGDIR, &nom, &den) )
+					if ( den != 0 )
+						direction = (gdouble)nom/(gdouble)den;
+
+				if ( !isnan(direction) )
+					vik_waypoint_set_image_direction_info ( wp, direction, ref );
+			}
+
 			vik_waypoint_set_image ( wp, filename );
 		}
 	}
-	gexiv2_metadata_free ( gemd );
+	metadata_free ( gemd );
 #else
 #ifdef HAVE_LIBEXIF
 	// TODO use log?
@@ -327,7 +361,7 @@ VikWaypoint* a_geotag_create_waypoint_from_file ( const gchar *filename, VikCoor
 	// Not worried if none of the other fields exist, as can default the values to something
 	//
 
-	gdouble alt = VIK_DEFAULT_ALTITUDE;
+	gdouble alt = NAN;
 	ee = exif_content_get_entry (ed->ifd[EXIF_IFD_GPS], EXIF_TAG_GPS_ALTITUDE);
 	if ( ee && ee->components == 1 && ee->format == EXIF_FORMAT_RATIONAL ) {
 		alt = Rational2Double ( ee->data,
@@ -350,7 +384,6 @@ VikWaypoint* a_geotag_create_waypoint_from_file ( const gchar *filename, VikCoor
 	// Now create Waypoint with acquired information
 	//
 	wp = vik_waypoint_new();
-	wp->visible = TRUE;
 	// Set info from exif values
 	// Location
 	vik_coord_load_from_latlon ( &(wp->coord), vcmode, &ll );
@@ -389,7 +422,6 @@ VikWaypoint* a_geotag_waypoint_positioned ( const gchar *filename, VikCoord coor
 	if ( wp == NULL ) {
 		// Need to create waypoint
 		wp = vik_waypoint_new();
-		wp->visible = TRUE;
 	}
 	wp->coord = coord;
 	wp->altitude = alt;
@@ -401,7 +433,7 @@ VikWaypoint* a_geotag_waypoint_positioned ( const gchar *filename, VikCoord coor
 			if ( gexiv2_metadata_has_tag ( gemd, "Exif.Image.XPTitle" ) )
 				*name = g_strdup ( gexiv2_metadata_get_tag_interpreted_string ( gemd, "Exif.Image.XPTitle" ) );
 	}
-	gexiv2_metadata_free ( gemd );
+	metadata_free ( gemd );
 #else
 #ifdef HAVE_LIBEXIF
 	ExifData *ed = exif_data_new_from_file ( filename );
@@ -457,7 +489,7 @@ gchar* a_geotag_get_exif_date_from_file ( const gchar *filename, gboolean *has_G
 		else
 			datetime = g_strdup ( gexiv2_metadata_get_tag_interpreted_string ( gemd, "Exif.Image.DateTimeOriginal" ) );
 	}
-	gexiv2_metadata_free ( gemd );
+	metadata_free ( gemd );
 #else
 #ifdef HAVE_LIBEXIF
 	ExifData *ed = exif_data_new_from_file ( filename );
@@ -620,11 +652,11 @@ static void convert_to_entry (const char *set_value, gdouble gdvalue, ExifEntry
 
 		if ( use_string ) {
 			if (!value_p) {
-				g_warning (_("Too few components specified (need %d, found %d)\n"), numcomponents, i);
+				g_warning (_("Too few components specified (need %d, found %d)"), numcomponents, i);
 				return;
 			}
 			if (!isdigit(*value_p) && (*value_p != '+') && (*value_p != '-')) {
-				g_warning (_("Numeric value expected\n"));
+				g_warning (_("Numeric value expected"));
 				return;
 			}
 		}
@@ -724,37 +756,57 @@ static void convert_to_entry (const char *set_value, gdouble gdvalue, ExifEntry
  * @filename: The image file to save information in
  * @coord:    The location
  * @alt:      The elevation
+ * @direction:     The image direction (if NAN then these direction fields are not written)
+ * @direction_ref: The image direction value type
  *
  * Returns: A value indicating success: 0, or some other value for failure
  *
  */
-gint a_geotag_write_exif_gps ( const gchar *filename, VikCoord coord, gdouble alt, gboolean no_change_mtime )
+gint a_geotag_write_exif_gps ( const gchar *filename, VikCoord coord, gdouble alt,
+                               gdouble direction, VikWaypointImageDirectionRef direction_ref,
+                               gboolean no_change_mtime )
 {
 	gint result = 0; // OK so far...
 
 	// Save mtime for later use
 	struct stat stat_save;
 	if ( no_change_mtime )
-		stat ( filename, &stat_save );
+		if ( stat ( filename, &stat_save ) != 0 )
+			g_warning ( "%s couldn't read: %s", __FUNCTION__, filename );
 
 #ifdef HAVE_LIBGEXIV2
 	GExiv2Metadata *gemd = gexiv2_metadata_new ();
 	if ( gexiv2_metadata_open_path ( gemd, filename, NULL ) ) {
 		struct LatLon ll;
 		vik_coord_to_latlon ( &coord, &ll );
+		// Use update method if available
+#if GEXIV2_CHECK_VERSION(0,12,1)
+		if ( ! gexiv2_metadata_update_gps_info ( gemd, ll.lon, ll.lat, alt ) ) {
+#else
 		if ( ! gexiv2_metadata_set_gps_info ( gemd, ll.lon, ll.lat, alt ) ) {
+#endif
 			result = 1; // Failed
 		}
 		else {
-			GError *error = NULL;
-			if ( ! gexiv2_metadata_save_file ( gemd, filename, &error ) ) {
-				result = 2;
-				g_warning ( "Write EXIF failure:%s" , error->message );
-				g_error_free ( error );
+			if ( !isnan(direction) ) {
+				gint nom = (gint)round(direction * 10.0);
+				gboolean set_d = gexiv2_metadata_set_exif_tag_rational ( gemd, EXIF_GPS_IMGDIR, nom, 10 );
+				if ( !set_d ) result = 1; // Failed
+				gboolean set_r = gexiv2_metadata_set_tag_string ( gemd, EXIF_GPS_IMGDIR_REF, direction_ref == WP_IMAGE_DIRECTION_REF_TRUE ? "T" : "M" );
+				if ( !set_r ) result = 1; // Failed
+			}
+			// Still OK to save - no fails yet
+			if ( result == 0 ) {
+				GError *error = NULL;
+				if ( ! gexiv2_metadata_save_file ( gemd, filename, &error ) ) {
+					result = 2;
+					g_warning ( "Write EXIF failure:%s" , error->message );
+					g_error_free ( error );
+				}
 			}
 		}
 	}
-	gexiv2_metadata_free ( gemd );
+	metadata_free ( gemd );
 #else
 #ifdef HAVE_LIBEXIF
 	/*
@@ -843,10 +895,13 @@ gint a_geotag_write_exif_gps ( const gchar *filename, VikCoord coord, gdouble al
 		// Restore mtime, using the saved value
 		struct stat stat_tmp;
 		struct utimbuf utb;
-		stat ( filename, &stat_tmp );
+		(void)stat ( filename, &stat_tmp );
 		utb.actime = stat_tmp.st_atime;
 		utb.modtime = stat_save.st_mtime;
-		utime ( filename, &utb );
+		// Not security critical, thus potential Time of Check Time of Use race condition is not bad
+		// coverity[toctou]
+		if ( g_utime ( filename, &utb ) != 0 )
+			g_warning ( "%s couldn't set time on: %s", __FUNCTION__, filename );
 	}
 
 	return result;
