@@ -29,7 +29,6 @@ typedef struct {
   gboolean is_route;
 } TP_write_info_type;
 
-static void a_gpspoint_write_track ( const gpointer id, const VikTrack *t, FILE *f );
 static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type *write_info );
 
 /* outline for file gpspoint.c
@@ -65,27 +64,39 @@ static gchar *line_name;
 static gchar *line_comment;
 static gchar *line_description;
 static gchar *line_source;
+static guint line_number = 0;
 static gchar *line_xtype;
 static gchar *line_color;
 static gint line_name_label = 0;
 static gint line_dist_label = 0;
 static gchar *line_image;
 static gchar *line_symbol;
+static gchar *line_url;
+static gchar *line_url_name;
 static gdouble line_image_direction = NAN;
 static VikWaypointImageDirectionRef line_image_direction_ref = WP_IMAGE_DIRECTION_REF_TRUE;
 static gboolean line_newsegment = FALSE;
 static gdouble line_timestamp = NAN;
 static gdouble line_altitude = NAN;
 static gboolean line_visible = TRUE;
+static gboolean line_hide_name = FALSE;
 
 static gboolean line_extended = FALSE;
 static gdouble line_speed = NAN;
 static gdouble line_course = NAN;
-static gint line_sat = 0;
-static gint line_fix = 0;
+static gdouble line_magvar = NAN;
+static gdouble line_geoidheight = NAN;
+static guint line_sat = 0;
+static guint line_fix = 0;
 static gdouble line_hdop = NAN;
 static gdouble line_vdop = NAN;
 static gdouble line_pdop = NAN;
+static gdouble line_ageofdgpsdata = NAN;
+static guint line_dgpsid = 0;
+static guint line_hr = 0;
+static gint line_cad = VIK_TRKPT_CADENCE_NONE;
+static gdouble line_temp = NAN;
+static gint line_power = VIK_TRKPT_POWER_NONE;
 /* other possible properties go here */
 
 
@@ -235,8 +246,20 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       have_read_something = TRUE;
       VikWaypoint *wp = vik_waypoint_new();
       wp->visible = line_visible;
+      wp->hide_name = line_hide_name;
       wp->altitude = line_altitude;
       wp->timestamp = line_timestamp;
+      wp->speed = line_speed;
+      wp->course = line_course;
+      wp->magvar = line_magvar;
+      wp->geoidheight = line_geoidheight;
+      wp->nsats = line_sat;
+      wp->fix_mode = line_fix;
+      wp->hdop = line_hdop;
+      wp->vdop = line_vdop;
+      wp->pdop = line_pdop;
+      wp->ageofdgpsdata = line_ageofdgpsdata;
+      wp->dgpsid = line_dgpsid;
 
       vik_coord_load_from_latlon ( &(wp->coord), coord_mode, &line_latlon );
 
@@ -253,6 +276,12 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       if ( line_source )
         vik_waypoint_set_source ( wp, line_source );
 
+      if ( line_url )
+        vik_waypoint_set_url ( wp, line_url );
+
+      if ( line_url_name )
+        vik_waypoint_set_url_name ( wp, line_url_name );
+
       if ( line_xtype )
         vik_waypoint_set_type ( wp, line_xtype );
 
@@ -276,8 +305,6 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       trackpoints_end ();
       have_read_something = TRUE;
       VikTrack *pl = vik_track_new();
-      // NB don't set defaults here as all properties are stored in the GPS_POINT format
-      //vik_track_set_defaults ( pl );
 
       /* Thanks to Peter Jones for this Fix */
       if (!line_name) line_name = g_strdup("UNK");
@@ -294,6 +321,9 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       if ( line_source )
         vik_track_set_source ( pl, line_source );
 
+      if ( line_number )
+        pl->number = line_number;
+
       if ( line_xtype )
         vik_track_set_type ( pl, line_xtype );
 
@@ -330,6 +360,10 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
         tp->hdop = line_hdop;
         tp->vdop = line_vdop;
         tp->pdop = line_pdop;
+        tp->heart_rate = line_hr;
+        tp->cadence = line_cad;
+        tp->temp = line_temp;
+        tp->power = line_power;
       }
       // Much faster to prepend and then reverse list once all points read in
       // Especially if hunderds of thousands or more trackpoints in a file
@@ -353,6 +387,10 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       g_free ( line_image );
     if (line_symbol)
       g_free ( line_symbol );
+    if (line_url)
+      g_free ( line_url );
+    if (line_url_name)
+      g_free ( line_url_name );
     line_comment = NULL;
     line_description = NULL;
     line_source = NULL;
@@ -367,7 +405,11 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
     line_timestamp = NAN;
     line_altitude = NAN;
     line_visible = TRUE;
-    line_symbol = NULL;
+    line_hide_name = FALSE;
+    line_url = NULL;
+    line_url_name = NULL;
+    line_magvar = NAN;
+    line_geoidheight = NAN;
 
     line_extended = FALSE;
     line_speed = NAN;
@@ -377,8 +419,15 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
     line_hdop = NAN;
     line_vdop = NAN;
     line_pdop = NAN;
+    line_ageofdgpsdata = NAN;
+    line_dgpsid = 0;
     line_name_label = 0;
     line_dist_label = 0;
+    line_number = 0;
+    line_hr = 0;
+    line_cad = VIK_TRKPT_CADENCE_NONE;
+    line_temp = NAN;
+    line_power = VIK_TRKPT_POWER_NONE;
   }
 
   // Handle a badly formatted file in case of missing explicit track/route end (this shouldn't happen)
@@ -444,7 +493,24 @@ value = NULL for none
 */
 static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, const gchar *value, guint value_len )
 {
-  if (key_len == 4 && strncasecmp( key, "type", key_len ) == 0 )
+  // Most commonlly encountered keys should be tested first
+  if (key_len == 8 && strncasecmp( key, "latitude", key_len ) == 0 && value != NULL)
+  {
+    line_latlon.lat = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 9 && strncasecmp( key, "longitude", key_len ) == 0 && value != NULL)
+  {
+    line_latlon.lon = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 8 && strncasecmp( key, "unixtime", key_len ) == 0 && value != NULL)
+  {
+    line_timestamp = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 8 && strncasecmp( key, "altitude", key_len ) == 0 && value != NULL)
+  {
+    line_altitude = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 4 && strncasecmp( key, "type", key_len ) == 0 )
   {
     if (value == NULL)
       line_type = GPSPOINT_TYPE_NONE;
@@ -488,6 +554,10 @@ static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, co
     if (line_source == NULL)
       line_source = deslashndup ( value, value_len );
   }
+  else if (key_len == 6 && strncasecmp( key, "number", key_len ) == 0 && value != NULL)
+  {
+    line_number = atoi(value);
+  }
   // NB using 'xtype' to differentiate from our own 'type' key
   else if (key_len == 5 && strncasecmp( key, "xtype", key_len ) == 0 && value != NULL)
   {
@@ -520,18 +590,6 @@ static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, co
   {
     line_image_direction_ref = atoi(value);
   }
-  else if (key_len == 8 && strncasecmp( key, "latitude", key_len ) == 0 && value != NULL)
-  {
-    line_latlon.lat = g_ascii_strtod(value, NULL);
-  }
-  else if (key_len == 9 && strncasecmp( key, "longitude", key_len ) == 0 && value != NULL)
-  {
-    line_latlon.lon = g_ascii_strtod(value, NULL);
-  }
-  else if (key_len == 8 && strncasecmp( key, "altitude", key_len ) == 0 && value != NULL)
-  {
-    line_altitude = g_ascii_strtod(value, NULL);
-  }
   else if (key_len == 7 && strncasecmp( key, "visible", key_len ) == 0 && value != NULL && value[0] != 'y' && value[0] != 'Y' && value[0] != 't' && value[0] != 'T')
   {
     line_visible = FALSE;
@@ -540,10 +598,6 @@ static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, co
   {
     line_symbol = g_strndup ( value, value_len );
   }
-  else if (key_len == 8 && strncasecmp( key, "unixtime", key_len ) == 0 && value != NULL)
-  {
-    line_timestamp = g_ascii_strtod(value, NULL);
-  }
   else if (key_len == 10 && strncasecmp( key, "newsegment", key_len ) == 0 && value != NULL)
   {
     line_newsegment = TRUE;
@@ -562,11 +616,11 @@ static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, co
   }
   else if (key_len == 3 && strncasecmp( key, "sat", key_len ) == 0 && value != NULL)
   {
-    line_sat = atoi(value);
+    line_sat = (guint)atoi(value);
   }
   else if (key_len == 3 && strncasecmp( key, "fix", key_len ) == 0 && value != NULL)
   {
-    line_fix = atoi(value);
+    line_fix = (guint)atoi(value);
   }
   else if (key_len == 4 && strncasecmp( key, "hdop", key_len ) == 0 && value != NULL)
   {
@@ -580,6 +634,50 @@ static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, co
   {
     line_pdop = g_ascii_strtod(value, NULL);
   }
+  else if (key_len == 2 && strncasecmp( key, "hr", key_len ) == 0 && value != NULL)
+  {
+    line_hr = (guint)atoi(value);
+  }
+  else if (key_len == 3 && strncasecmp( key, "cad", key_len ) == 0 && value != NULL)
+  {
+    line_cad = (guint)atoi(value);
+  }
+  else if (key_len == 4 && strncasecmp( key, "temp", key_len ) == 0 && value != NULL)
+  {
+    line_temp = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 3 && strncasecmp( key, "pow", key_len ) == 0 && value != NULL)
+  {
+    line_power = (guint)atoi(value);
+  }
+  else if (key_len == 6 && strncasecmp( key, "magvar", key_len ) == 0 && value != NULL)
+  {
+    line_magvar = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 11 && strncasecmp( key, "geoidheight", key_len ) == 0 && value != NULL)
+  {
+    line_geoidheight = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 3 && strncasecmp( key, "url", key_len ) == 0 && value != NULL)
+  {
+    line_url = deslashndup ( value, value_len );
+  }
+  else if (key_len == 8 && strncasecmp( key, "url_name", key_len ) == 0 && value != NULL)
+  {
+    line_url_name = deslashndup ( value, value_len );
+  }
+  else if (key_len == 13 && strncasecmp( key, "ageofdgpsdata", key_len ) == 0 && value != NULL)
+  {
+    line_ageofdgpsdata = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 6 && strncasecmp( key, "dgpsid", key_len ) == 0 && value != NULL)
+  {
+    line_dgpsid = (guint)atoi(value);
+  }
+  else if (key_len == 9 && strncasecmp( key, "hide_name", key_len ) == 0 && value != NULL && (value[0] == 'y' || value[0] == 'Y' || value[0] == 't' || value[0] == 'T'))
+  {
+    line_hide_name = TRUE;
+  }
 }
 
 typedef struct {
@@ -587,7 +685,31 @@ typedef struct {
   const gchar *dirpath;
 } WritingContext;
 
-static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp, WritingContext *wc )
+static void write_double ( FILE *ff, const gchar *tag, gdouble value )
+{
+  if ( !isnan(value) ) {
+    gchar buf[COORDS_STR_BUFFER_SIZE];
+    a_coords_dtostr_buffer ( value, buf );
+    fprintf ( ff, " %s=\"%s\"", tag, buf );
+  }
+}
+
+static void write_positive_uint ( FILE *ff, const gchar *tag, guint value )
+{
+  if ( value )
+    fprintf ( ff, " %s=\"%d\"", tag, value );
+}
+
+static void write_string ( FILE *ff, const gchar *tag, const gchar *value )
+{
+  if ( value && strlen(value) ) {
+    gchar *tmp = slashdup(value);
+    fprintf ( ff, " %s=\"%s\"", tag, tmp );
+    g_free ( tmp );
+  }
+}
+
+static void a_gpspoint_write_waypoint ( const VikWaypoint *wp, WritingContext *wc )
 {
   struct LatLon ll;
   gchar s_lat[COORDS_STR_BUFFER_SIZE];
@@ -609,40 +731,27 @@ static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp
   fprintf ( f, "type=\"waypoint\" latitude=\"%s\" longitude=\"%s\" name=\"%s\"", s_lat, s_lon, tmp_name );
   g_free ( tmp_name );
 
-  if ( !isnan(wp->altitude) ) {
-    gchar s_alt[COORDS_STR_BUFFER_SIZE];
-    a_coords_dtostr_buffer ( wp->altitude, s_alt );
-    fprintf ( f, " altitude=\"%s\"", s_alt );
-  }
-  if ( !isnan(wp->timestamp) ) {
-    gchar s_tm[COORDS_STR_BUFFER_SIZE];
-    a_coords_dtostr_buffer ( wp->timestamp, s_tm );
-    fprintf ( f, " unixtime=\"%s\"", s_tm );
-  }
-  if ( wp->comment )
-  {
-    gchar *tmp_comment = slashdup(wp->comment);
-    fprintf ( f, " comment=\"%s\"", tmp_comment );
-    g_free ( tmp_comment );
-  }
-  if ( wp->description )
-  {
-    gchar *tmp_description = slashdup(wp->description);
-    fprintf ( f, " description=\"%s\"", tmp_description );
-    g_free ( tmp_description );
-  }
-  if ( wp->source )
-  {
-    gchar *tmp_source = slashdup(wp->source);
-    fprintf ( f, " source=\"%s\"", tmp_source );
-    g_free ( tmp_source );
-  }
-  if ( wp->type )
-  {
-    gchar *tmp_type = slashdup(wp->type);
-    fprintf ( f, " xtype=\"%s\"", tmp_type );
-    g_free ( tmp_type );
-  }
+  write_double ( f, "altitude", wp->altitude );
+  write_double ( f, "unixtime", wp->timestamp );
+  write_double ( f, "speed", wp->speed );
+  write_double ( f, "course", wp->course );
+  write_double ( f, "magvar", wp->magvar );
+  write_double ( f, "geoidheight", wp->geoidheight );
+  write_string ( f, "comment", wp->comment );
+  write_string ( f, "description", wp->description );
+  write_string ( f, "source", wp->source );
+  write_string ( f, "url", wp->url );
+  write_string ( f, "url_name", wp->url_name );
+  write_string ( f, "xtype", wp->type );
+
+  write_positive_uint ( f, "fix", wp->fix_mode );
+  write_positive_uint ( f, "sat", wp->nsats );
+  write_double ( f, "hdop", wp->hdop );
+  write_double ( f, "vdop", wp->vdop );
+  write_double ( f, "pdop", wp->pdop );
+  write_double ( f, "ageofdgpsdata", wp->ageofdgpsdata );
+  write_positive_uint ( f, "dgpsid", wp->dgpsid );
+
   if ( wp->image )
   {
     gchar *tmp_image = NULL;
@@ -679,6 +788,8 @@ static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp
   }
   if ( ! wp->visible )
     fprintf ( f, " visible=\"n\"" );
+  if ( wp->hide_name )
+    fprintf ( f, " hide_name=\"yes\"" );
   fprintf ( f, "\n" );
 }
 
@@ -687,7 +798,6 @@ static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type
   struct LatLon ll;
   gchar s_lat[COORDS_STR_BUFFER_SIZE];
   gchar s_lon[COORDS_STR_BUFFER_SIZE];
-  gchar s_alt[COORDS_STR_BUFFER_SIZE];
   vik_coord_to_latlon ( &(tp->coord), &ll );
 
   FILE *f = write_info->f;
@@ -696,62 +806,37 @@ static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type
   a_coords_dtostr_buffer ( ll.lon, s_lon );
   fprintf ( f, "type=\"%spoint\" latitude=\"%s\" longitude=\"%s\"", write_info->is_route ? "route" : "track", s_lat, s_lon );
 
-  if ( tp->name ) {
-    gchar *name = slashdup(tp->name);
-    fprintf ( f, " name=\"%s\"", name );
-    g_free(name);
-  }
+  write_string ( f, "name", tp->name );
+  write_double ( f, "altitude", tp->altitude );
+  write_double ( f, "unixtime", tp->timestamp );
 
-  if ( !isnan(tp->altitude) ) {
-    a_coords_dtostr_buffer ( tp->altitude, s_alt );
-    fprintf ( f, " altitude=\"%s\"", s_alt );
-  }
-  if ( !isnan(tp->timestamp) ) {
-    gchar s_tm[COORDS_STR_BUFFER_SIZE];
-    a_coords_dtostr_buffer ( tp->timestamp, s_tm );
-    fprintf ( f, " unixtime=\"%s\"", s_tm );
-  }
   if ( tp->newsegment )
     fprintf ( f, " newsegment=\"yes\"" );
 
-  if (!isnan(tp->speed) || !isnan(tp->course) || tp->nsats > 0) {
+  if ( !isnan(tp->speed) || !isnan(tp->course) || tp->nsats > 0 ||
+       !isnan(tp->temp) || tp->heart_rate || tp->cadence != VIK_TRKPT_CADENCE_NONE || tp->cadence != VIK_TRKPT_POWER_NONE ) {
     fprintf ( f, " extended=\"yes\"" );
-    if (!isnan(tp->speed)) {
-      gchar s_speed[COORDS_STR_BUFFER_SIZE];
-      a_coords_dtostr_buffer ( tp->speed, s_speed );
-      fprintf ( f, " speed=\"%s\"", s_speed );
-    }
-    if (!isnan(tp->course)) {
-      gchar s_course[COORDS_STR_BUFFER_SIZE];
-      a_coords_dtostr_buffer ( tp->course, s_course );
-      fprintf ( f, " course=\"%s\"", s_course );
-    }
-    if (tp->nsats > 0)
-      fprintf ( f, " sat=\"%d\"", tp->nsats );
-    if (tp->fix_mode > 0)
-      fprintf ( f, " fix=\"%d\"", tp->fix_mode );
-
-    if ( !isnan(tp->hdop) ) {
-      gchar ss[COORDS_STR_BUFFER_SIZE];
-      a_coords_dtostr_buffer ( tp->hdop, ss );
-      fprintf ( f, " hdop=\"%s\"", ss );
-    }
-    if ( !isnan(tp->vdop) ) {
-      gchar ss[COORDS_STR_BUFFER_SIZE];
-      a_coords_dtostr_buffer ( tp->vdop, ss );
-      fprintf ( f, " vdop=\"%s\"", ss );
-    }
-    if ( !isnan(tp->pdop) ) {
-      gchar ss[COORDS_STR_BUFFER_SIZE];
-      a_coords_dtostr_buffer ( tp->pdop, ss );
-      fprintf ( f, " pdop=\"%s\"", ss );
-    }
+    write_double ( f, "speed", tp->speed );
+    write_double ( f, "course", tp->course );
+    write_positive_uint ( f, "sat", tp->nsats );
+    write_positive_uint ( f, "fix", tp->fix_mode );
+    write_double ( f, "hdop", tp->hdop );
+    write_double ( f, "vdop", tp->vdop );
+    write_double ( f, "pdop", tp->pdop );
+    // Note that in general these use shortened strings as there may be many thousands of points,
+    //  thus helping keep the filesize down.
+    write_positive_uint ( f, "hr", tp->heart_rate );
+    if ( tp->cadence != VIK_TRKPT_CADENCE_NONE )
+      fprintf ( f, " cad=\"%d\"", tp->cadence );
+    write_double ( f, "temp", tp->temp );
+    if ( tp->power != VIK_TRKPT_POWER_NONE )
+      fprintf ( f, " pow=\"%d\"", tp->power );
   }
   fprintf ( f, "\n" );
 }
 
 
-static void a_gpspoint_write_track ( const gpointer id, const VikTrack *trk, FILE *f )
+static void a_gpspoint_write_track ( const VikTrack *trk, FILE *f )
 {
   // Sanity clauses
   if ( !trk )
@@ -763,39 +848,18 @@ static void a_gpspoint_write_track ( const gpointer id, const VikTrack *trk, FIL
   fprintf ( f, "type=\"%s\" name=\"%s\"", trk->is_route ? "route" : "track", tmp_name );
   g_free ( tmp_name );
 
-  if ( trk->comment ) {
-    gchar *tmp = slashdup(trk->comment);
-    fprintf ( f, " comment=\"%s\"", tmp );
-    g_free ( tmp );
-  }
-
-  if ( trk->description ) {
-    gchar *tmp = slashdup(trk->description);
-    fprintf ( f, " description=\"%s\"", tmp );
-    g_free ( tmp );
-  }
-
-  if ( trk->source ) {
-    gchar *tmp = slashdup(trk->source);
-    fprintf ( f, " source=\"%s\"", tmp );
-    g_free ( tmp );
-  }
-
-  if ( trk->type ) {
-    gchar *tmp = slashdup(trk->type);
-    fprintf ( f, " xtype=\"%s\"", tmp );
-    g_free ( tmp );
-  }
+  write_string ( f, "comment", trk->comment );
+  write_string ( f, "description", trk->description );
+  write_string ( f, "source", trk->source );
+  write_positive_uint ( f, "number", trk->number );
+  write_string ( f, "xtype", trk->type );
 
   if ( trk->has_color ) {
     fprintf ( f, " color=#%.2x%.2x%.2x", (int)(trk->color.red/256),(int)(trk->color.green/256),(int)(trk->color.blue/256));
   }
 
-  if ( trk->draw_name_mode > 0 )
-    fprintf ( f, " draw_name_mode=\"%d\"", trk->draw_name_mode );
-
-  if ( trk->max_number_dist_labels > 0 )
-    fprintf ( f, " number_dist_labels=\"%d\"", trk->max_number_dist_labels );
+  write_positive_uint ( f, "draw_name_mode", trk->draw_name_mode );
+  write_positive_uint ( f, "number_dist_labels", trk->max_number_dist_labels );
 
   if ( ! trk->visible ) {
     fprintf ( f, " visible=\"n\"" );
@@ -807,16 +871,34 @@ static void a_gpspoint_write_track ( const gpointer id, const VikTrack *trk, FIL
   fprintf ( f, "type=\"%send\"\n", trk->is_route ? "route" : "track" );
 }
 
+/**
+ * Enforce writing waypoints/tracks/routes in the order they have been read in
+ * This should enable comparing changes between file saves much better,
+ *  as limits it to the actual changes
+ *  (rather than reorderings due to the internal usage of hashtables)
+ */
 void a_gpspoint_write_file ( VikTrwLayer *trw, FILE *f, const gchar *dirpath )
 {
   GHashTable *tracks = vik_trw_layer_get_tracks ( trw );
   GHashTable *routes = vik_trw_layer_get_routes ( trw );
   GHashTable *waypoints = vik_trw_layer_get_waypoints ( trw );
-
+  GList *gl = NULL;
   WritingContext wc = { f, dirpath };
+
   fprintf ( f, "type=\"waypointlist\"\n" );
-  g_hash_table_foreach ( waypoints, (GHFunc) a_gpspoint_write_waypoint, &wc );
+  gl = vu_sorted_list_from_hash_table ( waypoints, VL_SO_NONE, VIKING_WAYPOINT );
+  for ( GList *it = g_list_first(gl); it != NULL; it = g_list_next(it) )
+    a_gpspoint_write_waypoint ( (VikWaypoint*)((SortTRWHashT*)it->data)->data, &wc );
+  g_list_free_full ( gl, g_free );
   fprintf ( f, "type=\"waypointlistend\"\n" );
-  g_hash_table_foreach ( tracks, (GHFunc) a_gpspoint_write_track, f );
-  g_hash_table_foreach ( routes, (GHFunc) a_gpspoint_write_track, f );
+
+  gl = vu_sorted_list_from_hash_table ( tracks, VL_SO_NONE, VIKING_TRACK );
+  for ( GList *it = g_list_first(gl); it != NULL; it = g_list_next(it) )
+    a_gpspoint_write_track ( (VikTrack*)((SortTRWHashT*)it->data)->data, f );
+  g_list_free_full ( gl, g_free );
+
+  gl = vu_sorted_list_from_hash_table ( routes, VL_SO_NONE, VIKING_TRACK );
+  for ( GList *it = g_list_first(gl); it != NULL; it = g_list_next(it) )
+    a_gpspoint_write_track ( (VikTrack*)((SortTRWHashT*)it->data)->data, f );
+  g_list_free_full ( gl, g_free );
 }
