@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
+#include <gmodule.h>
 
 #ifdef HAVE_CONFIG
 #include "config.h"
@@ -30,9 +31,11 @@
 #include "dems.h"
 #include "babel.h"
 #include "curl_download.h"
-#include "preferences.h"
+#include "logging.h"
+#include "vikdemlayer.h"
 #include "vikmapslayer.h"
 #include "vikgeoreflayer.h"
+#include "viktrwlayer_propwin.h"
 #include "vikrouting.h"
 #include "toolbar.h"
 #include "thumbnails.h"
@@ -47,47 +50,9 @@
 #define LOCALEDIR "locale"
 #endif
 
-#ifdef HAVE_X11_XLIB_H
-#include "X11/Xlib.h"
-#endif
-
-#if GLIB_CHECK_VERSION (2, 32, 0)
-/* Callback to log message */
-static void log_debug(const gchar *log_domain,
-                      GLogLevelFlags log_level,
-                      const gchar *message,
-                      gpointer user_data)
-{
-  g_print("** (viking): DEBUG: %s\n", message);
-}
-#else
-/* Callback to mute log message */
-static void mute_log(const gchar *log_domain,
-                     GLogLevelFlags log_level,
-                     const gchar *message,
-                     gpointer user_data)
-{
-  /* Nothing to do, we just want to mute */
-}
-#endif
-
-#if HAVE_X11_XLIB_H
-static int myXErrorHandler(Display *display, XErrorEvent *theEvent)
-{
-  (void)g_fprintf (stderr,
-                   _("Ignoring Xlib error: error code %d request code %d\n"),
-                   theEvent->error_code,
-                   theEvent->request_code);
-  // No exit on X errors!
-  //  mainly to handle out of memory error when requesting large pixbuf from user request
-  //  see vikwindow.c::save_image_file ()
-  return 0;
-}
-#endif
-
 // Default values that won't actually get applied unless changed by command line parameter values
-static gdouble latitude = 0.0;
-static gdouble longitude = 0.0;
+static gdouble latitude = NAN;
+static gdouble longitude = NAN;
 static gint zoom_level_osm = -1;
 static gint map_id = -1;
 static gboolean external = FALSE;
@@ -106,10 +71,53 @@ static GOptionEntry entries[] =
   { NULL }
 };
 
+/**
+ * Basic 'geo:' URI RFC5870
+ * https://datatracker.ietf.org/doc/html/rfc5870
+ * https://en.wikipedia.org/wiki/Geo_URI_scheme
+ * ATM ignore any CRS parameters - assume always on earth in WGS-84
+ * Support optional zoom level parameter ('?z=n')
+ */
+gboolean check_for_geo_uri ( char *argv )
+{
+  GError *error = NULL;
+  GRegex *regex = g_regex_new ( "^geo:[\\-\\.0-9]*,[\\.\\-0-9]*", G_REGEX_CASELESS, 0, &error );
+  if ( error ) {
+    g_critical ("%s: %s", __FUNCTION__, error->message );
+    return FALSE;
+  }
+  if ( g_regex_match(regex, argv, 0, NULL) ) {
+    g_debug ( "%s: Geo URI detected in %s", __FUNCTION__, argv );
+    // Drop the 'geo:' bit
+    gchar *geostr = g_strdup ( argv+4 );
+    // Modify latitude, longitude & potentially zoom value
+    gchar **tokens = g_strsplit ( geostr, ",", 2 );
+    if ( tokens[0] && tokens[1] ) {
+      latitude = g_ascii_strtod ( tokens[0], NULL );
+      longitude = g_ascii_strtod ( tokens[1], NULL );
+      // May be a better way of extracting the z value, but this should suffice
+      gchar **exts = g_strsplit ( tokens[1], "?", 0 );
+      if ( exts[0] && exts[1] ) {
+        g_debug ( "%s: extensions %s", __FUNCTION__, exts[1] );
+        gchar **zs = g_strsplit ( exts[1], "z=", 0 );
+        if ( zs[0] && zs[1] ) {
+          g_debug ( "%s: z %s", __FUNCTION__, zs[1] );
+          zoom_level_osm = atoi ( zs[1] );
+        }
+      }
+    } else
+      g_warning ( "%s: no coordinates found in %s", __FUNCTION__, geostr );
+    g_free ( geostr );
+    g_strfreev ( tokens );
+    return TRUE;
+  }
+  g_regex_unref ( regex );
+  return FALSE;
+}
+
 int main( int argc, char *argv[] )
 {
   VikWindow *first_window;
-  GdkPixbuf *main_icon;
   gboolean dashdash_already = FALSE;
   int i = 0;
   GError *error = NULL;
@@ -119,10 +127,6 @@ int main( int argc, char *argv[] )
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-#if ! GLIB_CHECK_VERSION (2, 32, 0)
-  g_thread_init ( NULL );
-#endif
-
   gui_initialized = gtk_init_with_args (&argc, &argv, "files+", entries, NULL, &error);
   if (!gui_initialized)
   {
@@ -149,24 +153,25 @@ int main( int argc, char *argv[] )
     return EXIT_SUCCESS;
   }
 
-#if GLIB_CHECK_VERSION (2, 32, 0)
-  if (vik_debug)
-    g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, log_debug, NULL);
-#else
-  if (!vik_debug)
-    g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, mute_log, NULL);
-#endif
-
-#if HAVE_X11_XLIB_H
-  XSetErrorHandler(myXErrorHandler);
+  // Normally on Windows the command line arguments are passed in in the system codepage encoding.
+  // This converts it to the GLib filename encoding (which is UTF-8 on Windows).
+  // Doing this here also helps when the filename is used for the displayed main window title
+  //  and saving into the recently used documents list
+#ifdef G_OS_WIN32
+  argv = g_win32_get_command_line ();
 #endif
 
   // Ensure correct capitalization of the program name
   g_set_application_name ("Viking");
 
+  a_logging_init ();
+
   // Discover if this is the very first run
   a_vik_very_first_run ();
 
+  vik_icons_register_resource ();
+  ui_load_icons();
+
   a_settings_init ();
   a_preferences_init ();
   a_thumbnails_init ();
@@ -194,12 +199,17 @@ int main( int argc, char *argv[] )
 
   vik_georef_layer_init ();
   maps_layer_init ();
+  vik_dem_layer_init ();
   a_mapcache_init ();
   a_background_init ();
 
   a_toolbar_init();
   vik_routing_prefs_init();
   vik_trw_layer_export_init();
+  vik_trw_layer_propwin_init();
+
+  // Registration of preferences has now been done
+  a_preferences_finished_registering();
 
   /*
    * Second stage initialization
@@ -215,8 +225,9 @@ int main( int argc, char *argv[] )
     vu_setup_lat_lon_tz_lookup();
 
   /* Set the icon */
-  main_icon = gdk_pixbuf_from_pixdata(&viking_pixbuf, FALSE, NULL);
-  gtk_window_set_default_icon(main_icon);
+  GdkPixbuf *main_icon = ui_get_icon ( "viking", 48 );
+  if ( main_icon )
+    gtk_window_set_default_icon(main_icon);
 
   // Ask for confirmation of default settings on first run
   vu_set_auto_features_on_first_run ();
@@ -224,6 +235,8 @@ int main( int argc, char *argv[] )
   /* Create the first window */
   first_window = vik_window_new_window();
 
+  a_logging_update();
+
   vu_check_latest_version ( GTK_WINDOW(first_window) );
 
   // Load startup file first so that subsequent files are loaded on top
@@ -254,7 +267,10 @@ int main( int argc, char *argv[] )
         change_filename = TRUE;
       }
 
-      vik_window_open_file ( newvw, argv[i], change_filename, (i==1), (i+1 == argc), TRUE, external );
+      // Check if the file parameter is a 'geo:' URI
+      //  if so then then don't try to load this parameter as a file
+      if ( !check_for_geo_uri(argv[i]) )
+        vik_window_open_file ( newvw, argv[i], change_filename, (i==1), (i+1 == argc), TRUE, external );
     }
   }
 
@@ -264,9 +280,12 @@ int main( int argc, char *argv[] )
 
   gtk_main ();
 
+  vik_trwlayer_uninit ();
+  vik_aggregate_layer_uninit ();
   a_babel_uninit ();
   a_toolbar_uninit ();
   a_background_uninit ();
+  maps_layer_uninit ();
   a_mapcache_uninit ();
   a_dems_uninit ();
   a_layer_defaults_uninit ();
@@ -280,8 +299,14 @@ int main( int argc, char *argv[] )
 
   vu_finalize_lat_lon_tz_lookup ();
 
+  vik_icons_unregister_resource ();
+  
   // Clean up any temporary files
   util_remove_all_in_deletion_list ();
 
+#ifdef G_OS_WIN32
+  g_strfreev ( argv );
+#endif
+
   return 0;
 }
