@@ -28,7 +28,6 @@
 #include "osm-traces.h"
 #include "gpx.h"
 #include "background.h"
-#include "preferences.h"
 #include "curl_download.h"
 #ifdef HAVE_OAUTH_H
 #include <oauth.h>
@@ -154,7 +153,8 @@ gchar *osm_get_login()
 // https://wiki.openstreetmap.org/wiki/OAuth
 //
 static const gchar *viking_consumer_key = "bAUxFBhGSzwXo9R43gJ1JXqx8cVphItXLo0PsRV3";
-static const gchar *viking_consumer_secret = "DKWT1ydfZUF9VpiUVkMn3p7faIaUB2cBawh86a0z"; // Not exactly a secret as publically viewable right here!
+#define VSC_INIT "QXJG6lqsMHS4IcvHIxZa8c2snVnHO7pOnju31n5m"
+static gchar *viking_consumer_secret = NULL;
 
 /**
  * split and parse URL parameters replied by the server
@@ -278,7 +278,11 @@ static void new_access_token_cb ( )
   gchar *authorize_url = get_authorize_url ( request_key );
   g_debug ( "%s:%s", __FUNCTION__, authorize_url );
   if ( authorize_url ) {
-    gtk_show_uri ( gdk_screen_get_default(), authorize_url, GDK_CURRENT_TIME, NULL );
+#if GTK_CHECK_VERSION (3,22,0)
+    (void)gtk_show_uri_on_window ( parent, authorize_url, GDK_CURRENT_TIME, NULL );
+#else
+    (void)gtk_show_uri ( gdk_screen_get_default(), authorize_url, GDK_CURRENT_TIME, NULL );
+#endif
   }
   g_free ( authorize_url );
 
@@ -312,26 +316,17 @@ static void new_access_token_cb ( )
     return;
   }
 
-  // Unfortunately since the callback mechanism in preferences only allows a function call with no parameters
-  //  we have no way of accessing the dialog to effect any updates, other than closing it
-  a_uibuilder_factory_close ( GTK_RESPONSE_REJECT );
-  // Note since we are in an event handler already, using GTK_RESPONSE_ACCEPT means this signal
-  //  is processed after this current function finishes.
-  // This then in turn means it would save the values from the dialog,
-  //  overwriting the values set here :(
-  // The current side effect is that any other preferences the user has modified in the dialog are lost
-
   // Now apply the new values
   vlp_data.s = access_token_key;
   a_preferences_run_setparam ( vlp_data, pref_key );
   vlp_data.s = access_token_secret;
   a_preferences_run_setparam ( vlp_data, pref_secret );
 
-  a_preferences_save_to_file ();
-
   g_free ( access_token_key );
   g_free ( access_token_secret );
 
+  a_preferences_refresh ( VIKING_OSM_TRACES_PARAMS_GROUP_KEY );
+
   // On success mention can remove username/password if they already exist
   VikLayerParamData *pref_user = a_preferences_get ( OSM_USERNAME );
   VikLayerParamData *pref_pwd = a_preferences_get ( OSM_PASSWORD );
@@ -370,18 +365,19 @@ void osm_traces_init () {
 #ifdef HAVE_OAUTH_H
   tmp.ptr = new_access_token_cb;
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-  tmp.s = "";
+  tmp.s = NULL;
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-  tmp.s = "";
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
+  // Not that secret; but as least not in total plain sight anymore
+  viking_consumer_secret = g_strdup ( VSC_INIT );
+  viking_consumer_secret = util_frob ( viking_consumer_secret, strlen(viking_consumer_secret) );
 #endif
   // Only register depreciated preferences if they already exist
   //  or being forced to use the basic method
   if ( osm_use_basic_auth() ||
        (a_preferences_lookup("OSM_USERNAME") && a_preferences_lookup(OSM_PASSWORD)) ) {
-    tmp.s = "";
+    tmp.s = NULL;
     a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-    tmp.s = "";
     a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
   }
 
@@ -391,6 +387,9 @@ void osm_traces_init () {
 void osm_traces_uninit()
 {
   vik_mutex_free(login_mutex);
+#ifdef HAVE_OAUTH_H
+  g_free ( viking_consumer_secret );
+#endif
 }
 
 #define OSM_GPX_UPLOAD_URL "https://www.openstreetmap.org/api/0.6/gpx/create"
@@ -503,7 +502,6 @@ static gint osm_traces_upload_file(const char *user,
   }
 
   /* Memory */
-  g_free(base_url);
   g_free(user_pass); user_pass = NULL;
   g_free(base_url);
 
@@ -513,17 +511,17 @@ static gint osm_traces_upload_file(const char *user,
 }
 
 /**
- * uploading function executed by the background" thread
+ * uploading function executed by the background thread
  */
 static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
 {
-  /* Due to OSM limits, we have to enforce ele and time fields
-   also don't upload invisible tracks */
-  static GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE };
-
   if (!oti)
     return;
 
+  /* Due to OSM limits, we have to enforce ele and time fields
+   also don't upload invisible tracks */
+  GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE, vik_trw_layer_get_gpx_version(oti->vtl) };
+
   gchar *filename = NULL;
 
   /* writing gpx file */
@@ -534,11 +532,11 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
     {
       VikTrack *trk = vik_track_copy(oti->trk, TRUE);
       vik_track_anonymize_times(trk);
-      filename = a_gpx_write_track_tmp_file(trk, &options);
+      filename = a_gpx_write_track_tmp_file (oti->vtl, trk, &options);
       vik_track_free(trk);
     }
     else
-      filename = a_gpx_write_track_tmp_file (oti->trk, &options);
+      filename = a_gpx_write_track_tmp_file (oti->vtl, oti->trk, &options);
   }
   else
   {
@@ -566,7 +564,7 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
   timeinfo = localtime ( &timenow );
   gchar timestr[80];
   // Compact time only - as days/date isn't very useful here
-  strftime ( timestr, sizeof(timestr), "%X)", timeinfo );
+  strftime ( timestr, sizeof(timestr), "%X", timeinfo );
 
   //
   // Test to see if window it was invoked on is still valid
@@ -596,6 +594,7 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
   if (ret != 0) {
     g_critical(_("failed to unlink temporary file: %s"), strerror(errno));
   }
+  g_free ( filename );
 }
 
 /**
