@@ -33,9 +33,11 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
+#include "ui_util.h"
 #include "util.h"
 #include "dialog.h"
 #include "settings.h"
+#include "icons/icons.h"
 
 #ifdef WINDOWS
 #include <windows.h>
@@ -63,7 +65,8 @@ static gboolean spawn_command_line_async(const gchar * cmd,
 
 // Annoyingly gtk_show_uri() doesn't work so resort to ShellExecute method
 //   (non working at least in our Windows build with GTK+2.24.10 on Windows 7)
-
+// @parent: Maybe NULL
+//
 void open_url(GtkWindow *parent, const gchar * url)
 {
 #ifdef WINDOWS
@@ -83,7 +86,11 @@ void open_url(GtkWindow *parent, const gchar * url)
   }
   else {
     GError *error = NULL;
-    gtk_show_uri ( gtk_widget_get_screen (GTK_WIDGET(parent)), url, GDK_CURRENT_TIME, &error );
+#if GTK_CHECK_VERSION (3,22,0)
+	(void)gtk_show_uri_on_window ( parent, url, GDK_CURRENT_TIME, &error );
+#else
+    (void)gtk_show_uri ( parent ? gtk_widget_get_screen(GTK_WIDGET(parent)) : NULL, url, GDK_CURRENT_TIME, &error );
+#endif
     if ( error ) {
       a_dialog_error_msg_extra ( parent, _("Could not launch web browser. %s"), error->message );
       g_error_free ( error );
@@ -96,7 +103,11 @@ void new_email(GtkWindow *parent, const gchar * address)
 {
   gchar *uri = g_strdup_printf("mailto:%s", address);
   GError *error = NULL;
-  gtk_show_uri ( gtk_widget_get_screen (GTK_WIDGET(parent)), uri, GDK_CURRENT_TIME, &error );
+#if GTK_CHECK_VERSION (3,22,0)
+  (void)gtk_show_uri_on_window ( parent, uri, GDK_CURRENT_TIME, &error );
+#else
+  (void)gtk_show_uri ( gtk_widget_get_screen (GTK_WIDGET(parent)), uri, GDK_CURRENT_TIME, &error );
+#endif
   if ( error ) {
     a_dialog_error_msg_extra ( parent, _("Could not create new email. %s"), error->message );
     g_error_free ( error );
@@ -199,6 +210,24 @@ GtkWidget* ui_label_new_selectable ( const gchar* text )
 	return widget;
 }
 
+/**
+ * Create a new pixbuf of the specified color and size
+ */
+GdkPixbuf *ui_pixbuf_new ( GdkColor *color, guint width, guint height )
+{
+	GdkPixbuf *pixbuf = gdk_pixbuf_new ( GDK_COLORSPACE_RGB, FALSE, 8, width, height );
+	// Annoyingly the GdkColor.pixel does not give the correct color when passed to gdk_pixbuf_fill (even when alloc'ed)
+	// Here is some magic found to do the conversion
+	// http://www.cs.binghamton.edu/~sgreene/cs360-2011s/topics/gtk+-2.20.1/gtk/gtkcolorbutton.c
+	if ( color ) {
+		guint32 pixel = ((color->red & 0xff00) << 16) |
+			((color->green & 0xff00) << 8) |
+			(color->blue & 0xff00);
+		gdk_pixbuf_fill ( pixbuf, pixel );
+	}
+	return pixbuf;
+}
+
 /**
  * Apply the alpha value to the specified pixbuf
  */
@@ -325,6 +354,19 @@ GtkWidget *ui_entry_new ( const gchar *str, GtkEntryIconPosition position )
 	return entry;
 }
 
+/**
+ * Set the entry field text
+ *
+ * Handles NULL, unlike gtk_entry_set_text() which complains
+ */
+void ui_entry_set_text ( GtkWidget *widget, const gchar *str )
+{
+	if ( str )
+		gtk_entry_set_text ( GTK_ENTRY(widget), str );
+	else
+		gtk_entry_set_text ( GTK_ENTRY(widget), "" );
+}
+
 /**
  * Create a spinbutton with an icon to clear the entry
  *
@@ -375,3 +417,180 @@ GtkTreeViewColumn *ui_new_column_text ( const gchar *title, GtkCellRenderer *ren
 	gtk_tree_view_column_set_resizable ( column, TRUE );
 	return column;
 }
+
+/**
+ * ui_tree_model_number_tooltip_cb:
+ *
+ * A simple tooltip on a widget for showing number of items displayed in the tree model
+ *
+ */
+gboolean ui_tree_model_number_tooltip_cb ( GtkWidget    *widget,
+                                           gint          x,
+                                           gint          y,
+                                           gboolean      keyboard_mode,
+                                           GtkTooltip   *tooltip,
+                                           GtkTreeModel *tree_model )
+{
+	gint num = gtk_tree_model_iter_n_children ( tree_model, NULL );
+	if ( num ) {
+		gchar *text = g_strdup_printf ( _("Items: %d"), num );
+		gtk_tooltip_set_text ( tooltip, text );
+		g_free ( text );
+	}
+	return (num != 0);
+}
+
+/**
+ * Load Icons using GResource (see icons/icons.gresource.xml)
+ */
+void ui_load_icons ( void )
+{
+	const gchar *vikpath = "/org/viking-gps/viking/icons";
+	// Much easier in GTK3
+#if GTK_CHECK_VERSION(3, 14, 0)
+	gtk_icon_theme_add_resource_path ( gtk_icon_theme_get_default(), vikpath );
+#else
+	GResource *icon_res = vik_icons_get_resource();
+	char ** children = g_resource_enumerate_children ( icon_res, vikpath, 0, NULL );
+
+	if ( children == NULL ) {
+		g_critical ( "No icons in resources!" );
+		return;
+	}
+
+	for ( int ii = 0; children[ii] != NULL; ii++ ) {
+		char *path = g_strdup_printf ( "%s/%s", vikpath, children[ii] );
+		GError *error = NULL;
+
+		GdkPixbuf *pb = gdk_pixbuf_new_from_resource ( path, &error );
+		if ( error ) {
+			g_warning ( "%s: %s, %s", __FUNCTION__, children[ii], error->message );
+			g_error_free ( error );
+			break;
+		}
+		
+		int size = gdk_pixbuf_get_height ( pb );
+		// Use a normalized name (i.e. skip the '.png')
+		char *name = g_strdup ( children[ii] );
+		char *pos = g_strstr_len ( name, -1, "." );
+		if ( pos != NULL )
+			*pos = '\0';
+		gtk_icon_theme_add_builtin_icon ( name, size, pb );
+		g_object_unref ( pb );
+		g_free ( name );
+	}
+	g_strfreev ( children );
+#endif
+}
+
+GdkPixbuf *ui_get_icon ( const gchar *name, guint size )
+{
+	GError *error = NULL;
+	GdkPixbuf *icon = gtk_icon_theme_load_icon ( gtk_icon_theme_get_default(), name, size, GTK_ICON_LOOKUP_FORCE_SIZE, &error );
+	if ( error ) {
+		// Own icons should always be defined
+		g_critical ( "%s: icon '%s' - %s", __FUNCTION__, name, error->message );
+		g_error_free ( error );
+	}
+	return icon;
+}
+
+/**
+ * replacement for gdk_draw_layout()
+ * note that x,y are now doubles and may need to be offset by 0.5
+ */
+void ui_cr_draw_layout ( cairo_t *cr, gdouble xx, gdouble yy, PangoLayout *layout )
+{
+	cairo_move_to ( cr, xx, yy );
+	pango_cairo_show_layout ( cr, layout );
+}
+
+/**
+ * replacement for gdk_draw_line()
+ * note that as x,y are now doubles and describe the center of the line
+ * ATM no clipping is performed and unknown what happens if ones uses
+ *  very large numbers for x2, y2
+ *
+ * cairo_stroke() should be called after using this
+ *  (or after consecutive calls to this are done as a group)
+ */
+void ui_cr_draw_line ( cairo_t *cr, gdouble x1, gdouble y1, gdouble x2, gdouble y2 )
+{
+	cairo_move_to ( cr, x1, y1 );
+	cairo_line_to ( cr, x2, y2 );
+}
+
+/**
+ * replacement for gdk_draw_rectangle()
+ * note that as x,y are now doubles and describe the center of the line
+ * ATM no clipping is performed and unknown what happens if ones uses very large numbers
+ * cairo_stroke() should be called after using this
+ *  (or after consecutive calls to this are done as a group)
+ */
+void ui_cr_draw_rectangle ( cairo_t *cr, gboolean fill, gdouble xx, gdouble yy, gdouble ww, gdouble hh )
+{
+	cairo_rectangle ( cr, xx, yy, ww, hh );
+	if ( fill )
+		cairo_fill ( cr );
+}
+
+void ui_cr_set_color ( cairo_t *cr, const gchar *name )
+{
+  GdkColor color;
+  if ( gdk_color_parse(name, &color) ) {
+    gdk_cairo_set_source_color ( cr, &color );
+  }
+}
+
+// Generic dashed line
+void ui_cr_set_dash ( cairo_t *cr )
+{
+	// ATM assume this doesn't need to be multiplied by the vvp->scale
+	static const double dashed[] = {4.0};
+	cairo_set_dash ( cr, dashed, 1, 0 );
+}
+
+void ui_cr_clear ( cairo_t *cr )
+{
+	g_return_if_fail ( cr != NULL );
+
+	cairo_operator_t operator = cairo_get_operator ( cr );
+	// Clear the buffer
+	cairo_set_operator ( cr, CAIRO_OPERATOR_CLEAR );
+	cairo_paint ( cr );
+	// Return to the previous mode
+	//  typically will be the default operator CAIRO_OPERATOR_OVER, for the normal drawing on top mode
+	cairo_set_operator ( cr, operator );
+}
+
+// Typically should be used in the 'draw' event
+// Draw the specified surface (whole screen)
+//  which would be overlayed on any other surface
+void ui_cr_surface_paint ( cairo_t *cr, cairo_surface_t *surface )
+{
+	g_return_if_fail ( surface != NULL );
+	cairo_set_source_surface ( cr, surface, 0.0, 0.0 );
+	cairo_paint ( cr );
+}
+
+// Label with solid a background and a border
+void ui_cr_label_with_bg (cairo_t *cr, gint xd, gint yd, gint wd, gint hd, PangoLayout *pl)
+{
+	g_return_if_fail ( cr != NULL );
+	ui_cr_set_color ( cr, "#cccccc" );
+	ui_cr_draw_rectangle ( cr, TRUE, xd-2, yd-1, wd+4, hd+1 );
+	cairo_stroke ( cr );
+	ui_cr_set_color ( cr, "#000000" );
+	ui_cr_draw_rectangle ( cr, FALSE, xd-2, yd-1, wd+4, hd+1 );
+	cairo_stroke ( cr );
+	ui_cr_draw_layout ( cr, xd, yd, pl );
+}
+
+void ui_gc_unref ( GdkGC *gc )
+{
+#if GTK_CHECK_VERSION (3,0,0)
+	cairo_destroy ( gc );
+#else
+	g_object_unref ( G_OBJECT(gc) );
+#endif
+}
