@@ -31,36 +31,15 @@
 #include <glib/gprintf.h>
 #include <gio/gio.h>
 #include <math.h>
+#include <ctype.h> // For isalpha() etc...
 
 #include "util.h"
 #include "globals.h"
 #include "fileutils.h"
 
-#ifdef WINDOWS
-#include <windows.h>
-#else
-#include <unistd.h>
-#endif
-
 guint util_get_number_of_cpus ()
 {
-#if GLIB_CHECK_VERSION (2, 36, 0)
   return g_get_num_processors();
-#else
-  long nprocs = 1;
-#ifdef WINDOWS
-  SYSTEM_INFO info;
-  GetSystemInfo(&info);
-  nprocs = info.dwNumberOfProcessors;
-#else
-#ifdef _SC_NPROCESSORS_ONLN
-  nprocs = sysconf(_SC_NPROCESSORS_ONLN);
-  if (nprocs < 1)
-    nprocs = 1;
-#endif
-#endif
-  return nprocs;
-#endif
 }
 
 /**
@@ -197,25 +176,8 @@ gchar* util_write_tmp_file_from_bytes ( const void *buffer, gsize count )
 {
 	GFileIOStream *gios;
 	GError *error = NULL;
-	gchar *tmpname = NULL;
-
-#if GLIB_CHECK_VERSION(2,32,0)
 	GFile *gf = g_file_new_tmp ( "vik-tmp.XXXXXX", &gios, &error );
-	tmpname = g_file_get_path (gf);
-#else
-	gint fd = g_file_open_tmp ( "vik-tmp.XXXXXX", &tmpname, &error );
-	if ( error ) {
-		g_warning ( "%s", error->message );
-		g_error_free ( error );
-		return NULL;
-	}
-	gios = g_file_open_readwrite ( g_file_new_for_path (tmpname), NULL, &error );
-	if ( error ) {
-		g_warning ( "%s", error->message );
-		g_error_free ( error );
-		return NULL;
-	}
-#endif
+	gchar *tmpname = g_file_get_path (gf);
 
 	gios = g_file_open_readwrite ( g_file_new_for_path (tmpname), NULL, &error );
 	if ( error ) {
@@ -365,3 +327,60 @@ gboolean util_is_url ( const gchar *str )
 	}
 	return FALSE;
 }
+
+// 'Hide' a string
+gchar* util_frob ( gchar *str, guint ii )
+{
+	gchar *ptr = str;
+	while ( ptr && (ii-- > 0) ) {
+		if ( isalpha(*ptr) ) {
+			if ( tolower(*ptr) < 'n' )
+				*ptr += 13;
+			else
+				*ptr -= 13;
+		}
+		else if ( isdigit(*ptr) ) {
+			if ( *ptr < '5' )
+				*ptr += 5;
+			else
+				*ptr -= 5;
+		}
+		ptr++;
+	}
+	return str;
+}
+
+/**
+ * util_glist_of_strings_compare:
+ *
+ * Returns: TRUE if they have the same list of string contents
+ */
+gboolean util_glist_of_strings_compare ( GList *aa, GList *bb )
+{
+  gboolean ans = TRUE;
+  if ( g_list_length(aa) == g_list_length(bb) ) {
+    while ( aa ) {
+      if ( g_strcmp0((gchar*)aa->data, (gchar*)bb->data) != 0 ) {
+        ans = FALSE;
+        break;
+      }
+      aa = aa->next;
+      bb = bb->next;
+    }
+  }
+  else
+    // Must be different if different lengths
+    ans = FALSE;
+
+  return ans;
+}
+
+// NB directly comparing "NaN != NaN" results in true!!
+//  so need to check isnan() first
+//  and further directly comparing floating point numbers is not wise
+gboolean util_gdouble_different ( gdouble aa, gdouble bb )
+{
+  static const gdouble TOL = 0.000005; // Tolerance
+  return ( ( !isnan(aa) || !isnan(bb) ) &&
+           ( (aa < (bb - TOL)) || (aa > (bb + TOL)) ) );
+}
