@@ -30,11 +30,16 @@
 #include "vikfilelist.h"
 
 struct _VikFileList {
+#if GTK_CHECK_VERSION (3,0,0)
+  GtkBox parent;
+#else
   GtkVBox parent;
+#endif
   GtkWidget *treeview;
   GtkWidget *file_selector;
   GtkTreeModel *model;
   GtkFileFilter *filter;
+  gchar *dir;
 };
 
 static void file_list_add ( VikFileList *vfl )
@@ -56,6 +61,12 @@ static void file_list_add ( VikFileList *vfl )
     gtk_window_set_transient_for ( GTK_WINDOW(vfl->file_selector), GTK_WINDOW(win) );
     gtk_window_set_destroy_with_parent ( GTK_WINDOW(vfl->file_selector), TRUE );
 
+    if ( vfl->dir ) {
+      (void)gtk_file_chooser_set_current_folder ( GTK_FILE_CHOOSER(vfl->file_selector), vfl->dir );
+      g_free ( vfl->dir );
+      vfl->dir = NULL;
+    }
+
     if ( vfl->filter )
       gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER(vfl->file_selector), vfl->filter );
   }
@@ -158,13 +169,18 @@ GType vik_file_list_get_type (void)
  * Support just one filter, as that's all that's needed ATM
  * Probably need to use a GList of them if more than one is required
  */
-GtkWidget *vik_file_list_new ( const gchar *title, GtkFileFilter *filter )
+GtkWidget *vik_file_list_new ( const gchar *title, GtkFileFilter *filter, const gchar *dir )
 {
   GtkWidget *add_btn, *del_btn;
   GtkWidget *hbox, *scrolledwindow;
   VikFileList *vfl = VIK_FILE_LIST ( g_object_new ( VIK_FILE_LIST_TYPE, NULL ) );
 
+#if GTK_CHECK_VERSION (3,0,0)
+  // Force vertical mode for box (as default is horizontal in GTK3)
+  gtk_orientable_set_orientation ( GTK_ORIENTABLE(vfl), GTK_ORIENTATION_VERTICAL );
+#endif
   vfl->filter = filter;
+  vfl->dir = g_strdup ( dir );
 
   GtkTreeViewColumn *column;
 
@@ -220,8 +236,19 @@ GList *vik_file_list_get_files ( VikFileList *vfl )
   return list;
 }
 
+/**
+ * NB This set files specifically does not clear out previous values if the list is NULL
+ * (not clear if this was a previous oversight but in some ways OK)
+ * As a useful benefit is that the list is maintained e.g.
+ *   when otherwise resetting to default values, (although perhaps not obvious behaviour)
+ * ATM Users can use select all entries manually (e.g. Ctrl+a) and use 'Delete' if they wish
+ *  (rather than providing a specific 'Remove All' button which could be accidentally invoked)
+ */
 void vik_file_list_set_files ( VikFileList *vfl, GList *files )
 {
+  // if ( !files )
+  //   gtk_list_store_clear ( GTK_LIST_STORE(vfl->model) );
+
   while (files) {
     GtkTreeIter iter;
     gtk_list_store_append ( GTK_LIST_STORE(vfl->model), &iter );
