@@ -22,14 +22,16 @@
 #include "vikgeocluelayer.h"
 #include "viking.h"
 #include "libgeoclue.h"
-#include "icons/icons.h"
 
-static VikGeoclueLayer *vik_geoclue_layer_create ( VikViewport *vp);
-static void vik_geoclue_layer_realize ( VikGeoclueLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter );
-static void vik_geoclue_layer_post_read ( VikGeoclueLayer *vgl, VikViewport *vp, gboolean from_file );
-static void vik_geoclue_layer_free ( VikGeoclueLayer *vgl );
-static void vik_geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp );
-static VikGeoclueLayer *vik_geoclue_layer_new ( VikViewport *vp );
+#define GEOCLUE_FIXED_NAME "GeoClue"
+
+static VikGeoclueLayer *geoclue_layer_create ( VikViewport *vp);
+static void geoclue_layer_realize ( VikGeoclueLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter );
+static void geoclue_layer_post_read ( VikGeoclueLayer *vgl, VikViewport *vp, gboolean from_file );
+static void geoclue_layer_free ( VikGeoclueLayer *vgl );
+static void geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp );
+static VikGeoclueLayer *geoclue_layer_new ( VikViewport *vp );
+static void geoclue_layer_configure ( VikGeoclueLayer *vgl, VikViewport *vp );
 
 static void geoclue_layer_marshall( VikGeoclueLayer *vgl, guint8 **data, guint *len );
 static VikGeoclueLayer *geoclue_layer_unmarshall ( guint8 *data, guint len, VikViewport *vvp );
@@ -64,6 +66,13 @@ static VikLayerParamData color_default ( void ) {
 	VikLayerParamData data; gdk_color_parse ( "purple", &data.c ); return data;
 }
 
+static void reset_cb ( GtkWidget *widget, gpointer ptr )
+{
+	a_layer_defaults_reset_show ( GEOCLUE_FIXED_NAME, ptr, VIK_LAYER_GROUP_NONE );
+}
+
+static VikLayerParamData reset_default ( void ) { return VIK_LPD_PTR(reset_cb); }
+
 static VikLayerParam geoclue_layer_params[] = {
 	{ VIK_LAYER_GEOCLUE, "auto_connect", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Auto Connect"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("Automatically connect to GeoClue"), vik_lpd_true_default, NULL, NULL },
 	{ VIK_LAYER_GEOCLUE, "record_tracking", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Recording tracks"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
@@ -71,6 +80,8 @@ static VikLayerParam geoclue_layer_params[] = {
 	{ VIK_LAYER_GEOCLUE, "moving_map_method", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Moving Map Method:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_position, NULL, NULL, moving_map_method_default, NULL, NULL },
 	{ VIK_LAYER_GEOCLUE, "update_statusbar", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Update Statusbar:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("Display information in the statusbar on GeoClue updates"), vik_lpd_true_default, NULL, NULL },
 	{ VIK_LAYER_GEOCLUE, "color", VIK_LAYER_PARAM_COLOR, VIK_LAYER_GROUP_NONE, N_("Color:"), VIK_LAYER_WIDGET_COLOR, NULL, NULL, NULL, color_default, NULL, NULL },
+	{ VIK_LAYER_GEOCLUE, "reset", VIK_LAYER_PARAM_PTR_DEFAULT, VIK_LAYER_GROUP_NONE, NULL,
+	  VIK_LAYER_WIDGET_BUTTON, N_("Reset to Defaults"), NULL, NULL, reset_default, NULL, NULL },
 };
 
 typedef enum {
@@ -80,14 +91,15 @@ typedef enum {
 	PARAM_POSITION,
 	PARAM_UPDATE_STATUSBAR,
 	PARAM_COLOR,
+	PARAM_RESET,
 	NUM_PARAMS
 } geoclue_param_t;
 
 VikLayerInterface vik_geoclue_layer_interface = {
-  "GeoClue",
+  GEOCLUE_FIXED_NAME,
   N_("GeoClue"),
   NULL,
-  &vikgeocluelayer_pixbuf,
+  "vikgeocluelayer",
 
   NULL,
   0,
@@ -99,13 +111,14 @@ VikLayerInterface vik_geoclue_layer_interface = {
 
   VIK_MENU_ITEM_ALL,
 
-  (VikLayerFuncCreate)                  vik_geoclue_layer_create,
-  (VikLayerFuncRealize)                 vik_geoclue_layer_realize,
-  (VikLayerFuncPostRead)                vik_geoclue_layer_post_read,
-  (VikLayerFuncFree)                    vik_geoclue_layer_free,
+  (VikLayerFuncCreate)                  geoclue_layer_create,
+  (VikLayerFuncRealize)                 geoclue_layer_realize,
+  (VikLayerFuncPostRead)                geoclue_layer_post_read,
+  (VikLayerFuncFree)                    geoclue_layer_free,
 
   (VikLayerFuncProperties)              NULL,
-  (VikLayerFuncDraw)                    vik_geoclue_layer_draw,
+  (VikLayerFuncDraw)                    geoclue_layer_draw,
+  (VikLayerFuncConfigure)               geoclue_layer_configure,
   (VikLayerFuncChangeCoordMode)         geoclue_layer_change_coord_mode,
 
   (VikLayerFuncGetTimestamp)            NULL,
@@ -121,6 +134,7 @@ VikLayerInterface vik_geoclue_layer_interface = {
   (VikLayerFuncSublayerTooltip)         NULL,
   (VikLayerFuncLayerTooltip)            geoclue_layer_tooltip,
   (VikLayerFuncLayerSelected)           NULL,
+  (VikLayerFuncLayerToggleVisible)      NULL,
 
   (VikLayerFuncMarshall)                geoclue_layer_marshall,
   (VikLayerFuncUnmarshall)              geoclue_layer_unmarshall,
@@ -196,9 +210,9 @@ GType vik_geoclue_layer_get_type ()
   return val_type;
 }
 
-static VikGeoclueLayer *vik_geoclue_layer_create ( VikViewport *vp )
+static VikGeoclueLayer *geoclue_layer_create ( VikViewport *vp )
 {
-	VikGeoclueLayer *rv = vik_geoclue_layer_new ( vp );
+	VikGeoclueLayer *rv = geoclue_layer_new ( vp );
 	vik_layer_rename ( VIK_LAYER(rv), vik_geoclue_layer_interface.name );
 	rv->trw = VIK_TRW_LAYER(vik_layer_create ( VIK_LAYER_TRW, vp, FALSE ));
 	vik_layer_set_menu_items_selection ( VIK_LAYER(rv->trw), VIK_MENU_ITEM_ALL & ~(VIK_MENU_ITEM_CUT|VIK_MENU_ITEM_DELETE) );
@@ -251,7 +265,7 @@ static VikGeoclueLayer *geoclue_layer_unmarshall ( guint8 *data, guint len, VikV
 	len -= sizeof(guint) + alm_size; \
 	data += sizeof(guint) + alm_size;
   
-	VikGeoclueLayer *rv = vik_geoclue_layer_new(vvp);
+	VikGeoclueLayer *rv = geoclue_layer_new(vvp);
 
 	vik_layer_unmarshall_params ( VIK_LAYER(rv), data+sizeof(guint), alm_size, vvp );
 	alm_next;
@@ -260,7 +274,7 @@ static VikGeoclueLayer *geoclue_layer_unmarshall ( guint8 *data, guint len, VikV
 	if (layer) {
 		rv->trw = (VikTrwLayer*)layer;
 		// NB no need to attach signal update handler here
-		//  as this will always be performed later on in vik_geoclue_layer_realize()
+		//  as this will always be performed later on in geoclue_layer_realize()
 	}
 	alm_next;
 	return rv;
@@ -277,7 +291,7 @@ static gboolean geoclue_layer_set_param ( VikGeoclueLayer *vgl, VikLayerSetParam
 	case PARAM_POSITION:         vgl->position = vlsp->data.u; break;
 	case PARAM_UPDATE_STATUSBAR: vgl->update_statusbar = vlsp->data.b; break;
 	case PARAM_COLOR:            vgl->color = vlsp->data.c; break;
-	default: g_warning("geoclue_layer_set_param(): unknown parameter");	break;
+	default: break;
 	}
 	return TRUE;
 }
@@ -292,20 +306,24 @@ static VikLayerParamData geoclue_layer_get_param ( VikGeoclueLayer *vgl, guint16
 	case PARAM_POSITION:         rv.u = vgl->position; break;
 	case PARAM_UPDATE_STATUSBAR: rv.u = vgl->update_statusbar; break;
 	case PARAM_COLOR:            rv.c = vgl->color; break;
-	default:
-		g_warning ( _("%s: unknown parameter"), __FUNCTION__ );
-		break;
+	case PARAM_RESET:            rv.ptr = reset_cb; break;
+	default: break;
 	}
 	return rv;
 }
 
-VikGeoclueLayer *vik_geoclue_layer_new ( VikViewport *vp )
+static void gcs_create ( VikGeoclueLayer *vgl, VikViewport *vp )
 {
-	VikGeoclueLayer *vgl = VIK_GEOCLUE_LAYER ( g_object_new ( VIK_GEOCLUE_LAYER_TYPE, NULL ) );
-	vik_layer_set_type ( VIK_LAYER(vgl), VIK_LAYER_GEOCLUE );
 	if ( vp ) {
 		vgl->track_pt_gc = vik_viewport_new_gc_from_color ( vp, &(vgl->color), 2 );
 	}
+}
+
+static VikGeoclueLayer *geoclue_layer_new ( VikViewport *vp )
+{
+	VikGeoclueLayer *vgl = VIK_GEOCLUE_LAYER ( g_object_new ( VIK_GEOCLUE_LAYER_TYPE, NULL ) );
+	vik_layer_set_type ( VIK_LAYER(vgl), VIK_LAYER_GEOCLUE );
+	gcs_create ( vgl, vp );
 	vik_layer_set_defaults ( VIK_LAYER(vgl), vp );
 	// Everything else is 0, FALSE or NULL
 	return vgl;
@@ -324,7 +342,7 @@ static void tracking_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
 		vik_coord_to_latlon ( &nw, &lnw );
 		vik_coord_to_latlon ( &se, &lse );
 
-		vik_coord_load_from_latlon ( &vgl->coord, vik_viewport_get_coord_mode(vp), &ll );
+		vik_coord_to_latlon ( &vgl->coord, &ll );
 
 		// If it is in the viewport then draw something
 		if ( ll.lat > lse.lat &&
@@ -337,12 +355,12 @@ static void tracking_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
 			// Maybe allow configuration of triangle size?
 			gint x, y;
 			vik_viewport_coord_to_screen ( vp, &vgl->coord, &x, &y );
-			vik_viewport_draw_rectangle ( vp, vgl->track_pt_gc, TRUE, x-tp_size, y-tp_size, tp_size*2, tp_size*2 );
+			vik_viewport_draw_rectangle ( vp, vgl->track_pt_gc, TRUE, x-tp_size, y-tp_size, tp_size*2, tp_size*2, &(vgl->color) );
 		}
 	}
 }
 
-static void vik_geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
+static void geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
 {
 	// NB I don't understand this half drawn business
 	// This is just copied from vikgpslayer.c
@@ -375,6 +393,18 @@ static void vik_geoclue_layer_draw ( VikGeoclueLayer *vgl, VikViewport *vp )
   }
 }
 
+static void gcs_free ( VikGeoclueLayer *vgl )
+{
+	if ( vgl->track_pt_gc )
+		ui_gc_unref ( vgl->track_pt_gc );
+}
+
+static void geoclue_layer_configure ( VikGeoclueLayer *vgl, VikViewport *vp )
+{
+	gcs_free ( vgl );
+	gcs_create ( vgl, vp );
+}
+
 static void geoclue_layer_change_coord_mode ( VikGeoclueLayer *vgl, VikCoordMode mode )
 {
 	vik_layer_change_coord_mode ( VIK_LAYER(vgl->trw), mode );
@@ -420,19 +450,16 @@ static void disconnect_layer_signal ( VikLayer *vl, VikGeoclueLayer *vgl )
 	}
 }
 
-static void vik_geoclue_layer_free ( VikGeoclueLayer *vgl )
+static void geoclue_layer_free ( VikGeoclueLayer *vgl )
 {
-#if GLIB_CHECK_VERSION(2,28,0)
     if ( vgl->client )
 		g_clear_object ( &vgl->client );
     if ( vgl->simple )
 		g_clear_object ( &vgl->simple );
-#endif
 	if ( vgl->vl.realized )
 		disconnect_layer_signal ( VIK_LAYER(vgl->trw), vgl );
 	g_object_unref ( vgl->trw );
-	if ( vgl->track_pt_gc != NULL )
-		g_object_unref ( vgl->track_pt_gc );
+	gcs_free ( vgl );
 }
 
 gboolean vik_geoclue_layer_is_empty ( VikGeoclueLayer *vgl )
@@ -600,7 +627,7 @@ notify_location ( GClueSimple *simple,
 		vgl->trkpt_prev = vgl->trkpt;
 	}
 
-	vik_layer_emit_update ( update_all ? VIK_LAYER(vgl) : VIK_LAYER(vgl->trw) ); // NB update from background thread
+	vik_layer_emit_update ( update_all ? VIK_LAYER(vgl) : VIK_LAYER(vgl->trw), vgl->trkpt ? TRUE : FALSE ); // NB update from background thread
 }
 
 static void
@@ -611,7 +638,7 @@ notify_active ( GClueClient *client,
 	if ( gclue_client_get_active (client) )
 		return;
 
-	g_warning ( "%s: geoclue no longer active\n", __FUNCTION__ );
+	g_warning ( "%s: geoclue no longer active", __FUNCTION__ );
 	// Deactivate
 	VikGeoclueLayer *vgl = (VikGeoclueLayer*)user_data;
 	vgl->tracking = FALSE;
@@ -658,7 +685,6 @@ on_simple_ready ( GObject      *source_object,
 	if ( vgl->record ) {
 		VikTrwLayer *vtl = vgl->trw;
 		vgl->track = vik_track_new();
-		vgl->track->visible = TRUE;
 		gchar *name = make_track_name ( vtl );
 		vik_trw_layer_add_track ( vtl, name, vgl->track );
 		g_free ( name );
@@ -690,7 +716,7 @@ static void geoclue_connect ( VikGeoclueLayer *vgl )
 	                   vgl );
 }
 
-static void vik_geoclue_layer_realize ( VikGeoclueLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter )
+static void geoclue_layer_realize ( VikGeoclueLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter )
 {
 	GtkTreeIter iter;
 	VikLayer *trw = VIK_LAYER(vgl->trw);
@@ -707,7 +733,7 @@ static void vik_geoclue_layer_realize ( VikGeoclueLayer *vgl, VikTreeview *vt, G
 	}
 }
 
-static void vik_geoclue_layer_post_read ( VikGeoclueLayer *vgl, VikViewport *vp, gboolean from_file )
+static void geoclue_layer_post_read ( VikGeoclueLayer *vgl, VikViewport *vp, gboolean from_file )
 {
 	// Ensure bounds values are set (particularly if loading from a file)
 	trw_layer_calculate_bounds_waypoints ( vgl->trw );
@@ -724,12 +750,10 @@ static void geoclue_start_stop_tracking_cb ( menu_array_layer values )
 	else {
 		// stop tracking
 		vgl->tracking = FALSE;
-#if GLIB_CHECK_VERSION(2,28,0)
 		if ( vgl->client )
 			g_clear_object ( &vgl->client );
 		if ( vgl->simple )
 			g_clear_object ( &vgl->simple );
-#endif
 		vgl->first_trackpoint = FALSE;
 		vgl->trkpt = NULL;
 	}
