@@ -23,8 +23,6 @@
 #include "vikgeoreflayer.h"
 #include <ctype.h>
 
-#include "preferences.h"
-#include "icons/icons.h"
 #include "vikmapslayer.h"
 
 /*
@@ -75,37 +73,37 @@ static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboo
 
 /* tools */
 static gpointer georef_layer_move_create ( VikWindow *vw, VikViewport *vvp);
-static gboolean georef_layer_move_release ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
-static gboolean georef_layer_move_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
+static VikLayerToolFuncStatus georef_layer_move_release ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
+static VikLayerToolFuncStatus georef_layer_move_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
 static gpointer georef_layer_zoom_create ( VikWindow *vw, VikViewport *vvp);
-static gboolean georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
+static VikLayerToolFuncStatus georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp );
 
 // See comment in viktrwlayer.c for advice on values used
 static VikToolInterface georef_tools[] = {
-  { &geomove_18_pixbuf,
-    { "GeorefMoveMap", "vik-icon-Georef Move Map",  N_("_Georef Move Map"), NULL,  N_("Georef Move Map"), 0 },
+  { "geomove_18",
+    { "GeorefMoveMap", "geomove_18",  N_("_Georef Move Map"), NULL,  N_("Georef Move Map"), 0 },
     (VikToolConstructorFunc) georef_layer_move_create, NULL, NULL, NULL,
     (VikToolMouseFunc) georef_layer_move_press, NULL, (VikToolMouseFunc) georef_layer_move_release,
     (VikToolKeyFunc) NULL,
     (VikToolKeyFunc) NULL,
     FALSE,
-    GDK_CURSOR_IS_PIXMAP, &cursor_geomove_pixbuf, NULL },
+    GDK_CURSOR_IS_PIXMAP, "cursor_geomove", NULL },
 
-  { &geozoom_18_pixbuf,
-    { "GeorefZoomTool", "vik-icon-Georef Zoom Tool",  N_("Georef Z_oom Tool"), NULL,  N_("Georef Zoom Tool"), 0 },
+  { "geozoom_18",
+    { "GeorefZoomTool", "geozoom_18",  N_("Georef Z_oom Tool"), NULL,  N_("Georef Zoom Tool"), 0 },
     (VikToolConstructorFunc) georef_layer_zoom_create, NULL, NULL, NULL,
     (VikToolMouseFunc) georef_layer_zoom_press, NULL, NULL,
     (VikToolKeyFunc) NULL,
     (VikToolKeyFunc) NULL,
     FALSE,
-    GDK_CURSOR_IS_PIXMAP, &cursor_geozoom_pixbuf, NULL },
+    GDK_CURSOR_IS_PIXMAP, "cursor_geozoom", NULL },
 };
 
 VikLayerInterface vik_georef_layer_interface = {
   "GeoRef Map",
   N_("GeoRef Map"),
   NULL,
-  &vikgeoreflayer_pixbuf, /*icon */
+  "vikgeoreflayer", // Icon name
 
   georef_tools,
   sizeof(georef_tools) / sizeof(VikToolInterface),
@@ -124,6 +122,7 @@ VikLayerInterface vik_georef_layer_interface = {
 
   (VikLayerFuncProperties)              georef_layer_properties,
   (VikLayerFuncDraw)                    georef_layer_draw,
+  (VikLayerFuncConfigure)               NULL, //georef_layer_configure?,
   (VikLayerFuncChangeCoordMode)         NULL,
 
   (VikLayerFuncGetTimestamp)            NULL,
@@ -139,6 +138,7 @@ VikLayerInterface vik_georef_layer_interface = {
   (VikLayerFuncSublayerTooltip)         NULL,
   (VikLayerFuncLayerTooltip)            georef_layer_tooltip,
   (VikLayerFuncLayerSelected)           NULL,
+  (VikLayerFuncLayerToggleVisible)      NULL,
 
   (VikLayerFuncMarshall)		georef_layer_marshall,
   (VikLayerFuncUnmarshall)		georef_layer_unmarshall,
@@ -203,14 +203,12 @@ struct _VikGeorefLayer {
 
 static VikLayerParam io_prefs[] = {
   { VIK_LAYER_NUM_TYPES, VIKING_PREFERENCES_IO_NAMESPACE "georef_auto_read_world_file", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Auto Read World Files:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL,
-    N_("Automatically attempt to read associated world file of a new image for a GeoRef layer"), NULL, NULL, NULL}
+    N_("Automatically attempt to read associated world file of a new image for a GeoRef layer"), vik_lpd_true_default, NULL, NULL}
 };
 
 void vik_georef_layer_init (void)
 {
-  VikLayerParamData tmp;
-  tmp.b = TRUE;
-  a_preferences_register(&io_prefs[0], tmp, VIKING_PREFERENCES_IO_GROUP_KEY);
+  a_preferences_register ( &io_prefs[0], (VikLayerParamData){0}, VIKING_PREFERENCES_IO_GROUP_KEY );
 }
 
 GType vik_georef_layer_get_type ()
@@ -259,6 +257,7 @@ static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, guint len, VikView
 
 static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *vlsp )
 {
+  //gboolean changed = FALSE;
   switch ( vlsp->id )
   {
     case PARAM_IMAGE: georef_layer_set_image ( vgl, vlsp->data.s ); break;
@@ -267,10 +266,44 @@ static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *
     case PARAM_MN: vgl->mpp_northing = vlsp->data.d; break;
     case PARAM_ME: vgl->mpp_easting = vlsp->data.d; break;
     case PARAM_CZ: if ( vlsp->data.u <= 60 ) vgl->corner.zone = vlsp->data.u; break;
-    case PARAM_CL: if ( vlsp->data.u >= 65 || vlsp->data.u <= 90 ) vgl->corner.letter = vlsp->data.u; break;
+    case PARAM_CL: if ( vlsp->data.u >= 65 && vlsp->data.u <= 90 ) vgl->corner.letter = vlsp->data.u; break;
     case PARAM_AA: if ( vlsp->data.u <= 255 ) vgl->alpha = vlsp->data.u; break;
+/*
+  case PARAM_IMAGE: {
+      gchar* old = g_strdup ( vgl->image );
+      georef_layer_set_image ( vgl, vlsp->data.s );
+      changed = g_strcmp0 ( old, vlsp->data.s );
+      g_free ( old );
+      break;
+    }
+    case PARAM_CN:
+      changed = vik_layer_param_change_double ( vlsp->data, &vgl->corner.northing );
+      break;
+    case PARAM_CE:
+      changed = vik_layer_param_change_double ( vlsp->data, &vgl->corner.easting );
+      break;
+    case PARAM_MN:
+      changed = vik_layer_param_change_double ( vlsp->data, &vgl->mpp_northing );
+      break;
+    case PARAM_ME:
+      changed = vik_layer_param_change_double ( vlsp->data, &vgl->mpp_easting );
+      break;
+    case PARAM_CZ:
+      if ( vlsp->data.u <= 60 )
+        changed = vik_layer_param_change_gchar ( vlsp->data, &vgl->corner.zone );
+      break;
+    case PARAM_CL:
+      if ( vlsp->data.u >= 65 && vlsp->data.u <= 90 )
+        changed = vik_layer_param_change_gchar ( vlsp->data, &vgl->corner.letter );
+      break;
+    case PARAM_AA:
+      if ( vlsp->data.u <= 255 )
+        changed = vik_layer_param_change_uint8 ( vlsp->data, &vgl->alpha );
+      break;
+*/
     default: break;
   }
+  //return changed;
   return TRUE;
 }
 
@@ -477,7 +510,12 @@ static VikGeorefLayer *georef_layer_create ( VikViewport *vp )
 
 static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp, gboolean have_apply )
 {
-  return georef_layer_dialog ( vgl, vp, VIK_GTK_WINDOW_FROM_WIDGET(vp), have_apply );
+  // TODO Determine if anything actually changed;
+  // For simplicity 
+  gboolean OKed = georef_layer_dialog ( vgl, vp, VIK_GTK_WINDOW_FROM_WIDGET(vp), have_apply );
+  if ( OKed )
+    vik_window_set_modified ( (VikWindow *)(VIK_GTK_WINDOW_FROM_WIDGET(vp)) );
+  return OKed;
 }
 
 static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboolean from_file )
@@ -605,7 +643,8 @@ static void georef_layer_dialog_load ( changeable_widgets *cw )
   if ( gtk_dialog_run ( GTK_DIALOG ( file_selector ) ) == GTK_RESPONSE_ACCEPT )
   {
      gdouble values[4];
-     gint answer = world_file_read_file ( gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_selector)), values );
+     gchar *fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
+     gint answer = world_file_read_file ( fn, values );
      if ( answer == 1 )
        a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_WIDGET(cw->x_spin), _("The World file you requested could not be opened for reading.") );
      else if ( answer == 2 )
@@ -613,6 +652,7 @@ static void georef_layer_dialog_load ( changeable_widgets *cw )
      else
        // NB answer should == 0 for success
        set_widget_values ( cw, values );
+     g_free ( fn );
   }
 
   gtk_widget_destroy ( file_selector );
@@ -629,9 +669,10 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
 				      NULL);
   if ( gtk_dialog_run ( GTK_DIALOG ( file_selector ) ) == GTK_RESPONSE_ACCEPT )
   {
-    FILE *f = g_fopen ( gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) ), "w" );
-    
-    gtk_widget_destroy ( file_selector ); 
+    gchar *fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
+    FILE *f = g_fopen ( fn, "w" );
+    g_free ( fn );
+    gtk_widget_destroy ( file_selector );
     if ( !f )
     {
       a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_WIDGET(pass_along[0]), _("The file you requested could not be opened for writing.") );
@@ -641,11 +682,10 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
     {
       fprintf ( f, "%f\n%f\n%f\n%f\n%f\n%f\n", vgl->mpp_easting, 0.0, 0.0, vgl->mpp_northing, vgl->corner.easting, vgl->corner.northing );
       fclose ( f );
-      f = NULL;
     }
   }
   else
-   gtk_widget_destroy ( file_selector ); 
+    gtk_widget_destroy ( file_selector );
 }
 
 /**
@@ -828,10 +868,7 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
 
   /* Default to reject as user really needs to specify map file first */
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
-  GtkWidget *response_w = NULL;
-#if GTK_CHECK_VERSION (2, 20, 0)
-  response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
-#endif
+  GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
   GtkWidget *table, *wfp_hbox, *wfp_label, *wfp_button, *ce_label, *cn_label, *xlabel, *ylabel, *imagelabel;
   changeable_widgets cw;
 
@@ -1002,7 +1039,7 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
       a_settings_set_integer ( VIK_SETTINGS_GEOREF_TAB, gtk_notebook_get_current_page(GTK_NOTEBOOK(cw.tabs)) );
 
       if ( resp == GTK_RESPONSE_APPLY ) {
-        vik_layer_emit_update ( VIK_LAYER(vgl) );
+        vik_layer_emit_update ( VIK_LAYER(vgl), TRUE );
         answer = FALSE;
       }
       else
@@ -1017,7 +1054,7 @@ static void georef_layer_zoom_to_fit ( gpointer vgl_vlp[2] )
 {
   vik_viewport_set_xmpp ( vik_layers_panel_get_viewport(VIK_LAYERS_PANEL(vgl_vlp[1])), VIK_GEOREF_LAYER(vgl_vlp[0])->mpp_easting );
   vik_viewport_set_ympp ( vik_layers_panel_get_viewport(VIK_LAYERS_PANEL(vgl_vlp[1])), VIK_GEOREF_LAYER(vgl_vlp[0])->mpp_northing );
-  vik_layers_panel_emit_update ( VIK_LAYERS_PANEL(vgl_vlp[1]) );
+  vik_layers_panel_emit_update ( VIK_LAYERS_PANEL(vgl_vlp[1]), FALSE );
 }
 
 static void georef_layer_goto_center ( gpointer vgl_vlp[2] )
@@ -1035,7 +1072,7 @@ static void georef_layer_goto_center ( gpointer vgl_vlp[2] )
   vik_coord_load_from_utm ( &coord, vik_viewport_get_coord_mode ( vp ), &utm );
   vik_viewport_set_center_coord ( vp, &coord, TRUE );
 
-  vik_layers_panel_emit_update ( VIK_LAYERS_PANEL(vgl_vlp[1]) );
+  vik_layers_panel_emit_update ( VIK_LAYERS_PANEL(vgl_vlp[1]), FALSE );
 }
 
 static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp )
@@ -1057,19 +1094,19 @@ static gpointer georef_layer_move_create ( VikWindow *vw, VikViewport *vvp)
   return vvp;
 }
 
-static gboolean georef_layer_move_release ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
+static VikLayerToolFuncStatus georef_layer_move_release ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
 {
   if (!vgl || vgl->vl.type != VIK_LAYER_GEOREF)
-    return FALSE;
+    return VIK_LAYER_TOOL_IGNORED;
 
   if ( vgl->click_x != -1 )
   {
     vgl->corner.easting += (event->x - vgl->click_x) * vik_viewport_get_xmpp (vvp);
     vgl->corner.northing -= (event->y - vgl->click_y) * vik_viewport_get_ympp (vvp);
-    vik_layer_emit_update ( VIK_LAYER(vgl) );
-    return TRUE;
+    vik_layer_emit_update ( VIK_LAYER(vgl), TRUE );
+    return VIK_LAYER_TOOL_ACK;
   }
-  return FALSE; /* I didn't move anything on this layer! */
+  return VIK_LAYER_TOOL_IGNORED; // I didn't move anything on this layer!
 }
 
 static gpointer georef_layer_zoom_create ( VikWindow *vw, VikViewport *vvp)
@@ -1077,10 +1114,10 @@ static gpointer georef_layer_zoom_create ( VikWindow *vw, VikViewport *vvp)
   return vvp;
 }
 
-static gboolean georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
+static VikLayerToolFuncStatus georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
 {
   if (!vgl || vgl->vl.type != VIK_LAYER_GEOREF)
-    return FALSE;
+    return VIK_LAYER_TOOL_IGNORED;
   if ( event->button == 1 )
   {
     if ( vgl->mpp_easting < (VIK_VIEWPORT_MAX_ZOOM / 1.05) && vgl->mpp_northing < (VIK_VIEWPORT_MAX_ZOOM / 1.05) )
@@ -1099,17 +1136,17 @@ static gboolean georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *e
   }
   vik_viewport_set_xmpp ( vvp, vgl->mpp_easting );
   vik_viewport_set_ympp ( vvp, vgl->mpp_northing );
-  vik_layer_emit_update ( VIK_LAYER(vgl) );
-  return TRUE;
+  vik_layer_emit_update ( VIK_LAYER(vgl), FALSE );
+  return VIK_LAYER_TOOL_ACK;
 }
 
-static gboolean georef_layer_move_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
+static VikLayerToolFuncStatus georef_layer_move_press ( VikGeorefLayer *vgl, GdkEventButton *event, VikViewport *vvp )
 {
   if (!vgl || vgl->vl.type != VIK_LAYER_GEOREF)
-    return FALSE;
+    return VIK_LAYER_TOOL_IGNORED;
   vgl->click_x = event->x;
   vgl->click_y = event->y;
-  return TRUE;
+  return VIK_LAYER_TOOL_ACK;
 }
 
 static void goto_center_ll ( VikViewport *vp,
