@@ -22,7 +22,6 @@
  */
 #include "viking.h"
 #include "vikgpslayer.h"
-#include "icons/icons.h"
 #include "babel.h"
 #include "viktrwlayer.h"
 
@@ -33,10 +32,13 @@
 #include <gps.h>
 #endif
 
+#define GPS_FIXED_NAME "GPS"
+
 static VikGpsLayer *vik_gps_layer_create (VikViewport *vp);
 static void vik_gps_layer_realize ( VikGpsLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter );
 static void vik_gps_layer_free ( VikGpsLayer *vgl );
 static void vik_gps_layer_draw ( VikGpsLayer *vgl, VikViewport *vp );
+static void vik_gps_layer_configure ( VikGpsLayer *vgl, VikViewport *vp );
 static VikGpsLayer *vik_gps_layer_new ( VikViewport *vp );
 static void vik_gps_layer_post_read ( VikGpsLayer *vgl, VikViewport *vp, gboolean from_file );
 
@@ -61,6 +63,10 @@ static void gps_start_stop_tracking_cb( gpointer layer_and_vlp[2] );
 static void realtime_tracking_draw(VikGpsLayer *vgl, VikViewport *vp);
 static void rt_gpsd_disconnect(VikGpsLayer *vgl);
 static gboolean rt_gpsd_connect(VikGpsLayer *vgl, gboolean ask_if_failed);
+static VikLayerParamData color_default_tri ( void ) {
+  VikLayerParamData data; gdk_color_parse ( "#203070", &data.c ); return data;
+}
+static void layer_update_indictor_gc (VikGpsLayer *vgl, VikViewport *vp);
 #endif
 
 // Shouldn't need to use these much any more as the protocol is now saved as a string.
@@ -107,11 +113,11 @@ typedef struct {
   gboolean result;
   gchar *info;
   // GUI Updates
-  gint id_status_working;
-  gint id_status_end;
-  gint id_info;
-  gint id_total_count;
-  gint id_count;
+  guint id_status_working;
+  guint id_status_end;
+  guint id_info;
+  guint id_total_count;
+  guint id_count;
   VikViewport *vvp;
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
   gboolean realtime_tracking;
@@ -195,6 +201,16 @@ static VikLayerParamData gpsd_retry_interval_default ( void )
 
 #endif
 
+static void reset_cb ( GtkWidget *widget, gpointer ptr )
+{
+  a_layer_defaults_reset_show ( GPS_FIXED_NAME, ptr, GROUP_DATA_MODE );
+#if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
+  a_layer_defaults_reset_show ( GPS_FIXED_NAME, ptr, GROUP_REALTIME_MODE );
+#endif
+}
+
+static VikLayerParamData reset_default ( void ) { return VIK_LPD_PTR(reset_cb); }
+
 static VikLayerParam gps_layer_params[] = {
  //	NB gps_layer_inst_init() is performed after parameter registeration
  //  thus to give the protocols some potential values use the old static list
@@ -211,12 +227,15 @@ static VikLayerParam gps_layer_params[] = {
   { VIK_LAYER_GPS, "record_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Recording tracks"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
   { VIK_LAYER_GPS, "center_start_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Jump to current position on start"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_false_default, NULL, NULL },
   { VIK_LAYER_GPS, "moving_map_method", VIK_LAYER_PARAM_UINT, GROUP_REALTIME_MODE, N_("Moving Map Method:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_vehicle_position, NULL, NULL, moving_map_method_default, NULL, NULL },
+  { VIK_LAYER_GPS, "indicator_color", VIK_LAYER_PARAM_COLOR, GROUP_REALTIME_MODE, N_("Indicator Color:"), VIK_LAYER_WIDGET_COLOR, NULL, NULL, NULL, color_default_tri, NULL, NULL },
   { VIK_LAYER_GPS, "realtime_update_statusbar", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Update Statusbar:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("Display information in the statusbar on GPS updates"), vik_lpd_true_default, NULL, NULL },
   { VIK_LAYER_GPS, "auto_connect", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Auto Connect"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("Automatically connect to GPSD"), vik_lpd_false_default, NULL, NULL },
   { VIK_LAYER_GPS, "gpsd_host", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Host:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_host_default, NULL, NULL },
   { VIK_LAYER_GPS, "gpsd_port", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Port:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_port_default, NULL, NULL },
   { VIK_LAYER_GPS, "gpsd_retry_interval", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Retry Interval (seconds):"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_retry_interval_default, NULL, NULL },
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
+  { VIK_LAYER_GPS, "reset", VIK_LAYER_PARAM_PTR_DEFAULT, VIK_LAYER_GROUP_NONE, NULL,
+    VIK_LAYER_WIDGET_BUTTON, N_("Reset to Defaults"), NULL, NULL, reset_default, NULL, NULL },
 };
 enum {
   PARAM_PROTOCOL=0, PARAM_PORT,
@@ -227,19 +246,21 @@ enum {
   PARAM_REALTIME_REC,
   PARAM_REALTIME_CENTER_START,
   PARAM_VEHICLE_POSITION,
+  PARAM_INDICATOR_COLOR,
   PARAM_REALTIME_UPDATE_STATUSBAR,
   PARAM_GPSD_CONNECT,
   PARAM_GPSD_HOST,
   PARAM_GPSD_PORT,
   PARAM_GPSD_RETRY_INTERVAL,
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
+  PARAM_RESET,
   NUM_PARAMS};
 
 VikLayerInterface vik_gps_layer_interface = {
-  "GPS",
+  GPS_FIXED_NAME,
   N_("GPS"),
   NULL,
-  &vikgpslayer_pixbuf,
+  "vikgpslayer", // Icon name
 
   NULL,
   0,
@@ -258,6 +279,7 @@ VikLayerInterface vik_gps_layer_interface = {
 
   (VikLayerFuncProperties)              NULL,
   (VikLayerFuncDraw)                    vik_gps_layer_draw,
+  (VikLayerFuncConfigure)               vik_gps_layer_configure,
   (VikLayerFuncChangeCoordMode)         gps_layer_change_coord_mode,
 
   (VikLayerFuncGetTimestamp)            NULL,
@@ -273,6 +295,7 @@ VikLayerInterface vik_gps_layer_interface = {
   (VikLayerFuncSublayerTooltip)         NULL,
   (VikLayerFuncLayerTooltip)            gps_layer_tooltip,
   (VikLayerFuncLayerSelected)           NULL,
+  (VikLayerFuncLayerToggleVisible)      NULL,
 
   (VikLayerFuncMarshall)		gps_layer_marshall,
   (VikLayerFuncUnmarshall)		gps_layer_unmarshall,
@@ -345,10 +368,11 @@ struct _VikGpsLayer {
   guint realtime_retry_timer;
   GdkGC *realtime_track_gc;
   GdkGC *realtime_track_bg_gc;
-  GdkGC *realtime_track_pt_gc;
   GdkGC *realtime_track_pt1_gc;
   GdkGC *realtime_track_pt2_gc;
-
+  GdkColor realtime_track_bg_color;
+  GdkColor realtime_track_pt1_color;
+  GdkColor realtime_track_pt2_color;
   /* params */
   gboolean auto_connect_to_gpsd;
   gchar *gpsd_host;
@@ -357,6 +381,7 @@ struct _VikGpsLayer {
   gboolean realtime_record;
   gboolean realtime_jump_to_start;
   guint vehicle_position;
+  GdkColor indicator_color;
   gboolean realtime_update_statusbar;
   VikTrackpoint *trkpt;
   VikTrackpoint *trkpt_prev;
@@ -378,9 +403,9 @@ static void gps_layer_inst_init ( VikGpsLayer *self )
 {
   gint new_proto = 0;
   // +1 for luck (i.e the NULL terminator)
-  gchar **new_protocols = g_malloc_n(1 + g_list_length(a_babel_device_list), sizeof(gpointer));
+  gchar **new_protocols = g_malloc_n(1 + g_list_length(a_babel_device_list_get()), sizeof(gpointer));
 
-  GList *gl = g_list_first ( a_babel_device_list );
+  GList *gl = g_list_first ( a_babel_device_list_get() );
   while ( gl ) {
     // should be using label property but use name for now
     //  thus don't need to mess around converting label to name later on
@@ -521,10 +546,12 @@ static VikGpsLayer *gps_layer_unmarshall( guint8 *data, guint len, VikViewport *
 
 static gboolean gps_layer_set_param ( VikGpsLayer *vgl, VikLayerSetParam *vlsp )
 {
+  gboolean changed = FALSE;
   switch ( vlsp->id )
   {
     case PARAM_PROTOCOL:
       if (vlsp->data.s) {
+        gchar* old = g_strdup ( vgl->protocol );
         g_free(vgl->protocol);
         // Backwards Compatibility: previous versions <v1.4 stored protocol as an array index
         int index = vlsp->data.s[0] - '0';
@@ -536,6 +563,8 @@ static gboolean gps_layer_set_param ( VikGpsLayer *vgl, VikLayerSetParam *vlsp )
           vgl->protocol = g_strdup(protocols_args[index]);
         else
           vgl->protocol = g_strdup(vlsp->data.s);
+        changed = g_strcmp0 ( old, vgl->protocol );
+        g_free ( old );
         g_debug("%s: %s", __FUNCTION__, vgl->protocol);
       }
       else
@@ -543,6 +572,7 @@ static gboolean gps_layer_set_param ( VikGpsLayer *vgl, VikLayerSetParam *vlsp )
       break;
     case PARAM_PORT:
       if (vlsp->data.s) {
+        gchar* old = g_strdup ( vgl->serial_port );
         g_free(vgl->serial_port);
         // Backwards Compatibility: previous versions <v0.9.91 stored serial_port as an array index
         int index = vlsp->data.s[0] - '0';
@@ -554,68 +584,71 @@ static gboolean gps_layer_set_param ( VikGpsLayer *vgl, VikLayerSetParam *vlsp )
           vgl->serial_port = g_strdup(old_params_ports[index]);
         else
           vgl->serial_port = g_strdup(vlsp->data.s);
+        changed = g_strcmp0 ( old, vgl->serial_port );
+        g_free ( old );
         g_debug("%s: %s", __FUNCTION__, vgl->serial_port);
       }
       else
         g_warning(_("Unknown serial port device"));
       break;
     case PARAM_DOWNLOAD_TRACKS:
-      vgl->download_tracks = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->download_tracks );
       break;
     case PARAM_UPLOAD_TRACKS:
-      vgl->upload_tracks = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->upload_tracks );
       break;
     case PARAM_DOWNLOAD_ROUTES:
-      vgl->download_routes = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->download_routes );
       break;
     case PARAM_UPLOAD_ROUTES:
-      vgl->upload_routes = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->upload_routes );
       break;
     case PARAM_DOWNLOAD_WAYPOINTS:
-      vgl->download_waypoints = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->download_waypoints );
       break;
     case PARAM_UPLOAD_WAYPOINTS:
-      vgl->upload_waypoints = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->upload_waypoints );
       break;
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
     case PARAM_GPSD_CONNECT:
-      vgl->auto_connect_to_gpsd = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->auto_connect_to_gpsd );
       break;
     case PARAM_GPSD_HOST:
-      if (vlsp->data.s) {
-        if (vgl->gpsd_host)
-          g_free(vgl->gpsd_host);
-        vgl->gpsd_host = g_strdup(vlsp->data.s);
-      }
+      changed = vik_layer_param_change_string ( vlsp->data, &vgl->gpsd_host );
       break;
     case PARAM_GPSD_PORT:
-      if (vlsp->data.s) {
-        if (vgl->gpsd_port)
-          g_free(vgl->gpsd_port);
-        vgl->gpsd_port = g_strdup(vlsp->data.s);
-      }
+      changed = vik_layer_param_change_string ( vlsp->data, &vgl->gpsd_port );
       break;
-    case PARAM_GPSD_RETRY_INTERVAL:
+    case PARAM_GPSD_RETRY_INTERVAL: {
+      gint old = vgl->gpsd_retry_interval;
       vgl->gpsd_retry_interval = strtol(vlsp->data.s, NULL, 10);
+      changed = (old != vgl->gpsd_retry_interval);
       break;
+    }
     case PARAM_REALTIME_REC:
-      vgl->realtime_record = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->realtime_record );
       break;
     case PARAM_REALTIME_CENTER_START:
-      vgl->realtime_jump_to_start = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->realtime_jump_to_start );
       break;
     case PARAM_VEHICLE_POSITION:
-      vgl->vehicle_position = vlsp->data.u;
+      changed = vik_layer_param_change_uint ( vlsp->data, &vgl->vehicle_position );
+      break;
+    case PARAM_INDICATOR_COLOR:
+      changed = vik_layer_param_change_color ( vlsp->data, &vgl->indicator_color );
+      // Apply setting
+      if ( vgl->realtime_track_gc )
+        layer_update_indictor_gc ( vgl, vlsp->vp );
       break;
     case PARAM_REALTIME_UPDATE_STATUSBAR:
-      vgl->realtime_update_statusbar = vlsp->data.b;
+      changed = vik_layer_param_change_boolean ( vlsp->data, &vgl->realtime_update_statusbar );
       break;
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
-    default:
-      g_warning("gps_layer_set_param(): unknown parameter");
+    default: break;
   }
-
-  return TRUE;
+  if ( vik_debug && changed )
+    g_debug ( "%s: Detected change on param %d", __FUNCTION__, vlsp->id );
+  return changed;
 }
 
 static VikLayerParamData gps_layer_get_param ( VikGpsLayer *vgl, guint16 id, gboolean is_file_operation )
@@ -671,17 +704,29 @@ static VikLayerParamData gps_layer_get_param ( VikGpsLayer *vgl, guint16 id, gbo
     case PARAM_VEHICLE_POSITION:
       rv.u = vgl->vehicle_position;
       break;
+    case PARAM_INDICATOR_COLOR:
+      rv.c = vgl->indicator_color;
+      break;
     case PARAM_REALTIME_UPDATE_STATUSBAR:
-      rv.u = vgl->realtime_update_statusbar;
+      rv.b = vgl->realtime_update_statusbar;
       break;
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
-    default:
-      g_warning(_("%s: unknown parameter"), __FUNCTION__);
+    case PARAM_RESET: rv.ptr = reset_cb; break;
+    default: break;
   }
 
   return rv;
 }
 
+#if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
+static void gcs_create ( VikGpsLayer *vgl, VikViewport *vp )
+{
+  vgl->realtime_track_bg_gc = vik_viewport_new_gc ( vp, "grey", 2 );
+  vgl->realtime_track_pt1_gc = vik_viewport_new_gc ( vp, "red", 2 );
+  vgl->realtime_track_pt2_gc = vik_viewport_new_gc ( vp, "green", 2 );
+}
+#endif
+
 VikGpsLayer *vik_gps_layer_new (VikViewport *vp)
 {
   gint i;
@@ -702,11 +747,11 @@ VikGpsLayer *vik_gps_layer_new (VikViewport *vp)
   vgl->realtime_io_watch_id = 0;
   vgl->realtime_retry_timer = 0;
   if ( vp ) {
-    vgl->realtime_track_gc = vik_viewport_new_gc ( vp, "#203070", 2 );
-    vgl->realtime_track_bg_gc = vik_viewport_new_gc ( vp, "grey", 2 );
-    vgl->realtime_track_pt1_gc = vik_viewport_new_gc ( vp, "red", 2 );
-    vgl->realtime_track_pt2_gc = vik_viewport_new_gc ( vp, "green", 2 );
-    vgl->realtime_track_pt_gc = vgl->realtime_track_pt1_gc;
+    layer_update_indictor_gc ( vgl, vp );
+    gdk_color_parse ( "grey", &vgl->realtime_track_bg_color );
+    gdk_color_parse ( "red", &vgl->realtime_track_pt1_color );
+    gdk_color_parse ( "green", &vgl->realtime_track_pt2_color );
+    gcs_create ( vgl, vp );
   }
   vgl->realtime_track = NULL;
 #endif // VIK_CONFIG_REALTIME_GPS_TRACKING
@@ -722,6 +767,10 @@ static void vik_gps_layer_post_read ( VikGpsLayer *vgl, VikViewport *vvp, gboole
     trw_layer_calculate_bounds_waypoints ( vgl->trw_children[i] );
     trw_layer_calculate_bounds_tracks ( vgl->trw_children[i] );
   }
+  
+#if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
+  layer_update_indictor_gc ( vgl, vvp );
+#endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
 }
 
 static void vik_gps_layer_draw ( VikGpsLayer *vgl, VikViewport *vp )
@@ -759,6 +808,28 @@ static void vik_gps_layer_draw ( VikGpsLayer *vgl, VikViewport *vp )
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
 }
 
+#if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
+static void gcs_free ( VikGpsLayer *vgl )
+{
+  if (vgl->realtime_track_gc != NULL)
+    ui_gc_unref(vgl->realtime_track_gc);
+  if (vgl->realtime_track_bg_gc != NULL)
+    ui_gc_unref(vgl->realtime_track_bg_gc);
+  if (vgl->realtime_track_pt1_gc != NULL)
+    ui_gc_unref(vgl->realtime_track_pt1_gc);
+  if (vgl->realtime_track_pt2_gc != NULL)
+    ui_gc_unref(vgl->realtime_track_pt2_gc);
+}
+#endif
+
+static void vik_gps_layer_configure ( VikGpsLayer *vgl, VikViewport *vp )
+{
+#if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
+  gcs_free ( vgl );
+  gcs_create ( vgl, vp );
+#endif
+}
+
 static void gps_layer_change_coord_mode ( VikGpsLayer *vgl, VikCoordMode mode )
 {
   gint i;
@@ -805,6 +876,7 @@ static void disconnect_layer_signal ( VikLayer *vl, VikGpsLayer *vgl )
 
 static void vik_gps_layer_free ( VikGpsLayer *vgl )
 {
+  g_list_free(vgl->children);
   gint i;
   for (i = 0; i < NUM_TRW; i++) {
     if (vgl->vl.realized)
@@ -813,15 +885,12 @@ static void vik_gps_layer_free ( VikGpsLayer *vgl )
   }
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
   rt_gpsd_disconnect(vgl);
-  if (vgl->realtime_track_gc != NULL)
-    g_object_unref(vgl->realtime_track_gc);
-  if (vgl->realtime_track_bg_gc != NULL)
-    g_object_unref(vgl->realtime_track_bg_gc);
-  if (vgl->realtime_track_pt1_gc != NULL)
-    g_object_unref(vgl->realtime_track_pt1_gc);
-  if (vgl->realtime_track_pt2_gc != NULL)
-    g_object_unref(vgl->realtime_track_pt2_gc);
+  gcs_free(vgl);
+  g_free ( vgl->gpsd_host );
+  g_free ( vgl->gpsd_port );
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
+  g_free ( vgl->protocol );
+  g_free ( vgl->serial_port );
 }
 
 static void vik_gps_layer_realize ( VikGpsLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter )
@@ -1351,7 +1420,7 @@ static void gps_comm_thread(GpsSession *sess)
           vik_layer_post_read ( VIK_LAYER(sess->vtl), sess->vvp, TRUE );
           /* View the data available */
           vik_trw_layer_auto_set_view ( sess->vtl, sess->vvp ) ;
-          vik_layer_emit_update ( VIK_LAYER(sess->vtl) ); // NB update request from background thread
+          vik_layer_emit_update ( VIK_LAYER(sess->vtl), TRUE ); // NB update request from background thread
         }
       }
     } else {
@@ -1470,11 +1539,7 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
 
     gtk_dialog_set_default_response ( GTK_DIALOG(sess->dialog), GTK_RESPONSE_ACCEPT );
     // Starting gps read/write thread
-#if GLIB_CHECK_VERSION (2, 32, 0)
     g_thread_try_new ( "gps_comm_thread", (GThreadFunc)gps_comm_thread, sess, NULL );
-#else
-    g_thread_create ( (GThreadFunc)gps_comm_thread, sess, FALSE, NULL );
-#endif
 
     gtk_dialog_run(GTK_DIALOG(dialog));
   }
@@ -1648,12 +1713,16 @@ static void realtime_tracking_draw(VikGpsLayer *vgl, VikViewport *vp)
      GdkPoint trian[3] = { { pt_x, pt_y }, {side1_x, side1_y}, {side2_x, side2_y} };
      GdkPoint trian_bg[3] = { { ptbg_x, pt_y }, {side1bg_x, side1bg_y}, {side2bg_x, side2bg_y} };
 
-     vik_viewport_draw_polygon ( vp, vgl->realtime_track_bg_gc, TRUE, trian_bg, 3 );
-     vik_viewport_draw_polygon ( vp, vgl->realtime_track_gc, TRUE, trian, 3 );
-     vik_viewport_draw_rectangle ( vp,
-         (vgl->realtime_fix.fix.mode > MODE_2D) ? vgl->realtime_track_pt2_gc : vgl->realtime_track_pt1_gc,
-         TRUE, x-2, y-2, 4, 4 );
-     //vgl->realtime_track_pt_gc = (vgl->realtime_track_pt_gc == vgl->realtime_track_pt1_gc) ? vgl->realtime_track_pt2_gc : vgl->realtime_track_pt1_gc;
+     vik_viewport_draw_polygon ( vp, vgl->realtime_track_bg_gc, TRUE, trian_bg, 3, &vgl->realtime_track_bg_color );
+     vik_viewport_draw_polygon ( vp, vgl->realtime_track_gc, TRUE, trian, 3, &vgl->indicator_color );
+     if ( vgl->realtime_fix.fix.mode > MODE_2D )
+       vik_viewport_draw_rectangle ( vp,
+                                     vgl->realtime_track_pt2_gc,
+                                     TRUE, x-2, y-2, 4, 4, &vgl->realtime_track_pt2_color );
+     else
+       vik_viewport_draw_rectangle ( vp,
+                                     vgl->realtime_track_pt1_gc,
+                                     TRUE, x-2, y-2, 4, 4, &vgl->realtime_track_pt1_color );
   }
 }
 
@@ -1680,8 +1749,16 @@ static VikTrackpoint* create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forc
       gboolean replace = FALSE;
       int heading = isnan(vgl->realtime_fix.fix.track) ? 0 : (int)floor(vgl->realtime_fix.fix.track);
       int last_heading = isnan(vgl->last_fix.fix.track) ? 0 : (int)floor(vgl->last_fix.fix.track);
+#if GPSD_API_MAJOR_VERSION >= 9
+      // We currently support one altitude value, whereas GPSD has shifted to HAE & MSL options
+      //  (Height Above Ellipsoid & Mean Sea Level)
+      // and the original 'altitude' is apparently undefined
+      int alt = isnan(vgl->realtime_fix.fix.altHAE) ? 0 : (int)floor(vgl->realtime_fix.fix.altHAE);
+      int last_alt = isnan(vgl->last_fix.fix.altHAE) ? 0 : (int)floor(vgl->last_fix.fix.altHAE);
+#else
       int alt = isnan(vgl->realtime_fix.fix.altitude) ? 0 : (int)floor(vgl->realtime_fix.fix.altitude);
       int last_alt = isnan(vgl->last_fix.fix.altitude) ? 0 : (int)floor(vgl->last_fix.fix.altitude);
+#endif
       if (((last_tp = g_list_last(vgl->realtime_track->trackpoints)) != NULL) &&
           (vgl->realtime_fix.fix.mode > MODE_2D) &&
           (vgl->last_fix.fix.mode <= MODE_2D) &&
@@ -1721,6 +1798,7 @@ static VikTrackpoint* create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forc
         vgl->realtime_fix.dirty = FALSE;
         vgl->realtime_fix.satellites_used = 0;
         vgl->last_fix = vgl->realtime_fix;
+
         return tp;
       }
     }
@@ -1808,7 +1886,7 @@ static void gpsd_raw_hook(VglGpsd *vgpsd, gchar *data)
       vgl->trkpt_prev = vgl->trkpt;
     }
 
-    vik_layer_emit_update ( update_all ? VIK_LAYER(vgl) : VIK_LAYER(vgl->trw_children[TRW_REALTIME]) ); // NB update from background thread
+    vik_layer_emit_update ( update_all ? VIK_LAYER(vgl) : VIK_LAYER(vgl->trw_children[TRW_REALTIME]), vgl->trkpt ? TRUE : FALSE ); // NB update from background thread
   }
 }
 
@@ -2047,4 +2125,11 @@ static void gps_start_stop_tracking_cb( gpointer layer_and_vlp[2])
     rt_gpsd_disconnect(vgl);
   }
 }
+
+static void layer_update_indictor_gc (VikGpsLayer *vgl, VikViewport *vp)
+{
+  if ( vgl->realtime_track_gc )
+    ui_gc_unref ( vgl->realtime_track_gc );
+  vgl->realtime_track_gc = vik_viewport_new_gc_from_color ( vp, &(vgl->indicator_color), 2 );
+}
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
