@@ -36,7 +36,7 @@ extern VikLayerInterface vik_dem_layer_interface;
 #ifdef HAVE_LIBMAPNIK
 extern VikLayerInterface vik_mapnik_layer_interface;
 #endif
-#ifdef HAVE_LIBMAPNIK
+#ifdef HAVE_LIBGEOCLUE_2
 extern VikLayerInterface vik_geoclue_layer_interface;
 #endif
 
@@ -128,9 +128,10 @@ void vik_layer_redraw ( VikLayer *vl )
 
 /**
  * An update event
+ * @is_modified: Whether the layer has been modified
  * Redraw specified layer and a notification about the update
  */
-void vik_layer_emit_update ( VikLayer *vl )
+void vik_layer_emit_update ( VikLayer *vl, gboolean is_modified )
 {
   if ( vl->visible && vl->realized ) {
     GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
@@ -159,6 +160,12 @@ void vik_layer_emit_update ( VikLayer *vl )
 	(void)g_idle_add ( (GSourceFunc)vik_layer_interfaces[vl->type]->refresh, vl );
     }
   }
+
+  // Check for realized (similar to as above) is mainly to detect layers that in the process of being created
+  // As the widget doesn't exist yet, we can't get the owning vikwindow
+  // The modified status for new layers is generally handled in viklayerspanel instead.
+  if ( is_modified && vl->realized )
+    vik_window_set_modified ( (VikWindow *)(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
 }
 
 /**
@@ -302,6 +309,13 @@ void vik_layer_draw ( VikLayer *l, VikViewport *vp )
       vik_layer_interfaces[l->type]->draw ( l, vp );
 }
 
+void vik_layer_configure ( VikLayer *l, VikViewport *vp )
+{
+  if ( l->visible )
+    if ( vik_layer_interfaces[l->type]->configure )
+      vik_layer_interfaces[l->type]->configure ( l, vp );
+}
+
 void vik_layer_change_coord_mode ( VikLayer *l, VikCoordMode mode )
 {
   if ( vik_layer_interfaces[l->type]->change_coord_mode )
@@ -507,6 +521,14 @@ gboolean vik_layer_selected ( VikLayer *l, gint subtype, gpointer sublayer, gint
   return vik_window_clear_selected ( (VikWindow *)VIK_GTK_WINDOW_FROM_LAYER(l) );
 }
 
+// NB the layer->visible value has already been modified
+// This function acts to notify the layer of this event
+void vik_layer_layer_toggle_visible ( VikLayer *l )
+{
+  if ( vik_layer_interfaces[l->type]->layer_toggle_visible )
+    vik_layer_interfaces[l->type]->layer_toggle_visible ( l );
+}
+
 void vik_layer_realize ( VikLayer *l, VikTreeview *vt, GtkTreeIter *layer_iter )
 {
   l->vt = vt;
@@ -565,19 +587,30 @@ const gchar* vik_layer_layer_tooltip ( VikLayer *l )
   return NULL;
 }
 
+/**
+ * vik_layer_load_icon:
+ *
+ * Unref the returned pixbuf after use
+ */
 GdkPixbuf *vik_layer_load_icon ( VikLayerTypeEnum type )
 {
   g_assert ( type < VIK_LAYER_NUM_TYPES );
   if ( vik_layer_interfaces[type]->icon )
-    return gdk_pixbuf_from_pixdata ( vik_layer_interfaces[type]->icon, FALSE, NULL );
+    return ui_get_icon ( vik_layer_interfaces[type]->icon, 16 );
   return NULL;
 }
 
 gboolean vik_layer_set_param ( VikLayer *vl, VikLayerSetParam *vlsp )
 {
+  gboolean ans = FALSE;
   if ( vik_layer_interfaces[vl->type]->set_param )
-    return vik_layer_interfaces[vl->type]->set_param ( vl, vlsp );
-  return FALSE;
+    ans = vik_layer_interfaces[vl->type]->set_param ( vl, vlsp );
+  if ( ans && !vlsp->is_file_operation && vl->realized ) {
+    VikWindow *vw = (VikWindow*)VIK_GTK_WINDOW_FROM_LAYER(vl);
+    if ( vw )
+      vik_window_set_modified ( vw );
+  }
+  return ans;
 }
 
 void vik_layer_post_read ( VikLayer *layer, VikViewport *vp, gboolean from_file )
@@ -586,10 +619,9 @@ void vik_layer_post_read ( VikLayer *layer, VikViewport *vp, gboolean from_file
     vik_layer_interfaces[layer->type]->post_read ( layer, vp, from_file );
 }
 
-// Wrapper function to keep compiler happy
 static void layer_emit_update_internal ( gpointer gp )
 {
-  vik_layer_emit_update ( VIK_LAYER(gp));
+  vik_layer_emit_update ( VIK_LAYER(gp), TRUE );
 }
 
 static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gboolean have_apply )
@@ -600,7 +632,7 @@ static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gb
 					    vik_layer_interfaces[vl->type]->params_count,
 					    vik_layer_interfaces[vl->type]->params_groups,
 					    vik_layer_interfaces[vl->type]->params_groups_count,
-					    (gpointer) vik_layer_interfaces[vl->type]->set_param,
+					    (gpointer) vik_layer_set_param,
 					    NULL,
 					    vl, 
 					    vp,
@@ -609,12 +641,12 @@ static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gb
 					    (gpointer) vik_layer_interfaces[vl->type]->change_param,
 					    have_apply,
 					    layer_emit_update_internal,
-					    (gpointer)vl) ) {
+					    (gpointer)vl,
+					    have_apply) ) {
     case 0:
-    case 3:
       return FALSE;
-      /* redraw (?) */
     case 2:
+    case 3:
       vik_layer_post_read ( vl, vp, FALSE ); /* update any gc's */
     default:
       return TRUE;
@@ -727,7 +759,11 @@ void vik_layer_set_defaults ( VikLayer *vl, VikViewport *vvp )
       if ( vli->params[i].type != VIK_LAYER_PARAM_STRING_LIST ) {
         vlsp.data = a_layer_defaults_get ( layer_name, vli->params[i].name, vli->params[i].type );
         (void)vik_layer_set_param ( vl, &vlsp );
+        if ( vli->params[i].type == VIK_LAYER_PARAM_STRING )
+          g_free ( (gchar*)vlsp.data.s );
       }
+      else
+        g_debug ( "%s: No files list default", __FUNCTION__ );
     }
   }
 }
@@ -739,3 +775,60 @@ void vik_layer_expand_tree ( VikLayer *vl )
 {
   vik_treeview_expand ( vl->vt, &vl->iter );
 }
+
+/**
+ * Returns whether the new value is an actual change
+ */
+gboolean vik_layer_param_change_boolean ( VikLayerParamData val, gboolean *current )
+{
+  gboolean old = *current;
+  *current = val.b;
+  return (old != val.b);
+}
+
+gboolean vik_layer_param_change_uint ( VikLayerParamData val, guint *current )
+{
+  guint old = *current;
+  *current = val.u;
+  return (old != val.u);
+}
+
+gboolean vik_layer_param_change_uint8 ( VikLayerParamData val, guint8 *current )
+{
+  guint8 old = *current;
+  *current = val.u;
+  return (old != val.u);
+}
+
+gboolean vik_layer_param_change_int ( VikLayerParamData val, gint *current )
+{
+  gint old = *current;
+  *current = val.i;
+  return (old != val.i);
+}
+
+gboolean vik_layer_param_change_color ( VikLayerParamData val, GdkColor *current )
+{
+  GdkColor old = *current;
+  *current = val.c;
+  return !gdk_color_equal ( &old, &val.c );
+}
+
+// Doubles may be in some kind of units - metres/feet etc...
+//  and in that case should not use this function directly
+gboolean vik_layer_param_change_double ( VikLayerParamData val, gdouble *current )
+{
+  gdouble old = *current;
+  *current = val.d;
+  return util_gdouble_different ( old, val.d );
+}
+
+gboolean vik_layer_param_change_string ( VikLayerParamData val, gchar **current )
+{
+  gchar* old = g_strdup ( *current );
+  g_free ( *current );
+  *current = g_strdup ( val.s );
+  gboolean changed = g_strcmp0 ( old, val.s );
+  g_free ( old );
+  return changed;
+}
