@@ -91,7 +91,7 @@ typedef gboolean (*VikToolKeyFunc) (VikLayer *, GdkEventKey *, gpointer);
 
 typedef struct _VikToolInterface VikToolInterface;
 struct _VikToolInterface {
-  const GdkPixdata *icon;
+  const gchar *icon;
   GtkRadioActionEntry radioActionEntry;
   VikToolConstructorFunc create;
   VikToolDestructorFunc destroy;
@@ -104,7 +104,7 @@ struct _VikToolInterface {
   VikToolKeyFunc key_release; /* as above */
   gboolean pan_handler; // Call click & release funtions even when 'Pan Mode' is on
   GdkCursorType cursor_type;
-  const GdkPixdata *cursor_data;
+  const gchar *cursor_name;
   const GdkCursor *cursor;
 };
 
@@ -132,6 +132,7 @@ typedef void          (*VikLayerFuncFree)                  (VikLayer *);
 typedef gboolean      (*VikLayerFuncProperties)            (VikLayer *,VikViewport *, gboolean); // gboolean is for using an apply button
 
 typedef void          (*VikLayerFuncDraw)                  (VikLayer *,VikViewport *);
+typedef void          (*VikLayerFuncConfigure)             (VikLayer *,VikViewport *); // 'configure-event' events
 typedef void          (*VikLayerFuncChangeCoordMode)       (VikLayer *,VikCoordMode);
 
 typedef void          (*VikLayerFuncSetMenuItemsSelection)          (VikLayer *,guint16);
@@ -146,6 +147,7 @@ typedef gboolean      (*VikLayerFuncSublayerToggleVisible) (VikLayer *,gint,gpoi
 typedef const gchar * (*VikLayerFuncSublayerTooltip)       (VikLayer *,gint,gpointer);
 typedef const gchar * (*VikLayerFuncLayerTooltip)          (VikLayer *);
 typedef gboolean      (*VikLayerFuncLayerSelected)         (VikLayer *,gint,gpointer,gint,gpointer); /* 2nd gpointer is a VikLayersPanel */
+typedef void          (*VikLayerFuncLayerToggleVisible)    (VikLayer *);
 
 typedef void          (*VikLayerFuncMarshall)              (VikLayer *, guint8 **, guint *);
 typedef VikLayer *    (*VikLayerFuncUnmarshall)            (guint8 *, guint, VikViewport *);
@@ -179,7 +181,7 @@ typedef void 	      (*VikLayerFuncDragDropRequest)       (VikLayer *, VikLayer *
 typedef gboolean      (*VikLayerFuncSelectClick)           (VikLayer *, GdkEventButton *, VikViewport *, tool_ed_t*);
 typedef gboolean      (*VikLayerFuncSelectMove)            (VikLayer *, GdkEventMotion *, VikViewport *, tool_ed_t*);
 typedef gboolean      (*VikLayerFuncSelectRelease)         (VikLayer *, GdkEventButton *, VikViewport *, tool_ed_t*);
-typedef gboolean      (*VikLayerFuncSelectedViewportMenu)  (VikLayer *, guint, VikViewport *);
+typedef gboolean      (*VikLayerFuncSelectedViewportMenu)  (VikLayer *, GdkEventButton *, VikViewport *); // GdkEventButton may be NULL
 
 typedef double        (*VikLayerFuncGetTimestamp)          (VikLayer *);
 
@@ -204,7 +206,7 @@ struct _VikLayerInterface {
   const gchar *                     fixed_layer_name; // Used in .vik files - this should never change to maintain file compatibility
   const gchar *                     name;             // Translate-able name used for display purposes
   const gchar *                     accelerator;
-  const GdkPixdata *                icon;
+  const gchar *                     icon;
 
   VikToolInterface *                tools;
   guint16                           tools_count;
@@ -225,6 +227,7 @@ struct _VikLayerInterface {
 
   VikLayerFuncProperties            properties;
   VikLayerFuncDraw                  draw;
+  VikLayerFuncConfigure             configure;
   VikLayerFuncChangeCoordMode       change_coord_mode;
 
   VikLayerFuncGetTimestamp          get_timestamp;
@@ -239,6 +242,7 @@ struct _VikLayerInterface {
   VikLayerFuncSublayerTooltip       sublayer_tooltip;
   VikLayerFuncLayerTooltip          layer_tooltip;
   VikLayerFuncLayerSelected         layer_selected;
+  VikLayerFuncLayerToggleVisible    layer_toggle_visible;
 
   VikLayerFuncMarshall              marshall;
   VikLayerFuncUnmarshall            unmarshall;
@@ -273,6 +277,7 @@ VikLayerInterface *vik_layer_get_interface ( VikLayerTypeEnum type );
 
 void vik_layer_set_type ( VikLayer *vl, VikLayerTypeEnum type );
 void vik_layer_draw ( VikLayer *l, VikViewport *vp );
+void vik_layer_configure ( VikLayer *l, VikViewport *vp );
 void vik_layer_change_coord_mode ( VikLayer *l, VikCoordMode mode );
 void vik_layer_rename ( VikLayer *l, const gchar *new_name );
 void vik_layer_rename_no_copy ( VikLayer *l, gchar *new_name );
@@ -284,7 +289,7 @@ gboolean vik_layer_set_param ( VikLayer *vl, VikLayerSetParam *vlsp );
 
 void vik_layer_set_defaults ( VikLayer *vl, VikViewport *vvp );
 
-void vik_layer_emit_update ( VikLayer *vl );
+void vik_layer_emit_update ( VikLayer *vl, gboolean is_modified );
 
 void vik_layer_redraw ( VikLayer *vl );
 
@@ -315,6 +320,8 @@ const gchar* vik_layer_layer_tooltip ( VikLayer *l );
 
 gboolean vik_layer_selected ( VikLayer *l, gint subtype, gpointer sublayer, gint type, gpointer vlp );
 
+void vik_layer_layer_toggle_visible ( VikLayer *l );
+
 /* TODO: put in layerspanel */
 GdkPixbuf *vik_layer_load_icon ( VikLayerTypeEnum type );
 
@@ -335,6 +342,14 @@ void vik_layer_typed_param_data_free ( gpointer gp );
 VikLayerTypedParamData *vik_layer_typed_param_data_copy_from_data ( VikLayerParamType type, VikLayerParamData val );
 VikLayerTypedParamData *vik_layer_data_typed_param_copy_from_string ( VikLayerParamType type, const gchar *str );
 
+gboolean vik_layer_param_change_boolean ( VikLayerParamData val, gboolean *current );
+gboolean vik_layer_param_change_uint ( VikLayerParamData val, guint *current );
+gboolean vik_layer_param_change_uint8 ( VikLayerParamData val, guint8 *current );
+gboolean vik_layer_param_change_int ( VikLayerParamData val, gint *current );
+gboolean vik_layer_param_change_color ( VikLayerParamData val, GdkColor *current );
+gboolean vik_layer_param_change_double ( VikLayerParamData val, gdouble *current );
+gboolean vik_layer_param_change_string ( VikLayerParamData val, gchar **current );
+
 G_END_DECLS
 
 #endif
