@@ -21,6 +21,8 @@
  */
 #include "viking.h"
 #include <gdk/gdkkeysyms.h>
+#include "vikgoto.h"
+#include "viktrwlayer_propwin.h"
 
 enum {
   VLP_UPDATE_SIGNAL,
@@ -42,15 +44,25 @@ static guint layers_panel_signals[VLP_LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class;
 
 struct _VikLayersPanel {
+#if GTK_CHECK_VERSION (3,0,0)
+  GtkBox vbox;
+#else
   GtkVBox vbox;
+#endif
   GtkWidget *hbox;
+  GtkWidget *vpaned;
+  GtkWidget *tabs;
   GtkWidget *calendar;
-  // Could use gtk_widget_get_visible () instead, but needs Gtk2.18
-  // ATM still using Gtk2.16 - so manually track the state for now
-  // NB Doesn't consider if the layers panel is shown or not.
-  gboolean cal_shown;
   calendar_mu_t cal_markup;
 
+  GtkWidget *goto_pane;
+  GtkWidget *stats_pane; GtkWidget *stats_label;
+  GtkWidget *splits_pane; GtkWidget *splits_label;
+
+  // Effectively vik_trw_and_track_t:
+  VikTrwLayer *vtl;
+  VikTrack *trk;
+
   VikAggregateLayer *toplayer;
   GtkTreeIter toplayer_iter;
 
@@ -91,11 +103,26 @@ static void vik_layers_panel_class_init ( VikLayersPanelClass *klass )
 
   layers_panel_signals[VLP_UPDATE_SIGNAL] = g_signal_new ( "update", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayersPanelClass, update), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   layers_panel_signals[VLP_DELETE_LAYER_SIGNAL] = g_signal_new ( "delete_layer", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayersPanelClass, update), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-  }
+}
+
+#define VIK_LAYERS_PANEL_TABS_ORDER "layers_panel_tabs_order"
+
+// Enable hook into destroy sequence to enable saving tab order
+static void destroy_panel ( VikLayersPanel *vlp, gpointer data )
+{
+  gint vals[4];
+  vals[0] = gtk_notebook_page_num ( GTK_NOTEBOOK(vlp->tabs), vlp->calendar );
+  vals[1] = gtk_notebook_page_num ( GTK_NOTEBOOK(vlp->tabs), vlp->goto_pane );
+  vals[2] = gtk_notebook_page_num ( GTK_NOTEBOOK(vlp->tabs), vlp->stats_pane );
+  vals[3] = gtk_notebook_page_num ( GTK_NOTEBOOK(vlp->tabs), vlp->splits_pane );
+  a_settings_set_integer_list ( VIK_LAYERS_PANEL_TABS_ORDER, vals, G_N_ELEMENTS(vals) );
+}
 
 VikLayersPanel *vik_layers_panel_new ()
 {
-  return VIK_LAYERS_PANEL ( g_object_new ( VIK_LAYERS_PANEL_TYPE, NULL ) );
+  VikLayersPanel *vlp = VIK_LAYERS_PANEL ( g_object_new ( VIK_LAYERS_PANEL_TYPE, NULL ) );
+  g_signal_connect ( G_OBJECT(vlp), "destroy", G_CALLBACK(destroy_panel), NULL );
+  return vlp;
 }
 
 void vik_layers_panel_set_viewport ( VikLayersPanel *vlp, VikViewport *vvp )
@@ -114,6 +141,8 @@ static gboolean layers_panel_new_layer ( gpointer lpnl[2] )
   return vik_layers_panel_new_layer ( lpnl[0], GPOINTER_TO_INT(lpnl[1]) );
 }
 
+static void layers_panel_emit_update ( VikLayersPanel *vlp );
+
 /**
  * Create menu popup on demand
  * @full: offer cut/copy options as well - not just the new layer options
@@ -132,8 +161,7 @@ static GtkWidget* layers_panel_create_popup ( VikLayersPanel *vlp, gboolean full
   }
 
   GtkWidget *submenu = gtk_menu_new();
-  menuitem = gtk_menu_item_new_with_mnemonic ( _("New Layer") );
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  menuitem = vu_menu_add_item ( GTK_MENU(menu), _("New Layer"), GTK_STOCK_NEW, NULL, NULL );
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu );
 
   // Static: so memory accessible yet not continually allocated
@@ -143,7 +171,9 @@ static GtkWidget* layers_panel_create_popup ( VikLayersPanel *vlp, gboolean full
 
     if ( vik_layer_get_interface(ii)->icon ) {
       menuitem = gtk_image_menu_item_new_with_mnemonic ( _(vik_layer_get_interface(ii)->name) );
-      gtk_image_menu_item_set_image ( (GtkImageMenuItem*)menuitem, gtk_image_new_from_pixbuf ( vik_layer_load_icon (ii) ) );
+      GdkPixbuf *pixbuf = vik_layer_load_icon (ii);
+      gtk_image_menu_item_set_image ( (GtkImageMenuItem*)menuitem, gtk_image_new_from_pixbuf ( pixbuf ) );
+      g_object_unref ( pixbuf );
     }
     else
       menuitem = gtk_menu_item_new_with_mnemonic ( _(vik_layer_get_interface(ii)->name) );
@@ -164,6 +194,13 @@ static void calendar_mark_layer_in_month ( VikLayersPanel *vlp, VikTrwLayer *vtl
   guint year, month, day;
   gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
   GDate *gd = g_date_new();
+  // Beginning and end of month values as date strings in ISO8601 format
+  gchar *ds1 = g_strdup_printf ( "%d-%02d-%02dT00:00:00", year, month+1, 1 );
+  gchar *ds2 = NULL;
+  if ( month < 11 )
+    ds2 = g_strdup_printf ( "%d-%02d-%02dT00:00:00", year, month+2, 1 );
+  else
+    ds2 = g_strdup_printf ( "%d-%02d-%02dT00:00:00", year+1, 1, 1 );
   GHashTable *trks = vik_trw_layer_get_tracks ( vtl ); 
   GHashTableIter iter;
   gpointer key, value;
@@ -171,17 +208,45 @@ static void calendar_mark_layer_in_month ( VikLayersPanel *vlp, VikTrwLayer *vtl
   g_hash_table_iter_init ( &iter, trks );
   while ( g_hash_table_iter_next ( &iter, &key, &value ) ) {
     VikTrack *trk = VIK_TRACK(value);
-    // First trackpoint
     if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
       // Not worried about subsecond resolution here!
       g_date_set_time_t ( gd, (time_t)VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
-      // Is in selected month?
+      // Is first trackpoint in selected month?
       if ( g_date_get_year(gd) == year && (g_date_get_month(gd) == (month+1)) ) {
 	gtk_calendar_mark_day ( GTK_CALENDAR(vlp->calendar), g_date_get_day(gd) );
-	break;
+      }
+      // Now scan rest of track...
+      GTimeVal tv1, tv2;
+      // See also g_date_time_new_from_iso8601() but glib 2.56 needed
+      // NB the time val for each is the beginning of day,
+      //  so even as ds2 is on the next month - it's effectively equivalent to the end of the previous month
+      if ( g_time_val_from_iso8601(ds1, &tv1) && g_time_val_from_iso8601(ds2, &tv2) ) {
+        GList *tp_iter = trk->trackpoints;
+        guint count = 0;
+        // Step through the track to mark all the potential days that the track might cover
+        // Check every 100th point in order to speed up processing (also include the very last point too)
+        while ( tp_iter ) {
+          count++;
+          if ( count % 100 == 0 || tp_iter->next == NULL ) {
+            VikTrackpoint *tpt = VIK_TRACKPOINT(tp_iter->data);
+            if ( !isnan(tpt->timestamp) &&
+                 tpt->timestamp < tv2.tv_sec &&
+                 tpt->timestamp > tv1.tv_sec ) {
+              g_date_set_time_t ( gd, (time_t)tpt->timestamp );
+              if ( g_date_get_year(gd) == year && (g_date_get_month(gd) == (month+1)) )
+#if GTK_CHECK_VERSION(3,0,0)
+                if ( !gtk_calendar_get_day_is_marked(GTK_CALENDAR(vlp->calendar), g_date_get_day(gd)) )
+#endif
+                  gtk_calendar_mark_day ( GTK_CALENDAR(vlp->calendar), g_date_get_day(gd) );
+            }
+          }
+            tp_iter = tp_iter->next;
+        }  
       }
     }
   }
+  g_free ( ds1 );
+  g_free ( ds2 );
   g_date_free ( gd );
 }
 
@@ -191,7 +256,7 @@ static void calendar_mark_layer_in_month ( VikLayersPanel *vlp, VikTrwLayer *vtl
 void layers_panel_calendar_update ( VikLayersPanel *vlp )
 {
   // Skip if not shown...
-  if ( !vlp->cal_shown )
+  if ( !gtk_widget_get_visible(GTK_WIDGET(vlp)) )
     return;
 
   gtk_calendar_clear_marks ( GTK_CALENDAR(vlp->calendar) );
@@ -199,13 +264,12 @@ void layers_panel_calendar_update ( VikLayersPanel *vlp )
   GList *layers = vik_layers_panel_get_all_layers_of_type ( vlp, VIK_LAYER_TRW, TRUE );
   if ( !layers )
     return;
-  
-  while ( layers ) {
-    VikTrwLayer *vtl = VIK_TRW_LAYER(layers->data);
+
+  for ( GList *layer = layers; layer != NULL; layer = layer->next ) {
+    VikTrwLayer *vtl = VIK_TRW_LAYER(layer->data);
     calendar_mark_layer_in_month ( vlp, vtl );
-    layers = g_list_next ( layers );
   }
-  g_list_free ( layers );    
+  g_list_free ( layers );
 }
 
 /**
@@ -222,11 +286,13 @@ void vik_layers_panel_calendar_update ( VikLayersPanel *vlp )
   double time_spent = (double)(end - begin) / CLOCKS_PER_SEC;
   g_debug ( "%s: %f", __FUNCTION__, time_spent );
   // Downgrade automatic detail if taking too long
-  // Since ATM the detail method invokes x30 searches,
+  // Since ATM the detail method may invoke upto x30 searches,
   //  it could get noticably slow
   if ( vlp->cal_markup == VLP_CAL_MU_AUTO )
-    if ( time_spent > (0.5/30.0) )
+    if ( time_spent > (0.5/30.0) ) {
+      g_message ( "%s: detail level reduced as taking too long (%.5fs)", __FUNCTION__, time_spent );
       vlp->cal_markup = VLP_CAL_MU_MARKED;
+    }
 }
 
 static void layers_calendar_day_selected_dc_cb ( VikLayersPanel *vlp )
@@ -242,7 +308,7 @@ static void layers_calendar_day_selected_dc_cb ( VikLayersPanel *vlp )
 /**
  *
  */
-gboolean layers_calendar_forward ( VikLayersPanel *vlp )
+static gboolean layers_calendar_forward ( VikLayersPanel *vlp )
 {
   guint year, month, day;
   gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
@@ -273,6 +339,10 @@ gboolean layers_calendar_forward ( VikLayersPanel *vlp )
     gtk_calendar_select_day ( GTK_CALENDAR(vlp->calendar), day );
     gtk_calendar_select_month ( GTK_CALENDAR(vlp->calendar), month-1, year );
   } else {
+    if ( !date_str ) {
+      date_str = g_malloc0_n ( sizeof(gchar*), 64 );
+      (void)g_date_strftime ( date_str, 64, "%x", gd_now );
+    }
     a_dialog_info_msg_extra ( GTK_WINDOW(VIK_WINDOW_FROM_WIDGET(vlp)), _("Nothing found up to %s"), date_str );
   }
   g_debug ( "%s: %s %d", __FUNCTION__, date_str, found );
@@ -285,7 +355,7 @@ gboolean layers_calendar_forward ( VikLayersPanel *vlp )
 /**
  *
  */
-gboolean layers_calendar_back ( VikLayersPanel *vlp )
+static gboolean layers_calendar_back ( VikLayersPanel *vlp )
 {
   guint year, month, day;
   gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
@@ -322,7 +392,7 @@ gboolean layers_calendar_back ( VikLayersPanel *vlp )
 /**
  *
  */
-gboolean layers_calendar_today ( VikLayersPanel *vlp )
+static gboolean layers_calendar_today ( VikLayersPanel *vlp )
 {
   vu_calendar_set_to_today ( vlp->calendar );
   return TRUE;
@@ -353,7 +423,7 @@ static gchar *calendar_detail ( GtkCalendar *calendar,
 {
   VikLayersPanel *vlp = (VikLayersPanel*)user_data;
   // Skip if not shown...
-  if ( !vlp->cal_shown )
+  if ( !gtk_widget_get_visible(GTK_WIDGET(vlp)) )
     return NULL;
 
   // Check that detail is wanted
@@ -368,45 +438,55 @@ static gchar *calendar_detail ( GtkCalendar *calendar,
   if ( amonth != month )
     return NULL;
 
-  GList *layers = vik_layers_panel_get_all_layers_of_type ( vlp, VIK_LAYER_TRW, TRUE );
-  if ( !layers )
-    return NULL;
-
-  gboolean need_to_break = FALSE;
   VikTrwLayer *vtl = NULL;
-  GDate *gd = g_date_new();
-  while ( layers ) {
-    vtl = VIK_TRW_LAYER(layers->data);
 
-    GHashTable *trks = vik_trw_layer_get_tracks ( vtl ); 
-    GHashTableIter iter;
-    gpointer key, value;
-    // Foreach Track
-    g_hash_table_iter_init ( &iter, trks );
-    while ( g_hash_table_iter_next ( &iter, &key, &value ) ) {
-      VikTrack *trk = VIK_TRACK(value);
-      // First trackpoint
-      if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
-        // Not worried about subsecond resolution here!
-        g_date_set_time_t ( gd, (time_t)VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
-        // Is of this day?
-        if ( g_date_get_year(gd) == year &&
-             g_date_get_month(gd) == (month+1) &&
-             g_date_get_day(gd) == day ) {
-          need_to_break = TRUE;
-	  break;
-	}
+  // By performing a check against only those days already marked we
+  //  can skip needless full searches - as it would not find any matches
+#if GTK_CHECK_VERSION(3,0,0)
+  if ( gtk_calendar_get_day_is_marked(calendar, day) )
+#endif
+  {
+
+    GList *layers = vik_layers_panel_get_all_layers_of_type ( vlp, VIK_LAYER_TRW, TRUE );
+    if ( !layers )
+      return NULL;
+
+    gboolean need_to_break = FALSE;
+    GDate *gd = g_date_new();
+    for ( GList *layer = layers; layer != NULL; layer = layer->next ) {
+      vtl = VIK_TRW_LAYER(layer->data);
+
+      GHashTable *trks = vik_trw_layer_get_tracks ( vtl );
+      GHashTableIter iter;
+      gpointer key, value;
+      // Foreach Track
+      g_hash_table_iter_init ( &iter, trks );
+      while ( g_hash_table_iter_next ( &iter, &key, &value ) ) {
+        VikTrack *trk = VIK_TRACK(value);
+        // First trackpoint
+        if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
+          // Not worried about subsecond resolution here!
+          g_date_set_time_t ( gd, (time_t)VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
+          // Is of this day?
+          if ( g_date_get_year(gd) == year &&
+               g_date_get_month(gd) == (month+1) &&
+               g_date_get_day(gd) == day ) {
+            need_to_break = TRUE;
+            break;
+          }
+        }
+        // Note that this does not attempt to analyse the tracks for multi day support
+        //  in order to keep this tooltip function as fast as possible
       }
-    }
-    // Fully exit
-    if ( need_to_break )
-      break;
+      // Fully exit
+      if ( need_to_break )
+        break;
     
-    layers = g_list_next ( layers );
-    vtl = NULL;
+      vtl = NULL;
+    }
+    g_date_free ( gd );
+    g_list_free ( layers );
   }
-  g_date_free ( gd );
-  g_list_free ( layers );    
 
   if ( vtl)
     return g_strdup (vik_layer_get_name ( VIK_LAYER(vtl) ));
@@ -414,7 +494,27 @@ static gchar *calendar_detail ( GtkCalendar *calendar,
   return NULL;
 }
 
+static gboolean track_tabs_refresh_cb ( VikLayersPanel *vlp )
+{
+  if ( vlp->trk )
+    vik_layers_panel_track_add ( vlp, vlp->trk, (gpointer)vlp->vtl );
+  return TRUE;
+}
+
+static gboolean track_tabs_button_press_cb ( VikLayersPanel *vlp, GdkEventButton *event )
+{
+  if ( event->button == 3 ) {
+    GtkMenu *menu = GTK_MENU ( gtk_menu_new () );
+    (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_REFRESH, G_CALLBACK(track_tabs_refresh_cb), vlp );
+    gtk_widget_show_all ( GTK_WIDGET(menu) );
+    gtk_menu_popup ( menu, NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
+    return TRUE;
+  }
+  return FALSE;
+}
+
 #define VIK_SETTINGS_CAL_MUM "layers_panel_calendar_markup_mode"
+#define VIK_SETTINGS_LP_VPANE_POS "layers_panel_vpane_position"
 
 static void vik_layers_panel_init ( VikLayersPanel *vlp )
 {
@@ -427,6 +527,11 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
   GtkWidget *pastebutton, *pasteimage;
   GtkWidget *scrolledwindow;
 
+#if GTK_CHECK_VERSION (3,0,0)
+  // Force vertical mode for box (as default is horizontal in GTK3)
+  gtk_orientable_set_orientation ( GTK_ORIENTABLE(vlp), GTK_ORIENTATION_VERTICAL );
+#endif
+
   vlp->vvp = NULL;
 
   vlp->hbox = gtk_hbox_new ( TRUE, 2 );
@@ -434,7 +539,7 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
 
   vlp->toplayer = vik_aggregate_layer_new ();
   vik_layer_rename ( VIK_LAYER(vlp->toplayer), _("Top Layer"));
-  g_signal_connect_swapped ( G_OBJECT(vlp->toplayer), "update", G_CALLBACK(vik_layers_panel_emit_update), vlp );
+  g_signal_connect_swapped ( G_OBJECT(vlp->toplayer), "update", G_CALLBACK(layers_panel_emit_update), vlp );
 
   vik_treeview_add_layer ( vlp->vt, NULL, &(vlp->toplayer_iter), VIK_LAYER(vlp->toplayer)->name, NULL, TRUE, vlp->toplayer, VIK_LAYER_AGGREGATE, VIK_LAYER_AGGREGATE, 0 );
   vik_layer_realize ( VIK_LAYER(vlp->toplayer), vlp->vt, &(vlp->toplayer_iter) );
@@ -516,10 +621,9 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
   // (and then possibly this results in selecting a track) which would be counter intuitive
   g_signal_connect_swapped ( vlp->calendar, "day-selected-double-click", G_CALLBACK(layers_calendar_day_selected_dc_cb), vlp);
   g_signal_connect_swapped ( vlp->calendar, "button-press-event", G_CALLBACK(layers_calendar_button_press_cb), vlp );
-  vlp->cal_shown = TRUE;
 
   // Ensure any detail results in a tooltip rather than embedded in the calendar display
-  GValue sd = { 0 };
+  GValue sd = G_VALUE_INIT;
   g_value_init ( &sd, G_TYPE_BOOLEAN );
   g_value_set_boolean ( &sd, FALSE );
   g_object_set_property ( G_OBJECT(vlp->calendar), "show-details", &sd );
@@ -527,8 +631,61 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
   gtk_calendar_set_detail_func ( GTK_CALENDAR(vlp->calendar), calendar_detail, vlp, NULL );
   vik_layers_panel_set_preferences ( vlp );
   
-  gtk_box_pack_start ( GTK_BOX(vlp), scrolledwindow, TRUE, TRUE, 0 );
-  gtk_box_pack_start ( GTK_BOX(vlp), vlp->calendar, FALSE, FALSE, 0 );
+  vlp->vpaned = gtk_vpaned_new ();
+  vlp->tabs = gtk_notebook_new();
+
+  gtk_notebook_append_page ( GTK_NOTEBOOK(vlp->tabs), vlp->calendar, gtk_label_new(_("Calendar")) );
+  gtk_notebook_set_tab_reorderable ( GTK_NOTEBOOK(vlp->tabs), vlp->calendar, TRUE );
+
+  vlp->goto_pane = vik_goto_panel_widget ( vlp );
+  gtk_notebook_append_page ( GTK_NOTEBOOK(vlp->tabs), vlp->goto_pane, gtk_label_new(_("Goto")) );
+  gtk_notebook_set_tab_reorderable ( GTK_NOTEBOOK(vlp->tabs), vlp->goto_pane, TRUE );
+
+  vlp->stats_pane = gtk_scrolled_window_new ( NULL, NULL );
+  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(vlp->stats_pane), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+  // Enable right click menu on the stats & splits tab
+  // ATM These don't auto refresh on Track modification (e.g. split, trackpoint add, etc...)
+  //  so at least offer a manual method for now
+  vlp->stats_label = gtk_label_new ( _("Stats") );
+  GtkWidget *stats_eventbox = gtk_event_box_new ();
+  gtk_container_add ( GTK_CONTAINER(stats_eventbox), vlp->stats_label );
+  gtk_widget_show ( vlp->stats_label );
+  g_signal_connect_swapped ( stats_eventbox, "button-press-event", G_CALLBACK(track_tabs_button_press_cb), vlp ); 
+  gtk_widget_set_sensitive ( vlp->stats_label, FALSE );
+  gtk_notebook_append_page ( GTK_NOTEBOOK(vlp->tabs), vlp->stats_pane, stats_eventbox );
+  gtk_notebook_set_tab_reorderable ( GTK_NOTEBOOK(vlp->tabs), vlp->stats_pane, TRUE );
+
+  vlp->splits_pane = gtk_scrolled_window_new ( NULL, NULL );
+  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(vlp->splits_pane), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+  vlp->splits_label = gtk_label_new ( _("Splits") );
+  GtkWidget *splits_eventbox = gtk_event_box_new ();
+  gtk_container_add ( GTK_CONTAINER(splits_eventbox), vlp->splits_label );
+  gtk_widget_show ( vlp->splits_label );
+  g_signal_connect_swapped ( splits_eventbox, "button-press-event", G_CALLBACK(track_tabs_button_press_cb), vlp ); 
+  gtk_widget_set_sensitive ( vlp->splits_label, FALSE );
+  gtk_notebook_append_page ( GTK_NOTEBOOK(vlp->tabs), vlp->splits_pane, splits_eventbox );
+  gtk_notebook_set_tab_reorderable ( GTK_NOTEBOOK(vlp->tabs), vlp->splits_pane, TRUE );
+
+  gint *vals;
+  gsize length;
+  if ( a_settings_get_integer_list(VIK_LAYERS_PANEL_TABS_ORDER, &vals, &length) ) {
+    for ( guint nn = 0; nn < length; nn++ ) {
+      switch ( nn ) {
+      case 0: gtk_notebook_reorder_child ( GTK_NOTEBOOK(vlp->tabs), vlp->calendar, vals[nn] ); break;
+      case 1: gtk_notebook_reorder_child ( GTK_NOTEBOOK(vlp->tabs), vlp->goto_pane, vals[nn] ); break;
+      case 2: gtk_notebook_reorder_child ( GTK_NOTEBOOK(vlp->tabs), vlp->stats_pane, vals[nn] ); break;
+      case 3: gtk_notebook_reorder_child ( GTK_NOTEBOOK(vlp->tabs), vlp->splits_pane, vals[nn] ); break;
+      default: break;
+      }
+    }
+    if ( vals )
+      g_free (vals);
+  }
+
+  gtk_paned_pack1 ( GTK_PANED(vlp->vpaned), scrolledwindow, TRUE, TRUE );
+  gtk_paned_pack2 ( GTK_PANED(vlp->vpaned), vlp->tabs, FALSE, TRUE );
+
+  gtk_box_pack_start ( GTK_BOX(vlp), vlp->vpaned, TRUE, TRUE, 0 );
   gtk_box_pack_start ( GTK_BOX(vlp), vlp->hbox, FALSE, FALSE, 0 );
 }
 
@@ -541,7 +698,7 @@ static gboolean idle_draw_panel ( VikLayersPanel *vlp )
   return FALSE; // Nothing else to do
 }
 
-void vik_layers_panel_emit_update ( VikLayersPanel *vlp )
+void layers_panel_emit_update ( VikLayersPanel *vlp )
 {
   GThread *thread = vik_window_get_thread (VIK_WINDOW(VIK_GTK_WINDOW_FROM_WIDGET(vlp)));
   if ( !thread )
@@ -556,6 +713,19 @@ void vik_layers_panel_emit_update ( VikLayersPanel *vlp )
     (void)g_idle_add ( (GSourceFunc)idle_draw_panel, vlp );
 }
 
+/**
+ * vik_layers_panel_emit_update:
+ * @is_modified: Whether a layer has been modified
+ *  i.e. for a change of view (e.g. center, zoom level, selected item, etc...) - this is not a change to any layer
+ *
+ */
+void vik_layers_panel_emit_update ( VikLayersPanel *vlp, gboolean is_modified )
+{
+  layers_panel_emit_update ( vlp );
+  if ( is_modified )
+    vik_window_set_modified ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_WIDGET(vlp)) );
+}
+
 static void layers_item_toggled (VikLayersPanel *vlp, GtkTreeIter *iter)
 {
   gboolean visible;
@@ -570,13 +740,16 @@ static void layers_item_toggled (VikLayersPanel *vlp, GtkTreeIter *iter)
   {
     case VIK_TREEVIEW_TYPE_LAYER:
       visible = (VIK_LAYER(p)->visible ^= 1);
+      vik_layer_layer_toggle_visible ( VIK_LAYER(p) );
+      if ( visible )
+        vik_layer_configure ( VIK_LAYER(p), vlp->vvp );
       vik_layer_emit_update_although_invisible ( VIK_LAYER(p) ); /* set trigger for half-drawn */
-      if ( !visible && VIK_LAYER(p)->type == VIK_LAYER_TRW )
-        vik_window_close_graphs ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(VIK_LAYER(p))) );
       break;
     case VIK_TREEVIEW_TYPE_SUBLAYER: {
       VikLayer *vl = VIK_LAYER(vik_treeview_item_get_parent ( vlp->vt, iter ));
       visible = vik_layer_sublayer_toggle_visible ( vl, vik_treeview_item_get_data(vlp->vt, iter), p);
+      if ( visible )
+        vik_layer_configure ( vl, vlp->vvp );
       vik_layer_emit_update_although_invisible ( vl );
       break;
     }
@@ -584,6 +757,11 @@ static void layers_item_toggled (VikLayersPanel *vlp, GtkTreeIter *iter)
   }
 
   vik_treeview_item_set_visible ( vlp->vt, iter, visible );
+
+  gboolean ignore_toggle = FALSE;
+  (void)a_settings_get_boolean ( VIK_SETTINGS_IGNORE_VIS_MOD, &ignore_toggle );
+  if ( !ignore_toggle )
+    vik_window_set_modified ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_WIDGET(vlp)) );
 }
 
 static void layers_item_edited (VikLayersPanel *vlp, GtkTreeIter *iter, const gchar *new_text)
@@ -607,13 +785,16 @@ static void layers_item_edited (VikLayersPanel *vlp, GtkTreeIter *iter, const gc
     {
       vik_layer_rename ( l, new_text );
       vik_treeview_item_set_name ( vlp->vt, iter, l->name );
+      vik_window_set_modified ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_WIDGET(vlp)) );
     }
   }
   else
   {
     const gchar *name = vik_layer_sublayer_rename_request ( vik_treeview_item_get_parent ( vlp->vt, iter ), new_text, vlp, vik_treeview_item_get_data ( vlp->vt, iter ), vik_treeview_item_get_pointer ( vlp->vt, iter ), iter );
-    if ( name )
+    if ( name ) {
       vik_treeview_item_set_name ( vlp->vt, iter, name);
+      vik_window_set_modified ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_WIDGET(vlp)) );
+    }
   }
 }
 
@@ -637,7 +818,7 @@ static gboolean layers_button_press_cb ( VikLayersPanel *vlp, GdkEventButton *ev
 static gboolean layers_key_press_cb ( VikLayersPanel *vlp, GdkEventKey *event )
 {
   // Accept all forms of delete keys
-  if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete || event->keyval == GDK_BackSpace) {
+  if (event->keyval == GDK_KEY_Delete || event->keyval == GDK_KEY_KP_Delete || event->keyval == GDK_KEY_BackSpace) {
     vik_layers_panel_delete_selected (vlp);
     return TRUE;
  }
@@ -792,7 +973,11 @@ void vik_layers_panel_add_layer ( VikLayersPanel *vlp, VikLayer *l )
       vik_aggregate_layer_add_layer ( addtoagg, l, TRUE );
   }
 
-  vik_layers_panel_emit_update ( vlp );
+  // Choose whether the new layer should be selected
+  if ( a_vik_get_select_newly_created_layer() )
+    vik_treeview_item_select ( l->vt, &l->iter );
+
+  vik_layers_panel_emit_update ( vlp, TRUE );
 }
 
 static void layers_move_item ( VikLayersPanel *vlp, gboolean up )
@@ -812,7 +997,7 @@ static void layers_move_item ( VikLayersPanel *vlp, gboolean up )
     if ( parent ) /* not toplevel */
     {
       vik_aggregate_layer_move_layer ( parent, &iter, up );
-      vik_layers_panel_emit_update ( vlp );
+      vik_layers_panel_emit_update ( vlp, TRUE );
     }
   }
 }
@@ -825,9 +1010,8 @@ gboolean vik_layers_panel_properties ( VikLayersPanel *vlp )
   if ( vik_treeview_get_selected_iter ( vlp->vt, &iter ) && vik_treeview_item_get_type ( vlp->vt, &iter ) == VIK_TREEVIEW_TYPE_LAYER )
   {
     VikLayer *layer = VIK_LAYER( vik_treeview_item_get_pointer ( vlp->vt, &iter ) );
-    if ( vik_layer_properties ( layer, vlp->vvp, TRUE ))
-      vik_layer_emit_update ( layer );
-    return TRUE;
+    // NB vik_layer_emit_update() is called by vik_layer_properties() if necessary
+    return vik_layer_properties ( layer, vlp->vvp, TRUE );
   }
   else
     return FALSE;
@@ -839,6 +1023,12 @@ void vik_layers_panel_draw_all ( VikLayersPanel *vlp )
     vik_aggregate_layer_draw ( vlp->toplayer, vlp->vvp );
 }
 
+void vik_layers_panel_configure_layers ( VikLayersPanel *vlp )
+{
+  if ( vlp->vvp && VIK_LAYER(vlp->toplayer)->visible )
+    vik_aggregate_layer_configure ( vlp->toplayer, vlp->vvp );
+}
+
 void vik_layers_panel_cut_selected ( VikLayersPanel *vlp )
 {
   gint type;
@@ -865,8 +1055,9 @@ void vik_layers_panel_cut_selected ( VikLayersPanel *vlp )
 
         g_signal_emit ( G_OBJECT(vlp), layers_panel_signals[VLP_DELETE_LAYER_SIGNAL], 0 );
 
-        if ( vik_aggregate_layer_delete ( parent, &iter ) )
-          vik_layers_panel_emit_update ( vlp );
+        if ( vik_aggregate_layer_delete ( parent, &iter ) ) {
+          vik_layers_panel_emit_update ( vlp, TRUE );
+        }
       }
     }
     else
@@ -930,8 +1121,9 @@ void vik_layers_panel_delete_selected ( VikLayersPanel *vlp )
 
         g_signal_emit ( G_OBJECT(vlp), layers_panel_signals[VLP_DELETE_LAYER_SIGNAL], 0 );
 
-        if ( vik_aggregate_layer_delete ( parent, &iter ) )
-	  vik_layers_panel_emit_update ( vlp );	
+        if ( vik_aggregate_layer_delete ( parent, &iter ) ) {
+	  vik_layers_panel_emit_update ( vlp, TRUE );
+        }
       }
     }
     else
@@ -1056,19 +1248,106 @@ void vik_layers_panel_show_buttons ( VikLayersPanel *vlp, gboolean show )
     gtk_widget_hide ( vlp->hbox );
 }
 
+void vik_layers_panel_show_tabs ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show ) {
+    vik_layers_panel_calendar_update ( vlp );
+    gtk_widget_show ( vlp->tabs );
+  }
+  else {
+    gtk_widget_hide ( vlp->tabs );
+  }
+}
+
 void vik_layers_panel_show_calendar ( VikLayersPanel *vlp, gboolean show )
 {
   if ( show ) {
-    vlp->cal_shown = TRUE;
     vik_layers_panel_calendar_update ( vlp );
     gtk_widget_show ( vlp->calendar );
   }
   else {
-    vlp->cal_shown = FALSE;
     gtk_widget_hide ( vlp->calendar );
   }
 }
 
+void vik_layers_panel_show_goto ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show ) {
+    gtk_widget_show ( vlp->goto_pane );
+  }
+  else {
+    gtk_widget_hide ( vlp->goto_pane );
+  }
+}
+
+void vik_layers_panel_show_stats ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show )
+    gtk_widget_show ( vlp->stats_pane );
+  else
+    gtk_widget_hide ( vlp->stats_pane );
+}
+
+void vik_layers_panel_show_splits ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show )
+    gtk_widget_show ( vlp->splits_pane );
+  else
+    gtk_widget_hide ( vlp->splits_pane );
+}
+
+
+static void remove_container_widgets ( GtkWidget *widget, gpointer data )
+{
+  gtk_widget_destroy ( widget );
+}
+
+static void layers_panel_stats_remove ( VikLayersPanel *vlp )
+{
+  gtk_container_foreach ( GTK_CONTAINER(vlp->stats_pane), (GtkCallback)remove_container_widgets, NULL );
+}
+
+static void layers_panel_splits_remove ( VikLayersPanel *vlp )
+{
+  gtk_container_foreach ( GTK_CONTAINER(vlp->splits_pane), (GtkCallback)remove_container_widgets, NULL );
+}
+
+/**
+ *
+ */
+void vik_layers_panel_track_add ( VikLayersPanel *vlp, VikTrack *trk, gpointer vtl )
+{
+  vlp->trk = trk;
+  vlp->vtl = VIK_TRW_LAYER(vtl);
+
+  layers_panel_stats_remove ( vlp );
+  (void)vik_trw_propwin_attach_statistics_table ( vlp->stats_pane, trk, vlp->vtl, TRUE );
+  if ( gtk_widget_get_visible(vlp->stats_pane) )
+    gtk_widget_show_all ( vlp->stats_pane );
+  gtk_widget_set_sensitive ( vlp->stats_label, TRUE );
+
+  layers_panel_splits_remove ( vlp );
+  if ( vik_track_get_duration(trk,FALSE) > 1 ) {
+    GtkWidget *widget = vik_trw_propwin_create_splits_tabs ( trk );
+    gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW(vlp->splits_pane), widget );
+    if ( gtk_widget_get_visible(vlp->splits_pane) )
+      gtk_widget_show_all ( vlp->splits_pane );
+    gtk_widget_set_sensitive ( vlp->splits_label, TRUE );
+  }
+}
+
+/**
+ *
+ */
+void vik_layers_panel_track_remove ( VikLayersPanel *vlp )
+{
+  vlp->trk = NULL;
+  layers_panel_stats_remove ( vlp );
+  layers_panel_splits_remove ( vlp );
+  gtk_widget_set_sensitive ( vlp->stats_label, FALSE );
+  gtk_widget_set_sensitive ( vlp->splits_label, FALSE );
+}
+
 /**
  *
  */
@@ -1093,7 +1372,7 @@ void vik_layers_panel_calendar_date ( VikLayersPanel *vlp, time_t timestamp )
  */
 void vik_layers_panel_set_preferences ( VikLayersPanel *vlp )
 {
-  GValue sd = { 0 };
+  GValue sd = G_VALUE_INIT;
   g_value_init ( &sd, G_TYPE_BOOLEAN );
   g_value_set_boolean ( &sd, a_vik_get_calendar_show_day_names() );
   g_object_set_property ( G_OBJECT(vlp->calendar), "show-day-names", &sd );
