@@ -28,6 +28,7 @@
 
 #include "viklayer.h"
 #include "vikaggregatelayer.h"
+#include "viktrack.h"
 
 G_BEGIN_DECLS
 
@@ -50,9 +51,9 @@ struct _VikLayersPanelClass
 
 GType vik_layers_panel_get_type ();
 VikLayersPanel *vik_layers_panel_new ();
-void vik_layers_panel_free ( VikLayersPanel *vlp );
 void vik_layers_panel_add_layer ( VikLayersPanel *vlp, VikLayer *l );
 void vik_layers_panel_draw_all ( VikLayersPanel *vlp );
+void vik_layers_panel_configure_layers ( VikLayersPanel *vlp );
 VikLayer *vik_layers_panel_get_selected ( VikLayersPanel *vlp );
 void vik_layers_panel_cut_selected ( VikLayersPanel *vlp );
 void vik_layers_panel_copy_selected ( VikLayersPanel *vlp );
@@ -62,7 +63,7 @@ VikLayer *vik_layers_panel_get_layer_of_type ( VikLayersPanel *vlp, VikLayerType
 void vik_layers_panel_set_viewport ( VikLayersPanel *vlp, VikViewport *vvp );
 //gboolean vik_layers_panel_tool ( VikLayersPanel *vlp, guint16 layer_type, VikToolInterfaceFunc tool_func, GdkEventButton *event, VikViewport *vvp );
 VikViewport *vik_layers_panel_get_viewport ( VikLayersPanel *vlp );
-void vik_layers_panel_emit_update ( VikLayersPanel *vlp );
+void vik_layers_panel_emit_update ( VikLayersPanel *vlp, gboolean is_modified );
 gboolean vik_layers_panel_properties ( VikLayersPanel *vlp );
 gboolean vik_layers_panel_new_layer ( VikLayersPanel *vlp, VikLayerTypeEnum type );
 void vik_layers_panel_clear ( VikLayersPanel *vlp );
@@ -71,7 +72,13 @@ void vik_layers_panel_change_coord_mode ( VikLayersPanel *vlp, VikCoordMode mode
 GList *vik_layers_panel_get_all_layers_of_type(VikLayersPanel *vlp, gint type, gboolean include_invisible);
 VikTreeview *vik_layers_panel_get_treeview ( VikLayersPanel *vlp );
 void vik_layers_panel_show_buttons ( VikLayersPanel *vlp, gboolean show );
+void vik_layers_panel_show_tabs ( VikLayersPanel *vlp, gboolean show );
 void vik_layers_panel_show_calendar ( VikLayersPanel *vlp, gboolean show );
+void vik_layers_panel_show_goto ( VikLayersPanel *vlp, gboolean show );
+void vik_layers_panel_show_stats ( VikLayersPanel *vlp, gboolean show );
+void vik_layers_panel_show_splits ( VikLayersPanel *vlp, gboolean show );
+void vik_layers_panel_track_add ( VikLayersPanel *vlp, VikTrack *trk, gpointer vtl );
+void vik_layers_panel_track_remove ( VikLayersPanel *vlp );
 void vik_layers_panel_calendar_update ( VikLayersPanel *vlp );
 void vik_layers_panel_calendar_today ( VikLayersPanel *vlp );
 void vik_layers_panel_calendar_date ( VikLayersPanel *vlp, time_t timestamp );
