@@ -27,13 +27,13 @@
 #include "mapcoord.h"
 #include "mapcache.h"
 #include "dir.h"
-#include "preferences.h"
-#include "icons/icons.h"
 #include "mapnik_interface.h"
 #include "background.h"
 
 #include "vikmapslayer.h"
 
+#define MAPNIK_FIXED_NAME "Mapnik Rendering"
+
 struct _VikMapnikLayerClass
 {
 	VikLayerClass object_class;
@@ -62,6 +62,13 @@ static VikLayerParamScale scales[] = {
 	{ 0, 1024, 12, 0 }, // Rerender timeout hours
 };
 
+static void reset_cb ( GtkWidget *widget, gpointer ptr )
+{
+	a_layer_defaults_reset_show ( MAPNIK_FIXED_NAME, ptr, VIK_LAYER_GROUP_NONE );
+}
+
+static VikLayerParamData reset_default ( void ) { return VIK_LPD_PTR(reset_cb); }
+
 VikLayerParam mapnik_layer_params[] = {
   { VIK_LAYER_MAPNIK, "config-file-mml", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CSS (MML) Config File:"), VIK_LAYER_WIDGET_FILEENTRY, GINT_TO_POINTER(VF_FILTER_CARTO), NULL,
     N_("CartoCSS configuration file"), file_default, NULL, NULL },
@@ -73,6 +80,8 @@ VikLayerParam mapnik_layer_params[] = {
     NULL, vik_lpd_true_default, NULL, NULL },
   { VIK_LAYER_MAPNIK, "file-cache-dir", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("File Cache Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL,
     NULL, cache_dir_default, NULL, NULL },
+  { VIK_LAYER_MAPNIK, "reset", VIK_LAYER_PARAM_PTR_DEFAULT, VIK_LAYER_GROUP_NONE, NULL,
+    VIK_LAYER_WIDGET_BUTTON, N_("Reset to Defaults"), NULL, NULL, reset_default, NULL, NULL },
 };
 
 enum {
@@ -81,12 +90,13 @@ enum {
   PARAM_ALPHA,
   PARAM_USE_FILE_CACHE,
   PARAM_FILE_CACHE_DIR,
+  PARAM_RESET,
   NUM_PARAMS };
 
 static const gchar* mapnik_layer_tooltip ( VikMapnikLayer *vml );
 static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len );
 static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp );
-static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation );
+static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, VikLayerSetParam *vlsp );
 static VikLayerParamData mapnik_layer_get_param ( VikMapnikLayer *vml, guint16 id, gboolean is_file_operation );
 static VikMapnikLayer *mapnik_layer_new ( VikViewport *vvp );
 static VikMapnikLayer *mapnik_layer_create ( VikViewport *vp );
@@ -99,7 +109,7 @@ static gpointer mapnik_feature_create ( VikWindow *vw, VikViewport *vvp)
   return vvp;
 }
 
-static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp );
+static VikLayerToolFuncStatus mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp );
 
 // See comment in viktrwlayer.c for advice on values used
 // FUTURE:
@@ -124,10 +134,10 @@ static VikToolInterface mapnik_tools[] = {
 static void mapnik_layer_post_read (VikLayer *vl, VikViewport *vvp, gboolean from_file);
 
 VikLayerInterface vik_mapnik_layer_interface = {
-	"Mapnik Rendering",
+	MAPNIK_FIXED_NAME,
 	N_("Mapnik Rendering"),
 	NULL,
-	&vikmapniklayer_pixbuf, // icon
+	"vikmapniklayer", // Icon name
 
 	mapnik_tools,
 	G_N_ELEMENTS(mapnik_tools),
@@ -146,6 +156,7 @@ VikLayerInterface vik_mapnik_layer_interface = {
 
 	(VikLayerFuncProperties)              NULL,
 	(VikLayerFuncDraw)                    mapnik_layer_draw,
+	(VikLayerFuncConfigure)               NULL,
 	(VikLayerFuncChangeCoordMode)         NULL,
 
 	(VikLayerFuncGetTimestamp)            NULL,
@@ -161,6 +172,7 @@ VikLayerInterface vik_mapnik_layer_interface = {
 	(VikLayerFuncSublayerTooltip)         NULL,
 	(VikLayerFuncLayerTooltip)            mapnik_layer_tooltip,
 	(VikLayerFuncLayerSelected)           NULL,
+	(VikLayerFuncLayerToggleVisible)      NULL,
 
 	(VikLayerFuncMarshall)                mapnik_layer_marshall,
 	(VikLayerFuncUnmarshall)              mapnik_layer_unmarshall,
@@ -243,14 +255,22 @@ static VikLayerParamData fonts_default ( void )
 	return data;
 }
 
+static VikLayerParamData carto_default ( void ) {
+  VikLayerParamData data;
+  data.s = g_strdup ( "carto" );
+  return data;
+}
+
+static VikLayerParamData rr_to_default ( void ) { return VIK_LPD_UINT(168); } // One week in hours
+
 static VikLayerParam prefs[] = {
 	// Changing these values only applies before first mapnik layer is 'created'
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"plugins_directory", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Plugins Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), plugins_default, NULL, NULL },
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"fonts_directory", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Fonts Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), fonts_default, NULL, NULL },
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"recurse_fonts_directory", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Recurse Fonts Directory:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), vik_lpd_true_default, NULL, NULL },
-	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"rerender_after", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Rerender Timeout (hours):"), VIK_LAYER_WIDGET_SPINBUTTON, &scales[2], NULL, N_("You need to restart Viking for a change to this value to be used"), NULL, NULL, NULL },
+	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"rerender_after", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Rerender Timeout (hours):"), VIK_LAYER_WIDGET_SPINBUTTON, &scales[2], NULL, N_("You need to restart Viking for a change to this value to be used"), rr_to_default, NULL, NULL },
 	// Changeable any time
-	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"carto", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CartoCSS:"), VIK_LAYER_WIDGET_FILEENTRY, NULL, NULL,  N_("The program to convert CartoCSS files into Mapnik XML"), NULL, NULL, NULL },
+	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"carto", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CartoCSS:"), VIK_LAYER_WIDGET_FILEENTRY, NULL, NULL,  N_("The program to convert CartoCSS files into Mapnik XML"), carto_default, NULL, NULL },
 };
 
 static time_t planet_import_time;
@@ -258,6 +278,8 @@ static time_t planet_import_time;
 static GMutex *tp_mutex;
 static GHashTable *requests = NULL;
 
+static GdkColor black_color;
+
 /**
  * vik_mapnik_layer_init:
  *
@@ -267,21 +289,10 @@ void vik_mapnik_layer_init (void)
 {
 	a_preferences_register_group ( MAPNIK_PREFS_GROUP_KEY, _("Mapnik") );
 
-	guint i = 0;
-	VikLayerParamData tmp = plugins_default();
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
+	for ( guint ii = 0; ii < G_N_ELEMENTS(prefs); ii++ )
+	  a_preferences_register ( &prefs[ii], (VikLayerParamData){0}, MAPNIK_PREFS_GROUP_KEY );
 
-	tmp = fonts_default();
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.b = TRUE;
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.u = 168; // One week
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.s = "carto";
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
+        gdk_color_parse ( "#000000", &black_color );
 }
 
 /**
@@ -317,6 +328,7 @@ void vik_mapnik_layer_post_init (void)
 void vik_mapnik_layer_uninit ()
 {
 	vik_mutex_free (tp_mutex);
+	g_hash_table_destroy ( requests );
 }
 
 // NB Only performed once per program run
@@ -370,20 +382,6 @@ static void mapnik_layer_set_file_xml ( VikMapnikLayer *vml, const gchar *name )
 		vml->filename_xml = g_strdup (name);
 }
 
-static void mapnik_layer_set_file_css ( VikMapnikLayer *vml, const gchar *name )
-{
-	if ( vml->filename_css )
-		g_free (vml->filename_css);
-	vml->filename_css = g_strdup (name);
-}
-
-static void mapnik_layer_set_cache_dir ( VikMapnikLayer *vml, const gchar *name )
-{
-	if ( vml->file_cache_dir )
-		g_free (vml->file_cache_dir);
-	vml->file_cache_dir = g_strdup (name);
-}
-
 static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len )
 {
 	vik_layer_marshall_params ( VIK_LAYER(vml), data, len );
@@ -396,16 +394,34 @@ static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikView
 	return rv;
 }
 
-static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation )
+static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, VikLayerSetParam *vlsp )
 {
-	switch ( id ) {
-		case PARAM_CONFIG_CSS: mapnik_layer_set_file_css (vml, data.s); break;
-		case PARAM_CONFIG_XML: mapnik_layer_set_file_xml (vml, data.s); break;
-		case PARAM_ALPHA: if ( data.u <= 255 ) vml->alpha = data.u; break;
-		case PARAM_USE_FILE_CACHE: vml->use_file_cache = data.b; break;
-		case PARAM_FILE_CACHE_DIR: mapnik_layer_set_cache_dir (vml, data.s); break;
+	gboolean changed = FALSE;
+	switch ( vlsp->id ) {
+		case PARAM_CONFIG_CSS:
+			changed = vik_layer_param_change_string ( vlsp->data, &vml->filename_css );
+			break;
+		case PARAM_CONFIG_XML: {
+			gchar* old = g_strdup ( vml->filename_xml );
+			mapnik_layer_set_file_xml ( vml, vlsp->data.s );
+			changed = g_strcmp0 ( old, vlsp->data.s );
+			g_free ( old );
+			break;
+		}
+		case PARAM_ALPHA:
+			if ( vlsp->data.u <= 255 )
+				changed = vik_layer_param_change_uint8 ( vlsp->data, &vml->alpha );
+			break;
+		case PARAM_USE_FILE_CACHE:
+			changed = vik_layer_param_change_boolean ( vlsp->data, &vml->use_file_cache );
+			break;
+		case PARAM_FILE_CACHE_DIR:
+			changed = vik_layer_param_change_string ( vlsp->data, &vml->file_cache_dir );
+			break;
 		default: break;
 	}
+	if ( vik_debug && changed )
+		g_debug ( "%s: Detected change on param %d", __FUNCTION__, vlsp->id );
 	return TRUE;
 }
 
@@ -450,6 +466,7 @@ static VikLayerParamData mapnik_layer_get_param ( VikMapnikLayer *vml, guint16 i
 		case PARAM_ALPHA: data.u = vml->alpha; break;
 		case PARAM_USE_FILE_CACHE: data.b = vml->use_file_cache; break;
 		case PARAM_FILE_CACHE_DIR: data.s = vml->file_cache_dir; break;
+		case PARAM_RESET: data.ptr = reset_cb; break;
 		default: break;
 	}
 	return data;
@@ -499,17 +516,11 @@ gboolean carto_load ( VikMapnikLayer *vml, VikViewport *vvp )
 		g_free ( msg );
 	}
 
-	gint64 tt1 = 0;
+	gint64 tt1 = g_get_real_time ();
 	gint64 tt2 = 0;
-	// You won't get a sensible timing measurement if running too old a GLIB
-#if GLIB_CHECK_VERSION (2, 28, 0)
-	tt1 = g_get_real_time ();
-#endif
 
 	if ( g_spawn_command_line_sync ( command, &mystdout, &mystderr, NULL, &error ) ) {
-#if GLIB_CHECK_VERSION (2, 28, 0)
 		tt2 = g_get_real_time ();
-#endif
 		if ( mystderr )
 			if ( strlen(mystderr) > 1 ) {
 				a_dialog_error_msg_extra ( VIK_GTK_WINDOW_FROM_WIDGET(vvp), _("Error running carto command:\n%s"), mystderr );
@@ -662,7 +673,7 @@ static void render ( VikMapnikLayer *vml, VikCoord *ul, VikCoord *br, MapCoord *
 	g_debug ( "Mapnik rendering completed in %.3f seconds", tt );
 	if ( !pixbuf ) {
 		// A pixbuf to stick into cache incase of an unrenderable area - otherwise will get continually re-requested
-		pixbuf = gdk_pixbuf_scale_simple ( gdk_pixbuf_from_pixdata(&vikmapniklayer_pixbuf, FALSE, NULL), vml->tile_size_x, vml->tile_size_x, GDK_INTERP_BILINEAR );
+		pixbuf = gdk_pixbuf_scale_simple ( ui_get_icon("vikmapniklayer", 16), vml->tile_size_x, vml->tile_size_x, GDK_INTERP_BILINEAR );
 	}
 	possibly_save_pixbuf ( vml, pixbuf, ulm );
 
@@ -694,7 +705,7 @@ static void background ( RenderInfo *data, gpointer threaddata )
 	g_mutex_unlock(tp_mutex);
 
 	if (res == 0)
-		vik_layer_emit_update ( VIK_LAYER(data->vml) ); // NB update display from background
+		vik_layer_emit_update ( VIK_LAYER(data->vml), FALSE ); // NB update display from background
 }
 
 static void render_cancel_cleanup (RenderInfo *data)
@@ -809,7 +820,7 @@ static GdkPixbuf *get_pixbuf ( VikMapnikLayer *vml, MapCoord *ulm, MapCoord *brm
 			else {
 				// Run in the foreground
 				render ( vml, &ul, &br, ulm );
-				vik_layer_emit_update ( VIK_LAYER(vml) );
+				vik_layer_emit_update ( VIK_LAYER(vml), FALSE );
 			}
 		}
 	}
@@ -882,7 +893,7 @@ static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vvp )
 		// Done after so drawn on top
 		// Just a handy guide to tile blocks.
 		if ( vik_debug && vik_verbose ) {
-			GdkGC *black_gc = GTK_WIDGET(vvp)->style->black_gc;
+			GdkGC *black_gc = vik_viewport_get_black_gc ( vvp );
 			gint width = vik_viewport_get_width(vvp);
 			gint height = vik_viewport_get_height(vvp);
 			gint xx, yy;
@@ -892,11 +903,11 @@ static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vvp )
 			xx = xx - (vml->tile_size_x/2);
 			yy = yy - (vml->tile_size_x/2); // Yes use X ATM
 			for (gint x = xmin; x <= xmax; x++ ) {
-				vik_viewport_draw_line ( vvp, black_gc, xx, 0, xx, height );
+				vik_viewport_draw_line ( vvp, black_gc, xx, 0, xx, height, &black_color, 1 );
 				xx += vml->tile_size_x;
 			}
 			for (gint y = ymin; y <= ymax; y++ ) {
-				vik_viewport_draw_line ( vvp, black_gc, 0, yy, width, yy );
+				vik_viewport_draw_line ( vvp, black_gc, 0, yy, width, yy, &black_color, 1 );
 				yy += vml->tile_size_x; // Yes use X ATM
 			}
 		}
@@ -1100,10 +1111,10 @@ static void mapnik_layer_tile_info ( VikMapnikLayer *vml )
 	g_free ( filename );
 }
 
-static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp )
+static VikLayerToolFuncStatus mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp )
 {
 	if ( !vml )
-		return FALSE;
+		return VIK_LAYER_TOOL_IGNORED;
 	if ( event->button == 3 ) {
 		vik_viewport_screen_to_coord ( vvp, MAX(0, event->x), MAX(0, event->y), &vml->rerender_ul );
 		vml->rerender_zoom = vik_viewport_get_zoom ( vvp );
@@ -1119,5 +1130,5 @@ static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *ev
 		gtk_widget_show_all ( GTK_WIDGET(vml->right_click_menu) );
 	}
 
-	return FALSE;
+	return VIK_LAYER_TOOL_IGNORED;
 }
