@@ -44,6 +44,8 @@ static guint16 map_source_get_tilesize_y (VikMapSource *self);
 static gdouble map_source_get_scale (VikMapSource *self);
 static VikViewportDrawMode map_source_get_drawmode (VikMapSource *self);
 static const gchar *map_source_get_file_extension (VikMapSource *self);
+static gdouble map_source_get_offset_x (VikMapSource *self);
+static gdouble map_source_get_offset_y (VikMapSource *self);
 
 static DownloadResult_t _download ( VikMapSource *self, MapCoord *src, const gchar *dest_fn, void *handle );
 static void * _download_handle_init ( VikMapSource *self );
@@ -66,9 +68,12 @@ struct _VikMapSourceDefaultPrivate
 	gdouble scale;
 	VikViewportDrawMode drawmode;
 	gchar *file_extension;
+	gdouble offset_x;
+	gdouble offset_y;
 };
 
-#define VIK_MAP_SOURCE_DEFAULT_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIK_TYPE_MAP_SOURCE_DEFAULT, VikMapSourceDefaultPrivate))
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (VikMapSourceDefault, vik_map_source_default, VIK_TYPE_MAP_SOURCE);
+#define VIK_MAP_SOURCE_DEFAULT_PRIVATE(o)  (vik_map_source_default_get_instance_private (VIK_MAP_SOURCE_DEFAULT(o)))
 
 /* properties */
 enum
@@ -86,15 +91,14 @@ enum
   PROP_LICENSE,
   PROP_LICENSE_URL,
   PROP_FILE_EXTENSION,
+  PROP_OFFSET_X,
+  PROP_OFFSET_Y,
 };
 
-G_DEFINE_ABSTRACT_TYPE (VikMapSourceDefault, vik_map_source_default, VIK_TYPE_MAP_SOURCE);
-
 static void
 vik_map_source_default_init (VikMapSourceDefault *object)
 {
-  VikMapSourceDefault *self = VIK_MAP_SOURCE_DEFAULT (object);
-  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (self);
+  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (object);
 
   priv->label = NULL;
   priv->copyright = NULL;
@@ -108,8 +112,7 @@ vik_map_source_default_init (VikMapSourceDefault *object)
 static void
 vik_map_source_default_finalize (GObject *object)
 {
-  VikMapSourceDefault *self = VIK_MAP_SOURCE_DEFAULT (object);
-  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (self);
+  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (object);
 
   g_free (priv->label);
   priv->label = NULL;
@@ -135,8 +138,7 @@ vik_map_source_default_set_property (GObject      *object,
                                      const GValue *value,
                                      GParamSpec   *pspec)
 {
-  VikMapSourceDefault *self = VIK_MAP_SOURCE_DEFAULT (object);
-  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (self);
+  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (object);
 
   switch (property_id)
     {
@@ -145,7 +147,8 @@ vik_map_source_default_set_property (GObject      *object,
       // A simple check just to prevent containing slashes ATM
       g_free (priv->name);
       priv->name = g_strdup(g_value_get_string (value));
-      g_strdelimit (priv->name, "\\/", 'x' );
+      if ( priv->name )
+        g_strdelimit (priv->name, "\\/", '_' );
       break;
 
     case PROP_ID:
@@ -193,6 +196,14 @@ vik_map_source_default_set_property (GObject      *object,
       priv->file_extension = g_strdup(g_value_get_string(value));
       break;
 
+    case PROP_OFFSET_X:
+      priv->offset_x = g_value_get_double (value);
+      break;
+
+    case PROP_OFFSET_Y:
+      priv->offset_y = g_value_get_double (value);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -206,8 +217,7 @@ vik_map_source_default_get_property (GObject    *object,
                                      GValue     *value,
                                      GParamSpec *pspec)
 {
-  VikMapSourceDefault *self = VIK_MAP_SOURCE_DEFAULT (object);
-  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (self);
+  VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE (object);
 
   switch (property_id)
     {
@@ -255,6 +265,14 @@ vik_map_source_default_get_property (GObject    *object,
       g_value_set_string (value, priv->file_extension);
       break;
 
+    case PROP_OFFSET_X:
+      g_value_set_double (value, priv->offset_x);
+      break;
+
+    case PROP_OFFSET_Y:
+      g_value_set_double (value, priv->offset_y);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -285,6 +303,8 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
 	parent_class->get_scale =      map_source_get_scale;
 	parent_class->get_drawmode =   map_source_get_drawmode;
 	parent_class->get_file_extension = map_source_get_file_extension;
+	parent_class->get_offset_x = map_source_get_offset_x;
+	parent_class->get_offset_y = map_source_get_offset_y;
 	parent_class->download =                 _download;
 	parent_class->download_handle_init =     _download_handle_init;
 	parent_class->download_handle_cleanup =  _download_handle_cleanup;
@@ -297,7 +317,7 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
 	pspec = g_param_spec_string ("name",
 	                             "Name",
 	                             "The name of the map that may be used as the file cache directory",
-	                             "Unknown" /* default value */,
+	                             NULL, // default value
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_NAME, pspec);
 
@@ -313,7 +333,7 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
 	pspec = g_param_spec_string ("label",
 	                             "Label",
 	                             "The label of the map source",
-	                             "<no-set>" /* default value */,
+	                             "Unknown", // default value
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_LABEL, pspec);
 
@@ -380,7 +400,23 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_FILE_EXTENSION, pspec);
 
-	g_type_class_add_private (klass, sizeof (VikMapSourceDefaultPrivate));
+	pspec = g_param_spec_double ("offset-x",
+	                             "Offset X",
+	                             "The offset of a tile (x) in metres",
+	                             -G_MAXDOUBLE, // minimum value
+	                             G_MAXDOUBLE, // maximum value
+	                             0.0, // default value
+	                             G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_OFFSET_X, pspec);
+
+	pspec = g_param_spec_double ("offset-y",
+	                             "Offset Y",
+	                             "The offset of a tile (y) in metres",
+	                             -G_MAXDOUBLE, // minimum value
+	                             G_MAXDOUBLE, // maximum value
+	                             0.0, // default value
+	                             G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_OFFSET_Y, pspec);
 
 	object_class->finalize = vik_map_source_default_finalize;
 }
@@ -431,7 +467,14 @@ map_source_get_name (VikMapSource *self)
 {
 	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT(self), NULL);
 	VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE(self);
-	return priv->name;
+	if ( priv->name )
+		return priv->name;
+	else {
+		// Fallback if not set
+		priv->name = g_strdup ( priv->label );
+		g_strdelimit ( priv->name, "\\/", '_' );
+		return priv->name;
+	}
 }
 
 static guint16
@@ -499,8 +542,9 @@ _download ( VikMapSource *self, MapCoord *src, const gchar *dest_fn, void *handl
 {
    gchar *uri = vik_map_source_default_get_uri(VIK_MAP_SOURCE_DEFAULT(self), src);
    gchar *host = vik_map_source_default_get_hostname(VIK_MAP_SOURCE_DEFAULT(self));
-   DownloadFileOptions *options = vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self));
+   DownloadFileOptions *options = vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self), src);
    DownloadResult_t res = a_http_download_get_url ( host, uri, dest_fn, options, handle );
+   a_download_file_options_free ( options );
    g_free ( uri );
    g_free ( host );
    return res;
@@ -514,6 +558,22 @@ map_source_get_file_extension (VikMapSource *self)
     return priv->file_extension;
 }
 
+static gdouble
+map_source_get_offset_x (VikMapSource *self)
+{
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT(self), (gdouble)0.0);
+	VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE(self);
+	return priv->offset_x;
+}
+
+static gdouble
+map_source_get_offset_y (VikMapSource *self)
+{
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT(self), (gdouble)0.0);
+	VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE(self);
+	return priv->offset_y;
+}
+
 static void *
 _download_handle_init ( VikMapSource *self )
 {
@@ -531,11 +591,11 @@ gchar *
 vik_map_source_default_get_uri( VikMapSourceDefault *self, MapCoord *src )
 {
 	VikMapSourceDefaultClass *klass;
-	g_return_val_if_fail (self != NULL, 0);
-	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), 0);
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), NULL);
 	klass = VIK_MAP_SOURCE_DEFAULT_GET_CLASS(self);
 
-	g_return_val_if_fail (klass->get_uri != NULL, 0);
+	g_return_val_if_fail (klass->get_uri != NULL, NULL);
 
 	return (*klass->get_uri)(self, src);
 }
@@ -544,38 +604,38 @@ gchar *
 vik_map_source_default_get_hostname( VikMapSourceDefault *self )
 {
 	VikMapSourceDefaultClass *klass;
-	g_return_val_if_fail (self != NULL, 0);
-	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), 0);
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), NULL);
 	klass = VIK_MAP_SOURCE_DEFAULT_GET_CLASS(self);
 
-	g_return_val_if_fail (klass->get_hostname != NULL, 0);
+	g_return_val_if_fail (klass->get_hostname != NULL, NULL);
 
 	return (*klass->get_hostname)(self);
 }
 
 DownloadFileOptions *
-vik_map_source_default_get_download_options( VikMapSourceDefault *self )
+vik_map_source_default_get_download_options( VikMapSourceDefault *self, MapCoord *src )
 {
 	VikMapSourceDefaultClass *klass;
-	g_return_val_if_fail (self != NULL, 0);
-	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), 0);
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), NULL);
 	klass = VIK_MAP_SOURCE_DEFAULT_GET_CLASS(self);
 
-	g_return_val_if_fail (klass->get_download_options != NULL, 0);
+	g_return_val_if_fail (klass->get_download_options != NULL, NULL);
 
-	return (*klass->get_download_options)(self);
+	return (*klass->get_download_options)(self, src);
 }
 
 gchar *
 vik_map_source_default_get_url_display( VikMapSourceDefault *self, MapCoord *src )
 {
 	VikMapSourceDefaultClass *klass;
-	g_return_val_if_fail (self != NULL, 0);
-	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), 0);
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), NULL);
 	klass = VIK_MAP_SOURCE_DEFAULT_GET_CLASS(self);
 
-	g_return_val_if_fail (klass->get_uri != NULL, 0);
-	g_return_val_if_fail (klass->get_hostname != NULL, 0);
+	g_return_val_if_fail (klass->get_uri != NULL, NULL);
+	g_return_val_if_fail (klass->get_hostname != NULL, NULL);
 
 	gchar *newstr = NULL;
 	gchar *hostname = (*klass->get_hostname)(self);
@@ -588,16 +648,19 @@ vik_map_source_default_get_url_display( VikMapSourceDefault *self, MapCoord *src
 		else {
 			newstr = g_strdup ( hostname );
 		}
+		g_free ( hostname );
 	}
 	if ( url && strlen(url)>1 ) {
 		if ( newstr ) {
 			gchar *tmp = g_strdup ( newstr );
-			newstr = g_strdup_printf ( "%s%s", newstr, url );
+			g_free ( newstr );
+			newstr = g_strdup_printf ( "%s%s", tmp, url );
 			g_free ( tmp );
 		}
 		else {
 			newstr = g_strdup ( url );
 		}
+		g_free ( url );
 	}
 
 	return newstr;
