@@ -43,7 +43,7 @@
 static void vik_routing_web_engine_finalize ( GObject *gob );
 
 static gboolean vik_routing_web_engine_find ( VikRoutingEngine *self, VikTrwLayer *vtl, struct LatLon start, struct LatLon end );
-static gchar *vik_routing_web_engine_get_cmd_from_directions(VikRoutingEngine *self, const gchar *start, const gchar *end);
+static gchar *vik_routing_web_engine_get_url_from_directions(VikRoutingEngine *self, const gchar *start, const gchar *end);
 static gboolean vik_routing_web_engine_supports_direction(VikRoutingEngine *self);
 static gboolean vik_routing_web_engine_refine ( VikRoutingEngine *self, VikTrwLayer *vtl, VikTrack *vt );
 static gboolean vik_routing_web_engine_supports_refine ( VikRoutingEngine *self );
@@ -57,15 +57,17 @@ struct _VikRoutingWebEnginePrivate
 	gchar *url_start_ll_fmt;
 	gchar *url_stop_ll_fmt;
 	gchar *url_via_ll_fmt;
+	gboolean url_ll_lat_first_fmt;
 
 	/* Directions */
 	gchar *url_start_dir_fmt;
 	gchar *url_stop_dir_fmt;
 
-	DownloadMapOptions options;
+	DownloadFileOptions options;
 };
 
-#define VIK_ROUTING_WEB_ENGINE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIK_ROUTING_WEB_ENGINE_TYPE, VikRoutingWebEnginePrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (VikRoutingWebEngine, vik_routing_web_engine, VIK_ROUTING_ENGINE_TYPE)
+#define VIK_ROUTING_WEB_ENGINE_PRIVATE(o)  (vik_routing_web_engine_get_instance_private (VIK_ROUTING_WEB_ENGINE(o)))
 
 /* properties */
 enum
@@ -77,6 +79,7 @@ enum
   /* LatLon */
   PROP_URL_START_LL,
   PROP_URL_STOP_LL,
+  PROP_URL_LL_LAT_FIRST,
   PROP_URL_VIA_LL,
 
   /* Direction */
@@ -87,8 +90,6 @@ enum
   PROP_FOLLOW_LOCATION,
 };
 
-G_DEFINE_TYPE (VikRoutingWebEngine, vik_routing_web_engine, VIK_ROUTING_ENGINE_TYPE)
-
 static void
 vik_routing_web_engine_set_property (GObject      *object,
                           guint         property_id,
@@ -114,6 +115,10 @@ vik_routing_web_engine_set_property (GObject      *object,
       priv->url_stop_ll_fmt = g_strdup(g_value_get_string (value));
       break;
 
+    case PROP_URL_LL_LAT_FIRST:
+      priv->url_ll_lat_first_fmt = g_value_get_boolean (value);
+      break;
+
     case PROP_URL_VIA_LL:
       g_free (priv->url_via_ll_fmt);
       priv->url_via_ll_fmt = g_strdup(g_value_get_string (value));
@@ -167,6 +172,10 @@ vik_routing_web_engine_get_property (GObject    *object,
       g_value_set_string (value, priv->url_stop_ll_fmt);
       break;
 
+    case PROP_URL_LL_LAT_FIRST:
+      g_value_set_boolean (value, priv->url_ll_lat_first_fmt);
+      break;
+
     case PROP_URL_VIA_LL:
       g_value_set_string (value, priv->url_via_ll_fmt);
       break;
@@ -210,7 +219,7 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
 
   parent_class->find = vik_routing_web_engine_find;
   parent_class->supports_direction = vik_routing_web_engine_supports_direction;
-  parent_class->get_cmd_from_directions = vik_routing_web_engine_get_cmd_from_directions;
+  parent_class->get_url_from_directions = vik_routing_web_engine_get_url_from_directions;
   parent_class->refine = vik_routing_web_engine_refine;
   parent_class->supports_refine = vik_routing_web_engine_supports_refine;
 
@@ -238,7 +247,20 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
                                "<no-set>" /* default value */,
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_URL_START_LL, pspec);
-    
+
+
+  /**
+   * VikRoutingWebEngine:url-ll-lat-first:
+   *
+   * Coordinates are in latitude,longitude form if true (longitude,latitue else).
+   */
+  pspec = g_param_spec_boolean ("url-ll-lat-first",
+                                "Latitude first",
+                                "Coordinates in latitude,longitude (instead of default longitude,latitude)",
+                                TRUE/* default value */,
+                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_URL_LL_LAT_FIRST, pspec);
+   
 
   /**
    * VikRoutingWebEngine:url-stop-ll:
@@ -313,13 +335,11 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
   pspec = g_param_spec_long ("follow-location",
                              "Follow location",
                              "Specifies the number of retries to follow a redirect while downloading a page",
-                             0  /* minimum value */,
+                             -1,  // minimum value (unlimited)
                              G_MAXLONG /* maximum value */,
                              2  /* default value */,
                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_FOLLOW_LOCATION, pspec);
-
-  g_type_class_add_private (klass, sizeof (VikRoutingWebEnginePrivate));
 }
 
 static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
@@ -332,6 +352,7 @@ static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
   priv->url_start_ll_fmt = NULL;
   priv->url_stop_ll_fmt = NULL;
   priv->url_via_ll_fmt = NULL;
+  priv->url_ll_lat_first_fmt = TRUE;
 
   /* Directions */
   priv->url_start_dir_fmt = NULL;
@@ -342,6 +363,7 @@ static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
   priv->options.check_file = NULL;
   priv->options.check_file_server_time = FALSE;
   priv->options.use_etag = FALSE;
+  priv->options.custom_http_headers = NULL;
 }
 
 static void vik_routing_web_engine_finalize ( GObject *gob )
@@ -358,6 +380,7 @@ static void vik_routing_web_engine_finalize ( GObject *gob )
   priv->url_stop_ll_fmt = NULL;
   g_free (priv->url_via_ll_fmt);
   priv->url_via_ll_fmt = NULL;
+  priv->url_ll_lat_first_fmt = TRUE;
 
   /* Directions */
   g_free (priv->url_start_dir_fmt);
@@ -371,7 +394,7 @@ static void vik_routing_web_engine_finalize ( GObject *gob )
   G_OBJECT_CLASS (vik_routing_web_engine_parent_class)->finalize(gob);
 }
 
-static DownloadMapOptions *
+static DownloadFileOptions *
 vik_routing_web_engine_get_download_options ( VikRoutingEngine *self )
 {
 	g_return_val_if_fail (VIK_IS_ROUTING_WEB_ENGINE(self), NULL);
@@ -391,6 +414,16 @@ substitute_latlon ( const gchar *fmt, struct LatLon ll )
 	return substituted;
 }
 
+static gchar *
+substitute_lonlat ( const gchar *fmt, struct LatLon ll )
+{
+	gchar lat[G_ASCII_DTOSTR_BUF_SIZE], lon[G_ASCII_DTOSTR_BUF_SIZE];
+	gchar *substituted = g_strdup_printf(fmt,
+                          g_ascii_dtostr (lon, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lon),
+                          g_ascii_dtostr (lat, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lat));
+	return substituted;
+}
+
 static gchar *
 vik_routing_web_engine_get_url_for_coords ( VikRoutingEngine *self, struct LatLon start, struct LatLon end )
 {
@@ -406,8 +439,12 @@ vik_routing_web_engine_get_url_for_coords ( VikRoutingEngine *self, struct LatLo
 	g_return_val_if_fail ( priv->url_start_ll_fmt != NULL, NULL);
 	g_return_val_if_fail ( priv->url_stop_ll_fmt != NULL, NULL);
 
-	startURL = substitute_latlon ( priv->url_start_ll_fmt, start );
-	endURL = substitute_latlon ( priv->url_stop_ll_fmt, end );
+	startURL = (priv->url_ll_lat_first_fmt) ?
+			substitute_latlon ( priv->url_start_ll_fmt, start ):
+			substitute_lonlat ( priv->url_start_ll_fmt, start );
+	endURL =  (priv->url_ll_lat_first_fmt) ?
+			substitute_latlon ( priv->url_stop_ll_fmt, end ):
+			substitute_lonlat ( priv->url_stop_ll_fmt, end );
 	url = g_strconcat ( priv->url_base, startURL, endURL, NULL );
 
 	/* Free memory */
@@ -422,10 +459,11 @@ vik_routing_web_engine_find ( VikRoutingEngine *self, VikTrwLayer *vtl, struct L
 {
   gchar *uri = vik_routing_web_engine_get_url_for_coords(self, start, end);
 
-  DownloadMapOptions *options = vik_routing_web_engine_get_download_options(self);
+  DownloadFileOptions *options = vik_routing_web_engine_get_download_options(self);
   
   gchar *format = vik_routing_engine_get_format ( self );
-  gboolean ret = a_babel_convert_from_url ( vtl, uri, format, NULL, NULL, options );
+  ProcessOptions po = { NULL, NULL, format, uri, NULL, NULL };
+  gboolean ret = a_babel_convert_from ( vtl, &po, NULL, NULL, options );
 
   g_free(uri);
 
@@ -433,7 +471,7 @@ vik_routing_web_engine_find ( VikRoutingEngine *self, VikTrwLayer *vtl, struct L
 }
 
 static gchar *
-vik_routing_web_engine_get_cmd_from_directions ( VikRoutingEngine *self, const gchar *start, const gchar *end )
+vik_routing_web_engine_get_url_from_directions ( VikRoutingEngine *self, const gchar *start, const gchar *end )
 {
   g_return_val_if_fail ( VIK_IS_ROUTING_WEB_ENGINE (self), NULL);
 
@@ -492,7 +530,9 @@ _append_stringified_coords ( gpointer data, gpointer user_data )
   /* Stringify coordinate */
   struct LatLon position;
   vik_coord_to_latlon ( &(vtp->coord), &position );
-  gchar *string = substitute_latlon ( ctx->priv->url_via_ll_fmt, position );
+  gchar *string = (ctx->priv->url_ll_lat_first_fmt) ?
+	  	substitute_latlon ( ctx->priv->url_via_ll_fmt, position ):
+	  	substitute_lonlat ( ctx->priv->url_via_ll_fmt, position );
   
   /* Append */
   ctx->urlParts[ctx->nb] = string;
@@ -532,11 +572,15 @@ vik_routing_web_engine_get_url_for_track ( VikRoutingEngine *self, VikTrack *vt
   g_free ( urlParts[1] );
   vtp = g_list_first ( vt->trackpoints )->data;
   vik_coord_to_latlon ( &(vtp->coord ), &position );
-  urlParts[1] = substitute_latlon ( priv->url_start_ll_fmt, position );
+  urlParts[1] = (priv->url_ll_lat_first_fmt) ?
+	  	substitute_latlon ( priv->url_start_ll_fmt, position ):
+	  	substitute_lonlat ( priv->url_start_ll_fmt, position );
   g_free ( urlParts[len-2] );
   vtp = g_list_last ( vt->trackpoints )->data;
   vik_coord_to_latlon ( &(vtp->coord), &position );
-  urlParts[len-2] = substitute_latlon ( priv->url_stop_ll_fmt, position );
+  urlParts[len-2] = (priv->url_ll_lat_first_fmt) ?
+	  	substitute_latlon ( priv->url_stop_ll_fmt, position ):
+	  	substitute_lonlat ( priv->url_stop_ll_fmt, position );
 
   /* Concat */
   url = g_strjoinv ( NULL, urlParts );
@@ -555,11 +599,12 @@ vik_routing_web_engine_refine ( VikRoutingEngine *self, VikTrwLayer *vtl, VikTra
   gchar *uri = vik_routing_web_engine_get_url_for_track ( self, vt );
 
   /* Download data */
-  DownloadMapOptions *options = vik_routing_web_engine_get_download_options ( self );
+  DownloadFileOptions *options = vik_routing_web_engine_get_download_options ( self );
 
   /* Convert and insert data in model */
   gchar *format = vik_routing_engine_get_format ( self );
-  gboolean ret = a_babel_convert_from_url ( vtl, uri, format, NULL, NULL, options );
+  ProcessOptions po = { NULL, NULL, format, uri, NULL, NULL };
+  gboolean ret = a_babel_convert_from ( vtl, &po, NULL, NULL, options );
 
   g_free(uri);
 
