@@ -31,6 +31,7 @@
 
 #include "vikstatus.h"
 #include "background.h"
+#include "logging.h"
 
 enum
 {
@@ -39,7 +40,11 @@ enum
 };
 
 struct _VikStatusbar {
+#if GTK_CHECK_VERSION (3,0,0)
+  GtkBox hbox;
+#else
   GtkHBox hbox;
+#endif
   GtkWidget *status[VIK_STATUSBAR_NUM_TYPES];
   gboolean empty[VIK_STATUSBAR_NUM_TYPES];
 };
@@ -48,7 +53,7 @@ G_DEFINE_TYPE (VikStatusbar, vik_statusbar, GTK_TYPE_HBOX)
 
 static guint vik_statusbar_signals[LAST_SIGNAL] = { 0 };
 
-static gint
+static void
 forward_signal (GObject *object, gpointer user_data)
 {
     gint item = GPOINTER_TO_INT (g_object_get_data ( object, "type" ));
@@ -57,15 +62,12 @@ forward_signal (GObject *object, gpointer user_data)
     // Clicking on the items field will bring up the background jobs window
     if ( item == VIK_STATUSBAR_ITEMS )
       a_background_show_window();
-    else if ( item == VIK_STATUSBAR_INFO )
-      // Clear current info message
-      vik_statusbar_set_message ( vs, VIK_STATUSBAR_INFO, "" );
+    else if ( item == VIK_STATUSBAR_LOG )
+      a_logging_show_window();
     else
       g_signal_emit (G_OBJECT (vs),
                      vik_statusbar_signals[CLICKED], 0,
                      item);
-
-    return TRUE;
 }
 
 static void
@@ -86,21 +88,14 @@ vik_statusbar_class_init (VikStatusbarClass *klass)
 
 static gboolean button_release_event (GtkWidget* widget, GdkEvent *event, gpointer *user_data)
 {
-  if ( ((GdkEventButton*)event)->button == 3 ) {
+  if ( ((GdkEventButton*)event)->button == 2 ) {
     gint type = GPOINTER_TO_INT (g_object_get_data ( G_OBJECT(widget), "type" ));
     VikStatusbar *vs = VIK_STATUSBAR (user_data);
-    // Right Click: so copy the text in the INFO buffer only ATM
-    if ( type == VIK_STATUSBAR_INFO ) {
-      const gchar* msg = gtk_button_get_label (GTK_BUTTON(vs->status[VIK_STATUSBAR_INFO]));
-      if ( msg ) {
-        GtkClipboard *clipboard = gtk_clipboard_get ( GDK_SELECTION_CLIPBOARD );
-        gtk_clipboard_set_text  ( clipboard, msg, -1 );
-      }
-    }
-    // We've handled the event
-    return TRUE;
+    // Middle Click: clear the text
+    if ( type == VIK_STATUSBAR_INFO )
+      vik_statusbar_set_message ( vs, VIK_STATUSBAR_INFO, "" );
   }
-  // Otherwise carry on with other event handlers - i.e. ensure forward_signal() is called
+  // Otherwise carry on with other event handlers
   return FALSE;
 }
 
@@ -112,12 +107,16 @@ vik_statusbar_init (VikStatusbar *vs)
   for ( i = 0; i < VIK_STATUSBAR_NUM_TYPES; i++ ) {
     vs->empty[i] = TRUE;
     
-    if (i == VIK_STATUSBAR_ITEMS || i == VIK_STATUSBAR_ZOOM || i == VIK_STATUSBAR_INFO )
+    if (i == VIK_STATUSBAR_ITEMS || i == VIK_STATUSBAR_ZOOM || i == VIK_STATUSBAR_LOG )
       vs->status[i] = gtk_button_new();
+    else if ( i == VIK_STATUSBAR_INFO )
+      vs->status[i] = gtk_label_new ( NULL );
     else
     {
       vs->status[i] = gtk_statusbar_new();
+#if !GTK_CHECK_VERSION (3,0,0)
       gtk_statusbar_set_has_resize_grip ( GTK_STATUSBAR(vs->status[i]), FALSE );
+#endif
     }
     g_object_set_data (G_OBJECT (vs->status[i]), "type", GINT_TO_POINTER(i));
   }
@@ -136,11 +135,23 @@ vik_statusbar_init (VikStatusbar *vs)
 
   gtk_box_pack_start ( GTK_BOX(vs), vs->status[VIK_STATUSBAR_POSITION], FALSE, FALSE, 1);
 
+  g_signal_connect ( G_OBJECT(vs->status[VIK_STATUSBAR_LOG]), "clicked", G_CALLBACK (forward_signal), vs );
+  gtk_widget_set_tooltip_text (GTK_WIDGET (vs->status[VIK_STATUSBAR_LOG]), _("Current number of log messages. Click to see them."));
+  gtk_box_pack_start ( GTK_BOX(vs), vs->status[VIK_STATUSBAR_LOG], FALSE, FALSE, 1 );
+
   g_signal_connect ( G_OBJECT(vs->status[VIK_STATUSBAR_INFO]), "button-release-event", G_CALLBACK (button_release_event), vs);
-  g_signal_connect ( G_OBJECT(vs->status[VIK_STATUSBAR_INFO]), "clicked", G_CALLBACK (forward_signal), vs);
-  gtk_widget_set_tooltip_text (GTK_WIDGET (vs->status[VIK_STATUSBAR_INFO]), _("Left click to clear the message. Right click to copy the message."));
-  gtk_button_set_alignment ( GTK_BUTTON(vs->status[VIK_STATUSBAR_INFO]), 0.0, 0.5 ); // Left align the text
+  gtk_widget_set_tooltip_text (GTK_WIDGET (vs->status[VIK_STATUSBAR_INFO]), _("Middle click to clear the message."));
+  gtk_misc_set_alignment ( GTK_MISC(vs->status[VIK_STATUSBAR_INFO]), 0.0, 0.5 ); // Left align the text
   gtk_box_pack_end ( GTK_BOX(vs), vs->status[VIK_STATUSBAR_INFO], TRUE, TRUE, 1);
+  // Prevent it being a fixed size and not then being able to resize the main window smaller
+  // Previously in GTK2, seemingly the gtk statusbar would prevent the INFO text field resizing
+  //  and interfering with the main window sizing
+  gtk_label_set_ellipsize ( GTK_LABEL(vs->status[VIK_STATUSBAR_INFO]), PANGO_ELLIPSIZE_END );
+  gtk_label_set_selectable ( GTK_LABEL(vs->status[VIK_STATUSBAR_INFO]), TRUE );
+  // ATM in GTK3 it means the vikstatusbar can't be resized smaller than the first fixed 5 fields
+  // i.e. only within the INFO statusbar part; which is probably an acceptable limitation
+  // since most usage of Viking isn't going to be with a small window and if you really want a small
+  //  window you can remove the statusbar altogether.
 }
 
 /**
@@ -159,6 +170,7 @@ vik_statusbar_new (guint scale)
   gtk_widget_set_size_request ( vs->status[VIK_STATUSBAR_ITEMS], 100*scale, -1 );
   gtk_widget_set_size_request ( vs->status[VIK_STATUSBAR_ZOOM], 100*scale, -1 );
   gtk_widget_set_size_request ( vs->status[VIK_STATUSBAR_POSITION], 275*scale, -1 );
+  gtk_widget_set_size_request ( vs->status[VIK_STATUSBAR_LOG], 40*scale, -1 );
   // Set minimum overall size
   //  otherwise the individual size_requests above create an implicit overall size,
   //  and so one can't downsize horizontally as much as may be desired when the statusbar is on
@@ -180,10 +192,12 @@ vik_statusbar_set_message ( VikStatusbar *vs, vik_statusbar_type_t field, const
 {
   if ( field >= 0 && field < VIK_STATUSBAR_NUM_TYPES )
   {
-    if ( field == VIK_STATUSBAR_ITEMS || field == VIK_STATUSBAR_ZOOM || field == VIK_STATUSBAR_INFO )
+    if ( field == VIK_STATUSBAR_ITEMS || field == VIK_STATUSBAR_ZOOM || field == VIK_STATUSBAR_LOG )
     {
       gtk_button_set_label ( GTK_BUTTON(vs->status[field]), message);
     }
+    else if ( field == VIK_STATUSBAR_INFO )
+      gtk_label_set_text ( GTK_LABEL(vs->status[field]), message );
     else
     {
     GtkStatusbar *gsb = GTK_STATUSBAR(vs->status[field]);
