@@ -37,9 +37,7 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_MATH_H
 #include <math.h>
-#endif
 
 #include "globals.h"
 #include "viktmsmapsource.h"
@@ -60,13 +58,14 @@ static gdouble _get_lon_max(VikMapSource *self );
 
 static gchar *_get_uri( VikMapSourceDefault *self, MapCoord *src );
 static gchar *_get_hostname( VikMapSourceDefault *self );
-static DownloadFileOptions *_get_download_options( VikMapSourceDefault *self );
+static DownloadFileOptions *_get_download_options( VikMapSourceDefault *self, MapCoord *src );
 
 typedef struct _VikTmsMapSourcePrivate VikTmsMapSourcePrivate;
 struct _VikTmsMapSourcePrivate
 {
   gchar *hostname;
   gchar *url;
+  gchar *custom_http_headers;
   DownloadFileOptions options;
   guint zoom_min; // TMS Zoom level: 0 = Whole World // http://wiki.openstreetmap.org/wiki/Slippy_map_tilenames
   guint zoom_max; // TMS Zoom level: Often 18 for zoomed in.
@@ -76,7 +75,8 @@ struct _VikTmsMapSourcePrivate
   gdouble lon_max; // Degrees
 };
 
-#define VIK_TMS_MAP_SOURCE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIK_TYPE_TMS_MAP_SOURCE, VikTmsMapSourcePrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (VikTmsMapSource, vik_tms_map_source, VIK_TYPE_MAP_SOURCE_DEFAULT);
+#define VIK_TMS_MAP_SOURCE_PRIVATE(o)  (vik_tms_map_source_get_instance_private (VIK_TMS_MAP_SOURCE(o)))
 
 /* properties */
 enum
@@ -85,6 +85,7 @@ enum
 
   PROP_HOSTNAME,
   PROP_URL,
+  PROP_CUSTOM_HTTP_HEADERS,
   PROP_REFERER,
   PROP_FOLLOW_LOCATION,
   PROP_CHECK_FILE_SERVER_TIME,
@@ -96,8 +97,6 @@ enum
   PROP_LON_MAX,
 };
 
-G_DEFINE_TYPE (VikTmsMapSource, vik_tms_map_source, VIK_TYPE_MAP_SOURCE_DEFAULT);
-
 static void
 vik_tms_map_source_init (VikTmsMapSource *self)
 {
@@ -106,10 +105,12 @@ vik_tms_map_source_init (VikTmsMapSource *self)
 
   priv->hostname = NULL;
   priv->url = NULL;
+  priv->custom_http_headers = NULL;
   priv->options.referer = NULL;
   priv->options.follow_location = 0;
   priv->options.check_file = a_check_map_file;
   priv->options.check_file_server_time = FALSE;
+  priv->options.custom_http_headers = NULL;
   priv->zoom_min = 0;
   priv->zoom_max = 18;
   priv->lat_min = -90.0;
@@ -127,15 +128,18 @@ vik_tms_map_source_init (VikTmsMapSource *self)
 static void
 vik_tms_map_source_finalize (GObject *object)
 {
-  VikTmsMapSource *self = VIK_TMS_MAP_SOURCE (object);
-  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (self);
+  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (object);
 
   g_free (priv->hostname);
   priv->hostname = NULL;
   g_free (priv->url);
   priv->url = NULL;
+  g_free (priv->custom_http_headers);
+  priv->custom_http_headers = NULL;
   g_free (priv->options.referer);
   priv->options.referer = NULL;
+  g_free (priv->options.custom_http_headers);
+  priv->options.custom_http_headers = NULL;
 
   G_OBJECT_CLASS (vik_tms_map_source_parent_class)->finalize (object);
 }
@@ -146,8 +150,7 @@ vik_tms_map_source_set_property (GObject      *object,
                                     const GValue *value,
                                     GParamSpec   *pspec)
 {
-  VikTmsMapSource *self = VIK_TMS_MAP_SOURCE (object);
-  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (self);
+  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (object);
 
   switch (property_id)
     {
@@ -161,6 +164,18 @@ vik_tms_map_source_set_property (GObject      *object,
       priv->url = g_value_dup_string (value);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      {
+      g_free (priv->custom_http_headers);
+      const gchar *str = g_value_get_string ( value );
+      // Convert the literal characters '\'+'n' into actual newlines '\n'
+      if ( str )
+        priv->custom_http_headers = g_strcompress ( str );
+      else
+        priv->custom_http_headers = NULL;
+      }
+      break;
+
     case PROP_REFERER:
       g_free (priv->options.referer);
       priv->options.referer = g_value_dup_string (value);
@@ -211,8 +226,7 @@ vik_tms_map_source_get_property (GObject    *object,
                                     GValue     *value,
                                     GParamSpec *pspec)
 {
-  VikTmsMapSource *self = VIK_TMS_MAP_SOURCE (object);
-  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (self);
+  VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE (object);
 
   switch (property_id)
     {
@@ -224,6 +238,10 @@ vik_tms_map_source_get_property (GObject    *object,
       g_value_set_string (value, priv->url);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      g_value_set_string (value, priv->custom_http_headers);
+	  break;
+
     case PROP_REFERER:
       g_value_set_string (value, priv->options.referer);
       break;
@@ -310,6 +328,13 @@ vik_tms_map_source_class_init (VikTmsMapSourceClass *klass)
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_URL, pspec);
 
+	pspec = g_param_spec_string ("custom-http-headers",
+	                             "Custom HTTP Headers",
+	                             "Custom HTTP Headers, use '\n' to separate multiple headers",
+	                             NULL /* default value */,
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_CUSTOM_HTTP_HEADERS, pspec);
+
 	pspec = g_param_spec_string ("referer",
 	                             "Referer",
 	                             "The REFERER string to use in HTTP request",
@@ -320,7 +345,7 @@ vik_tms_map_source_class_init (VikTmsMapSourceClass *klass)
 	pspec = g_param_spec_long ("follow-location",
 	                           "Follow location",
                                "Specifies the number of retries to follow a redirect while downloading a page",
-                               0  /* minimum value */,
+	                           -1, // minimum value (unlimited)
                                G_MAXLONG /* maximum value */,
                                0  /* default value */,
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
@@ -387,8 +412,6 @@ vik_tms_map_source_class_init (VikTmsMapSourceClass *klass)
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_LON_MAX, pspec);
 
-	g_type_class_add_private (klass, sizeof (VikTmsMapSourcePrivate));
-	
 	object_class->finalize = vik_tms_map_source_finalize;
 }
 
@@ -492,12 +515,28 @@ _get_hostname( VikMapSourceDefault *self )
 }
 
 static DownloadFileOptions *
-_get_download_options( VikMapSourceDefault *self )
+_get_download_options( VikMapSourceDefault *self, MapCoord *src )
 {
 	g_return_val_if_fail (VIK_IS_TMS_MAP_SOURCE(self), NULL);
-	
 	VikTmsMapSourcePrivate *priv = VIK_TMS_MAP_SOURCE_PRIVATE(self);
-	return &(priv->options);
+
+	DownloadFileOptions *dfo = g_malloc0 ( sizeof(DownloadFileOptions) );
+	dfo->check_file_server_time = priv->options.check_file_server_time;
+	dfo->use_etag = priv->options.use_etag;
+	dfo->referer = g_strdup ( priv->options.referer );
+	dfo->follow_location = priv->options.follow_location;
+	if ( src )
+		if ( priv->custom_http_headers )
+			dfo->custom_http_headers = g_strdup_printf ( priv->custom_http_headers, 17-src->scale, src->x, src->y );
+		else
+			dfo->custom_http_headers = NULL;
+	else
+		dfo->custom_http_headers = g_strdup ( priv->custom_http_headers );
+	dfo->check_file = priv->options.check_file;
+	dfo->user_pass = g_strdup ( priv->options.user_pass );
+	dfo->convert_file = priv->options.convert_file;
+
+	return dfo;
 }
 
 /**
