@@ -46,6 +46,7 @@ enum
   ITEM_DATA_COLUMN,
   EDITABLE_COLUMN,
   ITEM_TIMESTAMP_COLUMN, // Date timestamp stored in tree model to enable sorting on this value
+  ITEM_NUMBER_COLUMN,    // Stored in tree model to enable sorting on this value
   NUM_COLUMNS
 };
 
@@ -277,6 +278,11 @@ void vik_treeview_item_set_timestamp ( VikTreeview *vt, GtkTreeIter *iter, gdoub
   gtk_tree_store_set ( GTK_TREE_STORE(vt->model), iter, ITEM_TIMESTAMP_COLUMN, timestamp, -1 );
 }
 
+void vik_treeview_item_set_number ( VikTreeview *vt, GtkTreeIter *iter, guint number )
+{
+  gtk_tree_store_set ( GTK_TREE_STORE(vt->model), iter, ITEM_NUMBER_COLUMN, number, -1 );
+}
+
 gpointer vik_treeview_item_get_parent ( VikTreeview *vt, GtkTreeIter *iter )
 {
   gpointer rv;
@@ -351,6 +357,25 @@ void vik_treeview_item_set_visible_tree ( VikTreeview *vt, GtkTreeIter *iter )
   }
 }
 
+static void add_vis_column ( VikTreeview *vt )
+{
+  /* Layer visible */
+  GtkCellRenderer *renderer = gtk_cell_renderer_toggle_new ();
+  g_object_set (G_OBJECT (renderer), "xalign", 0.5, NULL);
+
+  g_signal_connect (renderer, "toggled", G_CALLBACK (vik_treeview_toggled_cb), vt);
+
+  gint col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (vt),
+                                                                 -1, "",
+                                                                 renderer,
+                                                                 "active",
+                                                                 VISIBLE_COLUMN,
+                                                                 NULL);
+
+  GtkTreeViewColumn *column = gtk_tree_view_get_column (GTK_TREE_VIEW (vt), col_offset - 1);
+  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
+                                   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+}
 
 static void vik_treeview_add_columns ( VikTreeview *vt )
 {
@@ -382,6 +407,13 @@ static void vik_treeview_add_columns ( VikTreeview *vt )
                                    GTK_TREE_VIEW_COLUMN_FIXED);
   gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 
+  // Swap the columns around in GTK3
+  // Annoying scrollbars that resize on top of other widgets can obscure the checkbox button
+  //  so when trying to click on the checkbox you end up clicking on the scrollbar.
+#if GTK_CHECK_VERSION (3,0,0)
+  add_vis_column ( vt );
+#endif
+
   /* Layer type */
   renderer = gtk_cell_renderer_pixbuf_new ();
 
@@ -397,23 +429,9 @@ static void vik_treeview_add_columns ( VikTreeview *vt )
   gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
                                    GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
-  /* Layer visible */
-  renderer = gtk_cell_renderer_toggle_new ();
-  g_object_set (G_OBJECT (renderer), "xalign", 0.5, NULL);
-
-  g_signal_connect (renderer, "toggled", G_CALLBACK (vik_treeview_toggled_cb), vt);
-
-  col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (vt),
-							    -1, "",
-							    renderer,
-							    "active",
-							    VISIBLE_COLUMN,
-							    NULL);
-
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (vt), col_offset - 1);
-  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
-                                   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-
+#if !GTK_CHECK_VERSION (3,0,0)
+  add_vis_column ( vt );
+#endif
 
   g_object_set (GTK_TREE_VIEW (vt), "has-tooltip", TRUE, NULL);
   g_signal_connect (GTK_TREE_VIEW (vt), "query-tooltip", G_CALLBACK (vik_treeview_tooltip_cb), vt);
@@ -471,7 +489,7 @@ static void select_cb(GtkTreeSelection *selection, gpointer data)
 			    tmp_type,
 			    vik_window_layers_panel(vw) ) ) {
     /* Redraw required */
-    vik_layers_panel_emit_update ( vik_window_layers_panel(vw) );
+    vik_layers_panel_emit_update ( vik_window_layers_panel(vw), FALSE );
   }
 
 }
@@ -504,7 +522,8 @@ void vik_treeview_init ( VikTreeview *vt )
                                                   G_TYPE_POINTER, // pointer to the layer or sublayer
                                                   G_TYPE_INT,     // type of the sublayer
                                                   G_TYPE_BOOLEAN, // Editable
-                                                  G_TYPE_DOUBLE )); // Timestamp
+                                                  G_TYPE_DOUBLE,  // Timestamp
+                                                  G_TYPE_INT ));  // 'Track' Number (sublayers only)
 
   /* create tree view */
   gtk_tree_selection_set_select_function(gtk_tree_view_get_selection (GTK_TREE_VIEW(vt)), vik_treeview_selection_filter, vt, NULL);
@@ -697,9 +716,10 @@ void vik_treeview_add_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTree
     ITEM_PARENT_COLUMN, parent,
     ITEM_POINTER_COLUMN, item,
     ITEM_DATA_COLUMN, data,
-    EDITABLE_COLUMN, parent == NULL ? FALSE : TRUE,
+    EDITABLE_COLUMN, TRUE,
     ICON_COLUMN, layer_type >= 0 ? vt->layer_type_icons[layer_type] : NULL,
     ITEM_TIMESTAMP_COLUMN, timestamp,
+    ITEM_NUMBER_COLUMN, 0,
     -1 );
 }
 
@@ -729,11 +749,12 @@ void vik_treeview_insert_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkT
                        EDITABLE_COLUMN, TRUE,
                        ICON_COLUMN, layer_type >= 0 ? vt->layer_type_icons[layer_type] : NULL,
                        ITEM_TIMESTAMP_COLUMN, timestamp,
+                       ITEM_NUMBER_COLUMN, 0,
                        -1 );
 }
 
 void vik_treeview_add_sublayer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTreeIter *iter, const gchar *name, gpointer parent, gpointer item,
-                                 gint data, GdkPixbuf *icon, gboolean editable, gdouble timestamp )
+                                 gint data, GdkPixbuf *icon, gboolean editable, gdouble timestamp, guint number )
 {
   g_assert ( iter != NULL );
 
@@ -748,6 +769,7 @@ void vik_treeview_add_sublayer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkT
                        EDITABLE_COLUMN, editable,
                        ICON_COLUMN, icon,
                        ITEM_TIMESTAMP_COLUMN, timestamp,
+                       ITEM_NUMBER_COLUMN, number,
                        -1 );
 }
 
@@ -757,6 +779,7 @@ typedef struct _SortTuple
   gint offset;
   gchar *name;
   gdouble timestamp;
+  guint number;
 } SortTuple;
 
 /**
@@ -776,7 +799,7 @@ static gint sort_tuple_compare ( gconstpointer a, gconstpointer b, gpointer orde
     if ( GPOINTER_TO_INT(order) == VL_SO_ALPHABETICAL_DESCENDING )
       answer = -answer;
   }
-  else {
+  else if ( GPOINTER_TO_INT(order) < VL_SO_NUMBER_ASCENDING ) {
     // Date comparison
     gboolean ans = ( sa->timestamp > sb->timestamp );
     if ( ans )
@@ -784,6 +807,14 @@ static gint sort_tuple_compare ( gconstpointer a, gconstpointer b, gpointer orde
     // Invert sort order for descending order
     if ( GPOINTER_TO_INT(order) == VL_SO_DATE_DESCENDING )
       answer = -answer;
+  } else {
+    // Number comparison
+    gboolean ans = ( sa->number > sb->number );
+    if ( ans )
+      answer = 1;
+    // Invert sort order for descending order
+    if ( GPOINTER_TO_INT(order) == VL_SO_NUMBER_DESCENDING )
+      answer = -answer;
   }
   return answer;
 }
@@ -830,6 +861,7 @@ void vik_treeview_sort_children ( VikTreeview *vt, GtkTreeIter *parent, vik_laye
     sort_array[ii].offset = ii;
     gtk_tree_model_get ( model, &child, NAME_COLUMN, &(sort_array[ii].name), -1 );
     gtk_tree_model_get ( model, &child, ITEM_TIMESTAMP_COLUMN, &(sort_array[ii].timestamp), -1 );
+    gtk_tree_model_get ( model, &child, ITEM_NUMBER_COLUMN, &(sort_array[ii].number), -1 );
     ii++;
   } while ( gtk_tree_model_iter_next (model, &child) );
 
@@ -869,7 +901,6 @@ static gboolean vik_treeview_drag_data_received (GtkTreeDragDest *drag_dest, Gtk
   GtkTreeModel *tree_model;
   GtkTreeModel *src_model = NULL;
   GtkTreePath *src_path = NULL, *dest_cp = NULL;
-  gboolean retval = FALSE;
   GtkTreeIter src_iter, root_iter, dest_parent;
   VikTreeview *vt;
   VikLayer *vl;
@@ -906,7 +937,6 @@ static gboolean vik_treeview_drag_data_received (GtkTreeDragDest *drag_dest, Gtk
     TREEVIEW_GET(tree_model, &root_iter, ITEM_POINTER_COLUMN, &vl);
     vt = vl->vt;
 
-
     if (gtk_tree_path_get_depth(dest_cp)>1) { /* can't be sibling of top layer */
       VikLayer *vl_src, *vl_dest;
 
@@ -924,8 +954,13 @@ static gboolean vik_treeview_drag_data_received (GtkTreeDragDest *drag_dest, Gtk
 
       /* TODO: might want to allow different types, and let the clients handle how they want */
       if (vl_src->type == vl_dest->type && vik_layer_get_interface(vl_dest->type)->drag_drop_request) {
-	//	g_print("moving an item from layer '%s' into layer '%s'\n", vl_src->name, vl_dest->name);
-	vik_layer_get_interface(vl_dest->type)->drag_drop_request(vl_src, vl_dest, &src_iter, dest);
+        // g_print("moving an item from layer '%s' into layer '%s'\n", vl_src->name, vl_dest->name);
+        vik_layer_get_interface(vl_dest->type)->drag_drop_request(vl_src, vl_dest, &src_iter, dest);
+
+        // Update as things have changed - this needs to be performed at the root level
+        //  due to visibility settings of the heirachy effects what needs drawing (and removing)
+        //  so can't be in drag_drop_request() as an update on either vl_src or vl_dest layers isn't enough
+        vik_layer_emit_update ( vl, TRUE );
       }    
     }
   }
@@ -936,7 +971,7 @@ static gboolean vik_treeview_drag_data_received (GtkTreeDragDest *drag_dest, Gtk
   if (src_path)
     gtk_tree_path_free (src_path);
 
-  return retval;
+  return FALSE;
 }
 
 /* 
