@@ -46,6 +46,13 @@ enum {
   VIK_TRW_LAYER_SUBLAYER_ROUTE
 };
 
+typedef enum {
+  GPX_V1_0 = 0,
+  GPX_V1_1,
+  // ATM V1.1 is implicitly Garmin...
+  //GPX_V1_1_GARMIN,
+} gpx_version_t;
+
 typedef struct _VikTrwLayerClass VikTrwLayerClass;
 struct _VikTrwLayerClass
 {
@@ -63,12 +70,22 @@ typedef struct {
   //gboolean has_time;
   gchar *timestamp; // TODO: Consider storing as proper time_t.
   gchar *keywords; // TODO: handling/storing a GList of individual tags?
+  gchar *url;
 } VikTRWMetadata;
 
 VikTRWMetadata *vik_trw_metadata_new();
-void vik_trw_metadata_free ( VikTRWMetadata *metadata);
+void vik_trw_metadata_free ( VikTRWMetadata *metadata );
 VikTRWMetadata *vik_trw_layer_get_metadata ( VikTrwLayer *vtl );
-void vik_trw_layer_set_metadata ( VikTrwLayer *vtl, VikTRWMetadata *metadata);
+void vik_trw_layer_set_metadata ( VikTrwLayer *vtl, VikTRWMetadata *metadata );
+
+gpx_version_t vik_trw_layer_get_gpx_version ( VikTrwLayer *vtl );
+void vik_trw_layer_set_gpx_version ( VikTrwLayer *vtl, gpx_version_t value );
+gchar *vik_trw_layer_get_gpx_header ( VikTrwLayer *vtl );
+void vik_trw_layer_set_gpx_header ( VikTrwLayer *vtl, gchar* value );
+gchar *vik_trw_layer_get_gpx_extensions ( VikTrwLayer *vtl );
+void vik_trw_layer_set_gpx_extensions ( VikTrwLayer *vtl, gchar *value);
+
+void vik_trwlayer_uninit ();
 
 gboolean vik_trw_layer_find_date ( VikTrwLayer *vtl, const gchar *date_str, VikCoord *position, VikViewport *vvp, gboolean do_tracks, gboolean select );
 
@@ -100,6 +117,8 @@ GHashTable *vik_trw_layer_get_tracks ( VikTrwLayer *l );
 GHashTable *vik_trw_layer_get_routes ( VikTrwLayer *l );
 GHashTable *vik_trw_layer_get_waypoints ( VikTrwLayer *l );
 gboolean vik_trw_layer_is_empty ( VikTrwLayer *vtl );
+VikTrack *vik_trw_layer_get_only_track ( VikTrwLayer *vtl );
+LatLonBBox vik_trw_layer_get_bbox ( VikTrwLayer *vtl );
 
 gboolean vik_trw_layer_new_waypoint ( VikTrwLayer *vtl, GtkWindow *w, const VikCoord *def_coord );
 
@@ -115,6 +134,11 @@ void trw_layer_cancel_tps_of_track ( VikTrwLayer *vtl, VikTrack *trk );
 void vik_trw_layer_goto_track_prev_point ( VikTrwLayer *vtl );
 void vik_trw_layer_goto_track_next_point ( VikTrwLayer *vtl );
 
+// Functions for editing primarily from the Select Tool
+void vik_trw_layer_insert_tp_beside_current_tp ( VikTrwLayer *vtl, gboolean before );
+void vik_trw_layer_delete_trackpoint_selected ( VikTrwLayer *vtl );
+void vik_trw_layer_delete_waypoint ( VikTrwLayer *vtl, VikWaypoint *wpt );
+
 void vik_trw_layer_reset_waypoints ( VikTrwLayer *vtl );
 
 void vik_trw_layer_draw_highlight ( VikTrwLayer *vtl, VikViewport *vvp );
@@ -149,15 +173,19 @@ void trw_layer_verify_thumbnails ( VikTrwLayer *vtl );
 // Other functions only for use by other trw_layer subwindows
 gchar *trw_layer_new_unique_sublayer_name ( VikTrwLayer *vtl, gint sublayer_type, const gchar *name );
 void trw_layer_waypoint_rename ( VikTrwLayer *vtl, VikWaypoint *wp, const gchar *new_name );
-void trw_layer_waypoint_reset_icon ( VikTrwLayer *vtl, VikWaypoint *wp );
 void trw_layer_calculate_bounds_waypoints ( VikTrwLayer *vtl );
 void trw_layer_calculate_bounds_tracks ( VikTrwLayer *vtl );
+void trw_layer_waypoint_properties_changed ( VikTrwLayer *vtl, VikWaypoint *wp );
+void trw_layer_wpwin_set ( VikTrwLayer *vtl, VikWaypoint *wp, gpointer wpwin );
+gboolean trw_layer_modified ( VikTrwLayer *vtl );
 
 gboolean vik_trw_layer_get_tracks_visibility ( VikTrwLayer *vtl );
 gboolean vik_trw_layer_get_routes_visibility ( VikTrwLayer *vtl );
 gboolean vik_trw_layer_get_waypoints_visibility ( VikTrwLayer *vtl );
 
-void trw_layer_update_treeview ( VikTrwLayer *vtl, VikTrack *trk );
+gboolean vik_trw_layer_get_prefer_gps_speed ( VikTrwLayer *vtl );
+
+void trw_layer_update_treeview ( VikTrwLayer *vtl, VikTrack *trk, gboolean do_sort );
 
 void trw_layer_dialog_shift ( VikTrwLayer *vtl, GtkWindow *dialog, VikCoord *coord, gboolean vertical );
 
@@ -189,6 +217,12 @@ void vik_trw_layer_trackpoint_draw ( VikTrwLayer *vtl, VikViewport *vvp, VikTrac
 
 #define VIK_SETTINGS_LIST_DATE_FORMAT "list_date_format"
 
+typedef enum _VikTRWDataType
+{
+ VIKING_WAYPOINT = 0,
+ VIKING_TRACK,
+} VikTRWDataTypeT;
+
 G_END_DECLS
 
 #endif
