@@ -27,7 +27,6 @@
 #include "viking.h"
 #include "viktrwlayer_export.h"
 #include "gpx.h"
-#include "preferences.h"
 
 static gchar *last_folder_uri = NULL;
 
@@ -39,14 +38,12 @@ static gchar *last_folder_uri = NULL;
 
 static VikLayerParam io_prefs[] = {
   { VIK_LAYER_NUM_TYPES, VIK_PREFS_EXPORT_DEVICE_SIMPLIFY, VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Auto Device GPX Simplify:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL,
-     N_("GPX saves to certain devices will be simplified for device compatibility."), NULL, NULL, NULL },
+     N_("GPX saves to certain devices will be simplified for device compatibility."), vik_lpd_false_default, NULL, NULL },
 };
 
 void vik_trw_layer_export_init ()
 {
-  VikLayerParamData tmp;
-  tmp.b = FALSE;
-  a_preferences_register(&io_prefs[0], tmp, VIKING_PREFERENCES_IO_GROUP_KEY);
+  a_preferences_register ( &io_prefs[0], (VikLayerParamData){0}, VIKING_PREFERENCES_IO_GROUP_KEY );
 }
 
 static gboolean gpx_export_simplify_layer ( VikTrwLayer *vtl, const gchar *fn, VikTrack* trk )
@@ -66,14 +63,11 @@ static gboolean gpx_export_simplify_layer ( VikTrwLayer *vtl, const gchar *fn, V
   if ( trk )
     got_route = trk->is_route;
 
-  if ( !got_route ) {
-    if ( g_hash_table_size (vik_trw_layer_get_routes(vtl)) ) {
-      gint tmp;
-      if ( a_settings_get_integer ( VIK_SETTINGS_EXPORT_DEVICE_SIMPLIFY_ROUTEPOINT_LIMIT, &tmp ) )
-         limit = tmp;
-      else
-         limit = 250; // Route capacity is relatively limited
-    }
+  if ( got_route || g_hash_table_size(vik_trw_layer_get_routes(vtl)) ) {
+    if ( a_settings_get_integer ( VIK_SETTINGS_EXPORT_DEVICE_SIMPLIFY_ROUTEPOINT_LIMIT, &tmp ) )
+      limit = tmp;
+    else
+      limit = 250; // Route capacity is relatively limited
     // NB If there are tracks and trackpoints much > (route)limit,
     //  could warn that tracks may be overily simplified
     // and ask to continue...
@@ -159,10 +153,18 @@ static gboolean gpx_export ( VikTrwLayer *vtl, const gchar *fn, VikTrack* trk )
   return a_file_export ( vtl, fn, FILE_TYPE_GPX, trk, trk ? TRUE : FALSE );
 }
 
-void vik_trw_layer_export ( VikTrwLayer *vtl, const gchar *title, const gchar* default_name, VikTrack* trk, VikFileType_t file_type )
+/**
+ * vik_trw_layer_export:
+ *
+ * vl: The #VikLayer to operate on
+ *
+ * Note since we now use the #VikLayer type rather than #VikTrwLayer,
+ *  this function could be renamed and shifted into the appropriate file.
+ * However for now leave it as is.
+ */
+void vik_trw_layer_export ( VikLayer *vl, const gchar *title, const gchar* default_name, VikTrack* trk, VikFileType_t file_type )
 {
   GtkWidget *file_selector;
-  const gchar *fn;
   gboolean failed = FALSE;
   file_selector = gtk_file_chooser_dialog_new (title,
                                                NULL,
@@ -175,9 +177,34 @@ void vik_trw_layer_export ( VikTrwLayer *vtl, const gchar *title, const gchar* d
 
   gtk_file_chooser_set_current_name ( GTK_FILE_CHOOSER(file_selector), default_name );
 
+  GtkWidget *dialog = file_selector;
+  GtkFileFilter *filter;
+  switch ( file_type ) {
+  case FILE_TYPE_GPX:
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name( filter, _("GPX") );
+    gtk_file_filter_add_mime_type ( filter, "gpx+xml");
+    gtk_file_filter_add_pattern ( filter, "*.gpx" );
+    gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER(dialog), filter );
+    gtk_file_chooser_set_filter ( GTK_FILE_CHOOSER(dialog), filter );
+    break;
+  case FILE_TYPE_KML:
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name( filter, _("Google Earth") );
+    gtk_file_filter_add_mime_type ( filter, "application/vnd.google-earth.kml+xml");
+    gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER(dialog), filter);
+    gtk_file_chooser_set_filter ( GTK_FILE_CHOOSER(dialog), filter );
+    break;
+  default: break;
+  }
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name( filter, _("All") );
+  gtk_file_filter_add_pattern ( filter, "*" );
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+
   while ( gtk_dialog_run ( GTK_DIALOG(file_selector) ) == GTK_RESPONSE_ACCEPT )
   {
-    fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
+    gchar *fn = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER(file_selector) );
     if ( g_file_test ( fn, G_FILE_TEST_EXISTS ) == FALSE ||
          a_dialog_yes_or_no ( GTK_WINDOW(file_selector), _("The file \"%s\" exists, do you wish to overwrite it?"), a_file_basename ( fn ) ) )
     {
@@ -185,19 +212,30 @@ void vik_trw_layer_export ( VikTrwLayer *vtl, const gchar *title, const gchar* d
       last_folder_uri = gtk_file_chooser_get_current_folder_uri ( GTK_FILE_CHOOSER(file_selector) );
 
       gtk_widget_hide ( file_selector );
-      vik_window_set_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vtl)) );
-      // Don't Export invisible items - unless requested on this specific track
-      if ( file_type == FILE_TYPE_GPX )
-        failed = ! gpx_export ( vtl, fn, trk );
-      else
-        failed = ! a_file_export ( vtl, fn, file_type, trk, trk ? TRUE : FALSE );
-      vik_window_clear_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vtl)) );
+      vik_window_set_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+      switch ( vl->type ) {
+      case VIK_LAYER_TRW:
+        // Don't Export invisible items - unless requested on this specific track
+        if ( file_type == FILE_TYPE_GPX )
+          failed = ! gpx_export ( VIK_TRW_LAYER(vl), fn, trk );
+        else
+          failed = ! a_file_export ( VIK_TRW_LAYER(vl), fn, file_type, trk, trk ? TRUE : FALSE );
+        break;
+      case VIK_LAYER_AGGREGATE:
+        failed = ! vik_aggregate_layer_export_gpx_main ( VIK_AGGREGATE_LAYER(vl), fn );
+        break;
+      default:
+        g_critical ( "%s:%s %d", __FUNCTION__, "Export not expected for layer type", vl->type ); break;
+      }
+      vik_window_clear_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+      g_free ( fn );
       break;
     }
+    g_free ( fn );
   }
   gtk_widget_destroy ( file_selector );
   if ( failed )
-    a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_LAYER(vtl), _("The filename you requested could not be opened for writing.") );
+    a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_LAYER(vl), _("The filename you requested could not be opened for writing.") );
 }
 
 
@@ -208,7 +246,7 @@ void vik_trw_layer_export ( VikTrwLayer *vtl, const gchar *title, const gchar* d
 void vik_trw_layer_export_external_gpx ( VikTrwLayer *vtl, const gchar* external_program )
 {
   // Don't Export invisible items
-  static GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE };
+  GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE, vik_trw_layer_get_gpx_version(vtl) };
   gchar *name_used = a_gpx_write_tmp_file ( vtl, &options );
 
   if ( name_used ) {
@@ -228,6 +266,19 @@ void vik_trw_layer_export_external_gpx ( VikTrwLayer *vtl, const gchar* external
     a_dialog_error_msg (VIK_GTK_WINDOW_FROM_LAYER(vtl), _("Could not create temporary file for export.") );
 }
 
+static guint last_mode_index = 0;
+static BabelMode last_mode = { 0, 0, 0, 0, 0, 0 };
+static gchar* last_suboptions = NULL;
+
+gboolean BabelMode_is_equal ( BabelMode left, BabelMode right )
+{
+  return ( left.waypointsRead == right.waypointsRead &&
+           left.waypointsWrite == right.waypointsWrite &&
+           left.tracksRead == right.tracksRead &&
+           left.tracksWrite == right.tracksWrite &&
+           left.routesRead == right.routesRead &&
+           left.routesWrite == right.routesWrite );
+}
 
 void vik_trw_layer_export_gpsbabel ( VikTrwLayer *vtl, const gchar *title, const gchar* default_name )
 {
@@ -243,7 +294,6 @@ void vik_trw_layer_export_gpsbabel ( VikTrwLayer *vtl, const gchar *title, const
   }
 
   GtkWidget *file_selector;
-  const gchar *fn;
   gboolean failed = FALSE;
   file_selector = gtk_file_chooser_dialog_new (title,
                                                NULL,
@@ -261,39 +311,46 @@ void vik_trw_layer_export_gpsbabel ( VikTrwLayer *vtl, const gchar *title, const
   GtkWidget *babel_selector = a_babel_ui_file_type_selector_new ( mode );
   GtkWidget *label = gtk_label_new(_("File format:"));
   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start ( GTK_BOX(hbox), label, TRUE, TRUE, 0 );
-  gtk_box_pack_start ( GTK_BOX(hbox), babel_selector, TRUE, TRUE, 0 );
-  gtk_widget_show (babel_selector);
-  gtk_widget_show (label);
-  gtk_widget_show_all (hbox);
+  gtk_box_pack_start ( GTK_BOX(hbox), label, TRUE, TRUE, 2 );
+  gtk_box_pack_start ( GTK_BOX(hbox), babel_selector, TRUE, TRUE, 2 );
 
   gtk_widget_set_tooltip_text( babel_selector, _("Select the file format.") );
 
   GtkWidget *babel_modes = a_babel_ui_modes_new(mode.tracksWrite, mode.routesWrite, mode.waypointsWrite);
-  gtk_widget_show (babel_modes);
 
   gtk_widget_set_tooltip_text( babel_modes, _("Select the information to process.\n"
       "Warning: the behavior of these switches is highly dependent of the file format selected.\n"
       "Please, refer to GPSbabel if unsure.") );
 
+  GtkWidget *hbox2 = gtk_hbox_new(FALSE, 0);
+  GtkWidget *link = gtk_link_button_new_with_label ( "https://www.gpsbabel.org/capabilities.html", _("GPSBabel suboptions") );
+  GtkWidget *entry = ui_entry_new ( BabelMode_is_equal(mode, last_mode) ? last_suboptions : NULL, GTK_ENTRY_ICON_SECONDARY );
+  gtk_box_pack_start ( GTK_BOX(hbox2), link, FALSE, FALSE, 2 );
+  gtk_box_pack_start ( GTK_BOX(hbox2), entry, TRUE, TRUE, 2 );
+
   GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start ( GTK_BOX(vbox), hbox, TRUE, TRUE, 0 );
-  gtk_box_pack_start ( GTK_BOX(vbox), babel_modes, TRUE, TRUE, 0 );
-  gtk_widget_show_all (vbox);
+  gtk_box_pack_start ( GTK_BOX(vbox), hbox, TRUE, TRUE, 2 );
+  gtk_box_pack_start ( GTK_BOX(vbox), babel_modes, TRUE, TRUE, 2 );
+  gtk_box_pack_start ( GTK_BOX(vbox), hbox2, TRUE, TRUE, 2 );
 
   gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(file_selector), vbox);
 
   /* Add some dynamic: only allow dialog's validation when format selection is done */
   g_signal_connect (babel_selector, "changed", G_CALLBACK(a_babel_ui_type_selector_dialog_sensitivity_cb), file_selector);
+  // NB the selection list changes according to available modes, so only restore selection index if the same mode
+  if ( BabelMode_is_equal(mode, last_mode) )
+    gtk_combo_box_set_active ( GTK_COMBO_BOX(babel_selector), last_mode_index );
   /* Manually call the callback to fix the state */
   a_babel_ui_type_selector_dialog_sensitivity_cb ( GTK_COMBO_BOX(babel_selector), file_selector);
 
   /* Set possible name of the file */
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(file_selector), default_name);
 
+  gtk_widget_show_all ( file_selector );
+
   while ( gtk_dialog_run ( GTK_DIALOG(file_selector) ) == GTK_RESPONSE_ACCEPT )
   {
-    fn = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(file_selector) );
+    gchar *fn = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(file_selector) );
     if ( g_file_test ( fn, G_FILE_TEST_EXISTS ) == FALSE ||
          a_dialog_yes_or_no ( GTK_WINDOW(file_selector), _("The file \"%s\" exists, do you wish to overwrite it?"), a_file_basename ( fn ) ) )
     {
@@ -305,11 +362,19 @@ void vik_trw_layer_export_gpsbabel ( VikTrwLayer *vtl, const gchar *title, const
         vik_window_set_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vtl)) );
         gboolean tracks, routes, waypoints;
         a_babel_ui_modes_get( babel_modes, &tracks, &routes, &waypoints );
-        failed = ! a_file_export_babel ( vtl, fn, active->name, tracks, routes, waypoints );
+        failed = ! a_file_export_babel ( vtl, fn, active->name, tracks, routes, waypoints, gtk_entry_get_text(GTK_ENTRY(entry)) );
         vik_window_clear_busy_cursor ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vtl)) );
+        g_free ( fn );
+        // Save selections for usage next time
+        last_mode_index = gtk_combo_box_get_active ( GTK_COMBO_BOX(babel_selector) );
+	last_mode = mode;
+        if ( last_suboptions )
+          g_free ( last_suboptions );
+        last_suboptions = g_strdup ( gtk_entry_get_text(GTK_ENTRY(entry)) );
         break;
       }
     }
+    g_free ( fn );
   }
   //babel_ui_selector_destroy(babel_selector);
   gtk_widget_destroy ( file_selector );
