@@ -22,11 +22,13 @@
 #include "viking.h"
 #include "viktrwlayer_tpwin.h"
 #include "vikdatetime_edit_dialog.h"
+#include "degrees_converters.h"
 
 // For simplicity these are global values
 //  (i.e. would get shared between multiple windows)
 static gint width = 0;
 static gint height = 0;
+static gboolean expanded = FALSE;
 
 struct _VikTrwLayerTpwin {
   GtkDialog parent;
@@ -34,6 +36,11 @@ struct _VikTrwLayerTpwin {
   GtkWidget *trkpt_name;
   GtkWidget *time;
   GtkLabel *course, *diff_dist, *diff_time, *diff_speed, *speed, *hdop, *vdop, *pdop, *sat;
+  GtkWidget *tabs;
+  GtkWidget *extsw;
+  GtkLabel *extlab;
+  GtkWidget *expander;
+  GtkLabel *heart_rate, *cadence, *temp, *power;
   // Previously these buttons were in a glist, however I think the ordering behaviour is implicit
   //  thus control manually to ensure operating on the correct button
   GtkWidget *button_close;
@@ -77,6 +84,8 @@ GType vik_trw_layer_tpwin_get_type (void)
 static void tpwin_update_times ( VikTrwLayerTpwin *tpwin, VikTrackpoint *tp )
 {
   if ( !isnan(tp->timestamp) ) {
+    if ( gtk_button_get_image ( GTK_BUTTON(tpwin->time) ) )
+      gtk_button_set_image ( GTK_BUTTON(tpwin->time), NULL );
     gtk_spin_button_set_value ( tpwin->ts, tp->timestamp );
     time_t time = round ( tp->timestamp );
     gchar *msg = vu_get_time_string ( &time, "%c", &(tp->coord), NULL );
@@ -85,7 +94,7 @@ static void tpwin_update_times ( VikTrwLayerTpwin *tpwin, VikTrackpoint *tp )
   }
   else {
     gtk_spin_button_set_value ( tpwin->ts, 0 );
-    gtk_button_set_label ( GTK_BUTTON(tpwin->time), "" );
+    gtk_button_set_label ( GTK_BUTTON(tpwin->time), NULL );
   }
 }
 
@@ -124,6 +133,7 @@ static void tpwin_sync_alt_to_tp ( VikTrwLayerTpwin *tpwin )
       tpwin->cur_tp->altitude = gtk_spin_button_get_value ( tpwin->alt );
       g_critical("Houston, we've had a problem. height=%d", height_units);
     }
+    gtk_dialog_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_DATA_CHANGED );
   }
 }
 
@@ -133,14 +143,41 @@ static void tpwin_sync_alt_to_tp ( VikTrwLayerTpwin *tpwin )
 static void tpwin_sync_ts_to_tp ( VikTrwLayerTpwin *tpwin )
 {
   if ( tpwin->cur_tp && (!tpwin->sync_to_tp_block) ) {
-    tpwin->cur_tp->timestamp = gtk_spin_button_get_value ( tpwin->ts );
-
-    tpwin_update_times ( tpwin, tpwin->cur_tp );
+    gtk_widget_set_sensitive ( GTK_WIDGET(tpwin->ts), !isnan(tpwin->cur_tp->timestamp) );
+    if ( !isnan(tpwin->cur_tp->timestamp) ) {
+      tpwin->cur_tp->timestamp = gtk_spin_button_get_value ( tpwin->ts );
+      tpwin_update_times ( tpwin, tpwin->cur_tp );
+      gtk_dialog_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_DATA_CHANGED );
+    }
   }
 }
 
 static time_t last_edit_time = 0;
 
+/**
+ * time_remove_cb:
+ */
+static void time_remove_cb ( VikTrwLayerTpwin *tpwin )
+{
+  tpwin->cur_tp->timestamp = NAN;
+  GtkWidget *img = gtk_image_new_from_stock ( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU );
+  gtk_button_set_image ( GTK_BUTTON(tpwin->time), img );
+  tpwin_update_times ( tpwin, tpwin->cur_tp );
+  gtk_dialog_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_DATA_CHANGED );
+}
+
+/**
+ * time_menu:
+ */
+static void time_menu ( GtkWidget *widget, guint button, VikTrwLayerTpwin *tpwin )
+{
+  GtkWidget *menu = gtk_menu_new();
+  (void)vu_menu_add_item ( GTK_MENU(menu), NULL, GTK_STOCK_COPY, G_CALLBACK(vu_copy_label), widget );
+  (void)vu_menu_add_item ( GTK_MENU(menu), NULL, GTK_STOCK_REMOVE, G_CALLBACK(time_remove_cb), tpwin );
+  gtk_widget_show_all ( GTK_WIDGET(menu) );
+  gtk_menu_popup ( GTK_MENU(menu), NULL, NULL, NULL, NULL, button, gtk_get_current_event_time() );
+}
+
 /**
  * tpwin_sync_time_to_tp:
  *
@@ -151,15 +188,15 @@ static void tpwin_sync_time_to_tp ( GtkWidget* widget, GdkEventButton *event, Vi
     return;
 
   if ( event->button == 3 ) {
-    // On right click and when a time is available, allow a method to copy the displayed time as text
+    // On right click and when a time is available, allow a methods to copy or remove the time
     if ( !gtk_button_get_image ( GTK_BUTTON(widget) ) ) {
-       vu_copy_label_menu ( widget, event->button );
+      time_menu ( widget, event->button, tpwin );
     }
     return;
   }
-  else if ( event->button == 2 ) {
+
+  if ( event->button == 2 )
     return;
-  }
 
   if ( !tpwin->cur_tp || tpwin->sync_to_tp_block )
     return;
@@ -189,12 +226,19 @@ static void tpwin_sync_time_to_tp ( GtkWidget* widget, GdkEventButton *event, Vi
     gtk_button_set_image ( GTK_BUTTON(tpwin->time), NULL );
 
   tpwin_update_times ( tpwin, tpwin->cur_tp );
+  gtk_dialog_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_DATA_CHANGED );
 }
 
 static gboolean tpwin_set_name ( VikTrwLayerTpwin *tpwin )
 {
   if ( tpwin->cur_tp && (!tpwin->sync_to_tp_block) ) {
-    vik_trackpoint_set_name ( tpwin->cur_tp, gtk_entry_get_text(GTK_ENTRY(tpwin->trkpt_name)) );
+    const gchar *str = gtk_entry_get_text(GTK_ENTRY(tpwin->trkpt_name));
+    if ( g_strcmp0(tpwin->cur_tp->name, str) ) {
+      if ( tpwin->cur_tp->name || (str && strlen(str)) ) {
+        vik_trackpoint_set_name ( tpwin->cur_tp, str );
+        gtk_dialog_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_DATA_CHANGED );
+      }
+    }
   }
   return FALSE;
 }
@@ -211,8 +255,9 @@ static gboolean configure_event ( GtkWidget *widget, GdkEventConfigure *event, V
       height = event->height;
 
     // Allow sizing back down to the minimum
+    // Possibly because last row contains a button, using BASE_SIZE rather than MIN_SIZE hint is better
     GdkGeometry geom = { event->width, event->height, 0, 0, 0, 0, 0, 0, 0, 0, GDK_GRAVITY_STATIC };
-    gtk_window_set_geometry_hints ( GTK_WINDOW(widget), widget, &geom, GDK_HINT_MIN_SIZE );
+    gtk_window_set_geometry_hints ( GTK_WINDOW(widget), widget, &geom, GDK_HINT_BASE_SIZE );
 
     // Restore previous size (if one was set)
     gtk_window_resize ( GTK_WINDOW(widget), width, height );
@@ -220,6 +265,8 @@ static gboolean configure_event ( GtkWidget *widget, GdkEventConfigure *event, V
   return FALSE;
 }
 
+#define TPWIN_PAD 1
+
 VikTrwLayerTpwin *vik_trw_layer_tpwin_new ( GtkWindow *parent )
 {
   static gchar *left_label_texts[] = { N_("<b>Name:</b>"),
@@ -252,20 +299,8 @@ VikTrwLayerTpwin *vik_trw_layer_tpwin_new ( GtkWindow *parent )
   tpwin->button_back = gtk_dialog_add_button ( GTK_DIALOG(tpwin), GTK_STOCK_GO_BACK, VIK_TRW_LAYER_TPWIN_BACK);
   tpwin->button_forward = gtk_dialog_add_button ( GTK_DIALOG(tpwin), GTK_STOCK_GO_FORWARD, VIK_TRW_LAYER_TPWIN_FORWARD);
 
-  /*
-  gtk_dialog_add_buttons ( GTK_DIALOG(tpwin),
-      GTK_STOCK_CLOSE, VIK_TRW_LAYER_TPWIN_CLOSE,
-      _("_Insert After"), VIK_TRW_LAYER_TPWIN_INSERT,
-      GTK_STOCK_DELETE, VIK_TRW_LAYER_TPWIN_DELETE,
-      _("Split Here"), VIK_TRW_LAYER_TPWIN_SPLIT,
-      GTK_STOCK_GO_BACK, VIK_TRW_LAYER_TPWIN_BACK,
-      GTK_STOCK_GO_FORWARD, VIK_TRW_LAYER_TPWIN_FORWARD,
-      NULL );
-  tpwin->buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(tpwin)->action_area));
-  */
-
   /* main track info */
-  left_vbox = a_dialog_create_label_vbox ( left_label_texts, G_N_ELEMENTS(left_label_texts), 1, 3 );
+  left_vbox = a_dialog_create_label_vbox ( left_label_texts, G_N_ELEMENTS(left_label_texts), 1, TPWIN_PAD );
 
   tpwin->trkpt_name = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
   g_signal_connect_swapped ( G_OBJECT(tpwin->trkpt_name), "focus-out-event", G_CALLBACK(tpwin_set_name), tpwin );
@@ -292,16 +327,16 @@ VikTrwLayerTpwin *vik_trw_layer_tpwin_new ( GtkWindow *parent )
   g_signal_connect_swapped ( G_OBJECT(tpwin->ts), "value-changed", G_CALLBACK(tpwin_sync_ts_to_tp), tpwin );
 
   right_vbox = gtk_vbox_new ( TRUE, 1 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->trkpt_name), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->lat), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->lon), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->alt), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->course), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->ts), TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->time), TRUE, TRUE, 3 );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->trkpt_name), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->lat), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->lon), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->alt), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->course), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->ts), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(right_vbox), GTK_WIDGET(tpwin->time), TRUE, TRUE, TPWIN_PAD );
 
   /* diff info */
-  diff_left_vbox = a_dialog_create_label_vbox ( right_label_texts, G_N_ELEMENTS(right_label_texts), 1, 3 );
+  diff_left_vbox = a_dialog_create_label_vbox ( right_label_texts, G_N_ELEMENTS(right_label_texts), 1, TPWIN_PAD );
 
   tpwin->diff_dist = GTK_LABEL(ui_label_new_selectable(NULL));
   tpwin->diff_time = GTK_LABEL(ui_label_new_selectable(NULL));
@@ -314,30 +349,66 @@ VikTrwLayerTpwin *vik_trw_layer_tpwin_new ( GtkWindow *parent )
   tpwin->sat = GTK_LABEL(ui_label_new_selectable(NULL));
 
   diff_right_vbox = gtk_vbox_new ( TRUE, 1 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_dist), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_time), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_speed), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->speed), FALSE, FALSE, 3 );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_dist), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_time), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->diff_speed), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->speed), FALSE, FALSE, TPWIN_PAD );
 
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->vdop), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->hdop), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->pdop), FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->sat), FALSE, FALSE, 3 );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->vdop), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->hdop), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->pdop), FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(diff_right_vbox), GTK_WIDGET(tpwin->sat), FALSE, FALSE, TPWIN_PAD );
 
   main_hbox = gtk_hbox_new( FALSE, 0 );
-  gtk_box_pack_start ( GTK_BOX(main_hbox), left_vbox, FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(main_hbox), right_vbox, TRUE, TRUE, 3 );
-  gtk_box_pack_start ( GTK_BOX(main_hbox), diff_left_vbox, FALSE, FALSE, 3 );
-  gtk_box_pack_start ( GTK_BOX(main_hbox), diff_right_vbox, FALSE, FALSE, 3 );
+  gtk_box_pack_start ( GTK_BOX(main_hbox), left_vbox, FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(main_hbox), right_vbox, TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(main_hbox), diff_left_vbox, FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(main_hbox), diff_right_vbox, FALSE, FALSE, TPWIN_PAD );
 
-  gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(tpwin))), main_hbox, FALSE, FALSE, 0 );
+  GtkWidget *main_vbox = gtk_vbox_new ( FALSE, 1 );
+  gtk_box_pack_start ( GTK_BOX(main_vbox), main_hbox, FALSE, FALSE, TPWIN_PAD );
+
+  static gchar *ext_texts[] = { N_("<b>Heart Rate:</b>"),
+                                N_("<b>Cadence:</b>"),
+                                N_("<b>Temperature:</b>"),
+                                N_("<b>Power:</b>"), };
+
+  GtkWidget *vbox_ext_values = gtk_vbox_new ( TRUE, 1 );
+  tpwin->heart_rate = GTK_LABEL(ui_label_new_selectable(NULL));
+  tpwin->cadence    = GTK_LABEL(ui_label_new_selectable(NULL));
+  tpwin->temp       = GTK_LABEL(ui_label_new_selectable(NULL));
+  tpwin->power    = GTK_LABEL(ui_label_new_selectable(NULL));
+  gtk_box_pack_start ( GTK_BOX(vbox_ext_values), GTK_WIDGET(tpwin->heart_rate), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(vbox_ext_values), GTK_WIDGET(tpwin->cadence), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(vbox_ext_values), GTK_WIDGET(tpwin->temp), TRUE, TRUE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(vbox_ext_values), GTK_WIDGET(tpwin->power), TRUE, TRUE, TPWIN_PAD );
+
+  tpwin->expander = gtk_expander_new_with_mnemonic ( _("_Additional") );
+  gtk_expander_set_expanded ( GTK_EXPANDER(tpwin->expander), expanded );
+  gtk_box_pack_start ( GTK_BOX(main_vbox), tpwin->expander, FALSE, FALSE, TPWIN_PAD );
+
+  GtkWidget *vbox_ext_texts = a_dialog_create_label_vbox ( ext_texts, G_N_ELEMENTS(ext_texts), 1, TPWIN_PAD );
+  GtkWidget *hbox_ext = gtk_hbox_new ( FALSE, 5 );
+  gtk_container_add ( GTK_CONTAINER(tpwin->expander), hbox_ext );
+  gtk_box_pack_start ( GTK_BOX(hbox_ext), vbox_ext_texts, FALSE, FALSE, TPWIN_PAD );
+  gtk_box_pack_start ( GTK_BOX(hbox_ext), vbox_ext_values, FALSE, FALSE, TPWIN_PAD );
+
+  tpwin->extsw = gtk_scrolled_window_new ( NULL, NULL );
+  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(tpwin->extsw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+  tpwin->extlab = GTK_LABEL(ui_label_new_selectable ( NULL ));
+  gtk_widget_set_can_focus ( GTK_WIDGET(tpwin->extlab), FALSE ); // Don't let notebook autofocus on it
+  gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW(tpwin->extsw), GTK_WIDGET(tpwin->extlab) );
+
+  tpwin->tabs = gtk_notebook_new();
+  //gtk_notebook_append_page ( GTK_NOTEBOOK(tpwin->tabs), main_hbox, gtk_label_new(_("General")) );
+  gtk_notebook_append_page ( GTK_NOTEBOOK(tpwin->tabs), main_vbox, gtk_label_new(_("General")) );
+  gtk_notebook_append_page ( GTK_NOTEBOOK(tpwin->tabs), tpwin->extsw, gtk_label_new(_("GPX Extensions")) );
+  gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(tpwin))), tpwin->tabs, FALSE, FALSE, 0 );
+  gtk_notebook_set_show_tabs ( GTK_NOTEBOOK(tpwin->tabs), FALSE ); // Hide the tabs until really needed
 
   tpwin->cur_tp = NULL;
 
-  GtkWidget *response_w = NULL;
-#if GTK_CHECK_VERSION (2, 20, 0)
-  response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_CLOSE );
-#endif
+  GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(tpwin), VIK_TRW_LAYER_TPWIN_CLOSE );
   if ( response_w )
     gtk_widget_grab_focus ( response_w );
 
@@ -352,7 +423,8 @@ VikTrwLayerTpwin *vik_trw_layer_tpwin_new ( GtkWindow *parent )
  */
 void vik_trw_layer_tpwin_destroy ( VikTrwLayerTpwin *tpwin )
 {
-  // Save the size before closing the dialog
+  // Save the size and state before closing the dialog
+  expanded = gtk_expander_get_expanded ( GTK_EXPANDER(tpwin->expander) );
   gtk_window_get_size ( GTK_WINDOW(tpwin), &width, &height );
   gtk_widget_destroy ( GTK_WIDGET(tpwin) );
 }
@@ -387,6 +459,13 @@ void vik_trw_layer_tpwin_set_empty ( VikTrwLayerTpwin *tpwin )
   gtk_label_set_text ( tpwin->pdop, NULL );
   gtk_label_set_text ( tpwin->sat, NULL );
 
+  gtk_label_set_text ( tpwin->extlab, NULL );
+
+  gtk_label_set_text ( tpwin->heart_rate, NULL );
+  gtk_label_set_text ( tpwin->cadence, NULL );
+  gtk_label_set_text ( tpwin->temp, NULL );
+  gtk_label_set_text ( tpwin->power, NULL );
+
   gtk_window_set_title ( GTK_WINDOW(tpwin), _("Trackpoint") );
 }
 
@@ -433,7 +512,6 @@ void vik_trw_layer_tpwin_set_tp ( VikTrwLayerTpwin *tpwin, GList *tpl, const gch
     GtkWidget *img = gtk_image_new_from_stock ( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU );
     gtk_button_set_image ( GTK_BUTTON(tpwin->time), img );
   }
-  else
 
   vik_trw_layer_tpwin_set_track_name ( tpwin, track_name );
 
@@ -466,19 +544,10 @@ void vik_trw_layer_tpwin_set_tp ( VikTrwLayerTpwin *tpwin, GList *tpl, const gch
   vik_units_distance_t dist_units = a_vik_get_units_distance ();
   if ( tpwin->cur_tp )
   {
-    switch (dist_units) {
-    case VIK_UNITS_DISTANCE_KILOMETRES:
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.2f m", vik_coord_diff(&(tp->coord), &(tpwin->cur_tp->coord)));
-      break;
-    case VIK_UNITS_DISTANCE_MILES:
-    case VIK_UNITS_DISTANCE_NAUTICAL_MILES:
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.2f yards", vik_coord_diff(&(tp->coord), &(tpwin->cur_tp->coord))*1.0936133);
-      break;
-    default:
-      g_critical("Houston, we've had a problem. distance=%d", dist_units);
-    }
-
+    gdouble diff_dist = vik_coord_diff(&(tp->coord), &(tpwin->cur_tp->coord));
+    vu_distance_text_precision ( tmp_str, sizeof(tmp_str), dist_units, diff_dist, "%.2f" );
     gtk_label_set_text ( tpwin->diff_dist, tmp_str );
+
     if ( !isnan(tp->timestamp) && !isnan(tpwin->cur_tp->timestamp) )
     {
       g_snprintf ( tmp_str, sizeof(tmp_str), "%.3f s", tp->timestamp - tpwin->cur_tp->timestamp);
@@ -488,7 +557,7 @@ void vik_trw_layer_tpwin_set_tp ( VikTrwLayerTpwin *tpwin, GList *tpl, const gch
       else
       {
         gdouble tmp_speed = vik_coord_diff(&(tp->coord), &(tpwin->cur_tp->coord)) / ABS(tp->timestamp - tpwin->cur_tp->timestamp);
-        vu_speed_text ( tmp_str, sizeof(tmp_str), speed_units, tmp_speed, TRUE, "%.2f" );
+        vu_speed_text ( tmp_str, sizeof(tmp_str), speed_units, tmp_speed, TRUE, "%.2f", FALSE );
         gtk_label_set_text ( tpwin->diff_speed, tmp_str );
       }
     }
@@ -502,41 +571,17 @@ void vik_trw_layer_tpwin_set_tp ( VikTrwLayerTpwin *tpwin, GList *tpl, const gch
   if ( isnan(tp->course) )
     g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
   else
-    g_snprintf ( tmp_str, sizeof(tmp_str), "%05.1f\302\260", tp->course );
+    g_snprintf ( tmp_str, sizeof(tmp_str), "%05.1f%s", tp->course, DEGREE_SYMBOL );
   gtk_label_set_text ( tpwin->course, tmp_str );
 
-  vu_speed_text ( tmp_str, sizeof(tmp_str), speed_units, tp->speed, TRUE, "%.2f" );
+  vu_speed_text ( tmp_str, sizeof(tmp_str), speed_units, tp->speed, TRUE, "%.2f", FALSE );
   gtk_label_set_text ( tpwin->speed, tmp_str );
 
-  switch (dist_units) {
-  case VIK_UNITS_DISTANCE_KILOMETRES:
-    if ( isnan(tp->hdop) )
-      g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
-    else
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.5f m", tp->hdop );
-    gtk_label_set_text ( tpwin->hdop, tmp_str );
-    if ( isnan(tp->pdop) )
-      g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
-    else
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.5f m", tp->pdop );
-    gtk_label_set_text ( tpwin->pdop, tmp_str );
-    break;
-  case VIK_UNITS_DISTANCE_NAUTICAL_MILES:
-  case VIK_UNITS_DISTANCE_MILES:
-    if ( isnan(tp->hdop) )
-      g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
-    else
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.5f yards", tp->hdop*1.0936133 );
-    gtk_label_set_text ( tpwin->hdop, tmp_str );
-    if ( isnan(tp->pdop) )
-      g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
-    else
-      g_snprintf ( tmp_str, sizeof(tmp_str), "%.5f yards", tp->pdop*1.0936133 );
-    gtk_label_set_text ( tpwin->pdop, tmp_str );
-    break;
-  default:
-    g_critical("Houston, we've had a problem. distance=%d", dist_units);
-  }
+  vu_distance_text_precision ( tmp_str, sizeof(tmp_str), dist_units, tp->hdop, "%.5f" );
+  gtk_label_set_text ( tpwin->hdop, tmp_str );
+
+  vu_distance_text_precision ( tmp_str, sizeof(tmp_str), dist_units, tp->pdop, "%.5f" );
+  gtk_label_set_text ( tpwin->pdop, tmp_str );
 
   if ( isnan(tp->vdop) )
     g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
@@ -558,6 +603,39 @@ void vik_trw_layer_tpwin_set_tp ( VikTrwLayerTpwin *tpwin, GList *tpl, const gch
   g_snprintf ( tmp_str, sizeof(tmp_str), "%d / %d", tp->nsats, tp->fix_mode );
   gtk_label_set_text ( tpwin->sat, tmp_str );
 
+  if ( tp->cadence != VIK_TRKPT_CADENCE_NONE )
+    g_snprintf ( tmp_str, sizeof(tmp_str), _("%d RPM"), tp->cadence );
+  else
+    g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
+  gtk_label_set_text ( tpwin->cadence, tmp_str );
+
+  if ( tp->heart_rate )
+    g_snprintf ( tmp_str, sizeof(tmp_str), _("%d bpm"), tp->heart_rate );
+  else
+    g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
+  gtk_label_set_text ( tpwin->heart_rate, tmp_str );
+
+  if ( !isnan(tp->temp) )
+    if ( a_vik_get_units_temp() == VIK_UNITS_TEMP_CELSIUS )
+      g_snprintf ( tmp_str, sizeof(tmp_str), "%.1f%sC", tp->temp, DEGREE_SYMBOL );
+    else
+      g_snprintf ( tmp_str, sizeof(tmp_str), "%.1f%sF", VIK_CELSIUS_TO_FAHRENHEIT(tp->temp), DEGREE_SYMBOL );
+  else
+    g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
+  gtk_label_set_text ( tpwin->temp, tmp_str );
+
+  if ( tp->power != VIK_TRKPT_POWER_NONE )
+    g_snprintf ( tmp_str, sizeof(tmp_str), _("%d Watts"), tp->power );
+  else
+    g_snprintf ( tmp_str, sizeof(tmp_str), "--" );
+  gtk_label_set_text ( tpwin->power, tmp_str );
+
+  gtk_label_set_text ( tpwin->extlab, tp->extensions );
+  GtkWidget *ext_tab = gtk_notebook_get_tab_label ( GTK_NOTEBOOK(tpwin->tabs), tpwin->extsw );
+  gtk_widget_set_sensitive ( ext_tab, tp->extensions ? TRUE : FALSE );
+  if ( tp->extensions )
+    gtk_notebook_set_show_tabs ( GTK_NOTEBOOK(tpwin->tabs), TRUE );
+
   tpwin->cur_tp = tp;
 }
 
