@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2013-2015, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2013-2020, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,12 @@
 // Long formatted date+basic time - listing this way ensures the string comparison sort works - so no local type format %x or %c here!
 #define WAYPOINT_LIST_DATE_FORMAT "%Y-%m-%d %H:%M"
 
+// For simplicity these are global values
+//  (i.e. would get shared between multiple windows)
+static gint width = 0;
+static gint height = 400;
+static gboolean configured = FALSE;
+
 /**
  * waypoint_close_cb:
  *
@@ -39,6 +45,24 @@ static void waypoint_close_cb ( GtkWidget *dialog, gint resp, GList *data )
 	gtk_widget_destroy (dialog);
 }
 
+static gboolean configure_event ( GtkWidget *widget, GdkEventConfigure *event, gpointer notused )
+{
+	if ( !configured ) {
+		configured = TRUE;
+
+		// Allow sizing back down to a minimum
+		GdkGeometry geom = { event->width, event->height, 0, 0, 0, 0, 0, 0, 0, 0, GDK_GRAVITY_STATIC };
+		gtk_window_set_geometry_hints ( GTK_WINDOW(widget), widget, &geom, GDK_HINT_BASE_SIZE );
+
+		// Restore previous size
+		gtk_window_resize ( GTK_WINDOW(widget), width, height );
+	} else {
+		width = event->width;
+		height = event->height;
+	}
+	return FALSE;
+}
+
 #define WPT_LIST_COLS 9
 #define WPT_COL_NUM WPT_LIST_COLS-1
 #define TRW_COL_NUM WPT_COL_NUM-1
@@ -146,16 +170,7 @@ static void trw_layer_waypoint_properties ( menu_array_values values )
 		GtkWidget *gw = gtk_widget_get_toplevel ( values[MA_TREEVIEW] );
 		waypoint_close_cb ( gw, 0, values[MA_WPTS_LIST] );
 
-		gboolean updated = FALSE;
-		gchar *new_name = a_dialog_waypoint ( VIK_GTK_WINDOW_FROM_LAYER(vtl), wpt->name, vtl, wpt, vik_trw_layer_get_coord_mode(vtl), FALSE, &updated );
-		if ( new_name )
-			trw_layer_waypoint_rename ( vtl, wpt, new_name );
-
-		if ( updated )
-			trw_layer_waypoint_reset_icon ( vtl, wpt );
-
-		if ( updated && VIK_LAYER(vtl)->visible )
-			vik_layer_emit_update ( VIK_LAYER(vtl) );
+		trw_layer_wpwin_set ( vtl, wpt, vik_trw_layer_wpwin_show ( VIK_GTK_WINDOW_FROM_LAYER(vtl), NULL, wpt->name, vtl, wpt, vik_trw_layer_get_coord_mode(vtl), FALSE ) );
 	}
 }
 
@@ -169,7 +184,7 @@ static void trw_layer_waypoint_view ( menu_array_values values )
 
 	trw_layer_waypoint_select (values);
 
-	vik_layer_emit_update ( VIK_LAYER(vtl) );
+	vik_layer_emit_update ( VIK_LAYER(vtl), FALSE );
 }
 
 static void trw_layer_show_picture ( menu_array_values values )
@@ -278,7 +293,7 @@ static void add_copy_menu_items ( GtkMenu *menu, GtkWidget *tree_view )
 {
 	(void)vu_menu_add_item ( menu, _("_Copy Data"), GTK_STOCK_COPY,
 	                         G_CALLBACK(trw_layer_copy_selected_only_visible_columns), tree_view );
-	(void)vu_menu_add_item ( menu, _("_Copy Data (with _positions)"), GTK_STOCK_COPY,
+	(void)vu_menu_add_item ( menu, _("Copy Data (with _positions)"), GTK_STOCK_COPY,
 	                         G_CALLBACK(trw_layer_copy_selected_with_position), tree_view );
 }
 
@@ -427,24 +442,14 @@ static void trw_layer_waypoint_list_add ( vik_trw_waypoint_list_t *vtdl,
 	gchar time_buf[32];
 	time_buf[0] = '\0';
 	if ( !isnan(wpt->timestamp) ) {
-
-#if GLIB_CHECK_VERSION(2,26,0)
-		GDateTime* gdt = g_date_time_new_from_unix_utc ( wpt->timestamp );
-		gchar *time = g_date_time_format ( gdt, date_format );
+		time_t tt = wpt->timestamp;
+		gchar *time = vu_get_time_string ( &tt, date_format, &wpt->coord, NULL );
 		g_strlcpy ( time_buf, time, sizeof(time_buf) );
 		g_free ( time );
-		g_date_time_unref ( gdt);
-#else
-		GDate* gdate_start = g_date_new ();
-		g_date_set_time_t ( gdate_start, wpt->timestamp );
-		g_date_strftime ( time_buf, sizeof(time_buf), date_format, gdate_start );
-		g_date_free ( gdate_start );
-#endif
 	}
 
-	// NB: doesn't include aggegrate visibility
-	gboolean visible = VIK_LAYER(vtl)->visible && wpt->visible;
-	visible = visible && vik_trw_layer_get_waypoints_visibility(vtl);
+	gboolean visible = wpt->visible && vik_trw_layer_get_waypoints_visibility ( vtl );
+	visible = visible && vik_treeview_item_get_visible_tree ( VIK_LAYER(vtl)->vt, &(VIK_LAYER(vtl)->iter) );
 
 	gdouble alt = wpt->altitude;
 	if ( isnan(alt) ) {
@@ -472,6 +477,67 @@ static void trw_layer_waypoint_list_add ( vik_trw_waypoint_list_t *vtdl,
 	                     -1 );
 }
 
+static gboolean
+filter_waypoints_cb (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	const gchar *filter = gtk_entry_get_text ( GTK_ENTRY(data) );
+
+	if ( filter == NULL || strlen ( filter ) == 0 )
+		return TRUE;
+
+	gchar *filter_case = g_utf8_casefold ( filter, -1 );
+
+	VikWaypoint *wpt;
+	gtk_tree_model_get ( model, iter, WPT_COL_NUM, &wpt, -1 );
+
+	if ( wpt == NULL )
+		return FALSE;
+
+	gboolean visible = FALSE;
+
+	gchar *layer_name;
+	gtk_tree_model_get ( model, iter, 0, &layer_name, -1 );
+	if ( layer_name != NULL ) {
+		gchar *layer_name_case = g_utf8_casefold ( layer_name, -1 );
+		visible |= ( strstr ( layer_name_case, filter_case ) != NULL );
+		g_free ( layer_name_case );
+	}
+	g_free ( layer_name );
+
+	gchar *wpt_name_case = g_utf8_casefold ( wpt->name, -1 );
+	visible |= ( strstr ( wpt_name_case, filter_case ) != NULL );
+	g_free ( wpt_name_case );
+
+	gchar *time_buf;
+	gtk_tree_model_get ( model, iter, 2, &time_buf, -1 );
+	visible |= (time_buf != NULL && strstr ( time_buf, filter_case )  != NULL);
+	g_free ( time_buf );
+
+	if ( wpt->comment != NULL ) {
+		gchar *wpt_comment_case = g_utf8_casefold ( wpt->comment, -1 );
+		visible |= ( strstr ( wpt_comment_case, filter_case ) != NULL );
+		g_free ( wpt_comment_case );
+	}
+
+	if ( wpt->description != NULL ) {
+		gchar *wpt_description_case = g_utf8_casefold ( wpt->description, -1 );
+		visible |= ( strstr ( wpt_description_case, filter_case ) != NULL );
+		g_free ( wpt_description_case );
+	}
+
+	g_free ( filter_case );
+
+	return visible;
+}
+
+static void
+filter_changed_cb (GtkEntry   *entry,
+                   GParamSpec *pspec,
+                   gpointer   data)
+{
+    gtk_tree_model_filter_refilter( GTK_TREE_MODEL_FILTER(data) );
+}
+
 /*
  * Instead of comparing the pixbufs,
  *  look at the waypoint data and compare the symbol (as text).
@@ -532,7 +598,6 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	// It's simple storing the gdouble values in the tree store as the sort works automatically
 	// Then apply specific cell data formatting (rather default double is to 6 decimal places!)
 	// However not storing any doubles for waypoints ATM
-	// TODO: Consider adding the waypoint icon into this store for display in the list
 	GtkTreeStore *store = gtk_tree_store_new ( WPT_LIST_COLS,
 	                                           G_TYPE_STRING,    // 0: Layer Name
 	                                           G_TYPE_STRING,    // 1: Waypoint Name
@@ -613,7 +678,10 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	gtk_tree_view_column_set_sort_column_id ( column, column_runner );
 	gtk_tree_view_append_column ( GTK_TREE_VIEW(view), column );
 
-	gtk_tree_view_set_model ( GTK_TREE_VIEW(view), GTK_TREE_MODEL(store) );
+	GtkTreeModelFilter *model = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new ( GTK_TREE_MODEL(store), NULL));
+	GtkTreeModelSort *sorted = GTK_TREE_MODEL_SORT(gtk_tree_model_sort_new_with_model ( GTK_TREE_MODEL(model) ));
+
+	gtk_tree_view_set_model ( GTK_TREE_VIEW(view), GTK_TREE_MODEL(sorted) );
 	gtk_tree_selection_set_mode ( gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), GTK_SELECTION_MULTIPLE );
 	gtk_tree_view_set_rules_hint ( GTK_TREE_VIEW(view), TRUE );
 
@@ -631,14 +699,24 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	g_signal_connect ( view, "popup-menu", G_CALLBACK(trw_layer_waypoint_menu_popup), waypoints_and_layers );
 	g_signal_connect ( view, "button-press-event", G_CALLBACK(trw_layer_waypoint_button_pressed), waypoints_and_layers );
 
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), scrolledwindow, TRUE, TRUE, 0);
+	GtkWidget *filter_entry = ui_entry_new( NULL, GTK_ENTRY_ICON_SECONDARY );
+	g_signal_connect ( filter_entry, "notify::text", G_CALLBACK(filter_changed_cb), model );
+	gtk_tree_model_filter_set_visible_func ( model, filter_waypoints_cb, filter_entry, NULL );
+
+	GtkWidget *filter_label = gtk_label_new ( _("Filter") );
+	g_object_set ( filter_label, "has-tooltip", TRUE, NULL );
+	g_signal_connect ( filter_label, "query-tooltip", G_CALLBACK(ui_tree_model_number_tooltip_cb), model );
+
+	GtkWidget *filter_box = gtk_hbox_new( FALSE, 10 );
+	gtk_box_pack_start (GTK_BOX(filter_box), filter_label, FALSE, TRUE, 10);
+	gtk_box_pack_start (GTK_BOX(filter_box), filter_entry, TRUE, TRUE, 10);
+
+	GtkBox *vbox = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
+	gtk_box_pack_start (vbox, scrolledwindow, TRUE, TRUE, 0);
+	gtk_box_pack_end (vbox, filter_box, FALSE, TRUE, 10);
 
 	// Set ordering of the initial view by one of the name columns
 	gtk_tree_view_column_clicked ( sort_by_column );
-
-	// Ensure a reasonable number of items are shown
-	//  TODO: may be save window size, column order, sorted by between invocations.
-	gtk_window_set_default_size ( GTK_WINDOW(dialog), show_layer_names ? 700 : 500, 400 );
 }
 
 
@@ -673,9 +751,14 @@ void vik_trw_layer_waypoint_list_show_dialog ( gchar *title,
 	// Use response to close the dialog with tidy up
 	g_signal_connect ( G_OBJECT(dialog), "response", G_CALLBACK(waypoint_close_cb), gl );
 
+	configured = FALSE;
+	g_signal_connect ( G_OBJECT(dialog), "configure-event", G_CALLBACK(configure_event), NULL );
+
 	gtk_widget_show_all ( dialog );
-	// Yes - set the size *AGAIN* - this time widgets are expanded nicely
-	gtk_window_resize ( GTK_WINDOW(dialog), show_layer_names ? 800 : 600, 400 );
+
+	// Ensure a reasonable number of items are shown
+	if ( width == 0 )
+		width = show_layer_names ? 800 : 650;
 
 	// ATM lock out on dialog run - to prevent list contents being manipulated in other parts of the GUI whilst shown here.
 	gtk_dialog_run (GTK_DIALOG (dialog));
