@@ -40,14 +40,27 @@ typedef struct _VikWaypoint VikWaypoint;
 struct _VikWaypoint {
   VikCoord coord;
   gboolean visible;
-  gdouble timestamp; /* NAN if data unavailable */
+  gdouble timestamp;         /* NAN if data unavailable */
   gdouble altitude;
+  gdouble course;            /* NAN if data unavailable */
+  gdouble speed;             /* NAN if data unavailable */
+  gdouble magvar;            /* NAN if data unavailable */
+  gdouble geoidheight;       /* NAN if data unavailable */
+  gboolean hide_name;
   gchar *name;
   gchar *comment;
   gchar *description;
   gchar *source;
-  gchar *type;
   gchar *url;
+  gchar *url_name;
+  gchar *type;
+  guint fix_mode;            /* VIK_GPS_MODE_NOT_SEEN if data unavailable */
+  guint nsats;               /* number of satellites used. 0 if data unavailable */
+  gdouble hdop;              /* NAN if data unavailable */
+  gdouble vdop;              /* NAN if data unavailable */
+  gdouble pdop;              /* NAN if data unavailable */
+  gdouble ageofdgpsdata;     /* NAN if data unavailable */
+  guint dgpsid;              /* 0 .. 1023 */
   gchar *image;
   // NB Only really applicable if geotagging(exif info) is being used
   gdouble image_direction;   /* NAN if data unavailable */
@@ -58,6 +71,7 @@ struct _VikWaypoint {
   guint8 image_width;
   guint8 image_height;
   gchar *symbol;
+  gchar *extensions;         // GPX 1.1
   // Only for GUI display
   GdkPixbuf *symbol_pixbuf;
 };
@@ -69,9 +83,11 @@ void vik_waypoint_set_description(VikWaypoint *wp, const gchar *description);
 void vik_waypoint_set_source(VikWaypoint *wp, const gchar *source);
 void vik_waypoint_set_type(VikWaypoint *wp, const gchar *type);
 void vik_waypoint_set_url(VikWaypoint *wp, const gchar *url);
+void vik_waypoint_set_url_name(VikWaypoint *wp, const gchar *url_name);
 void vik_waypoint_set_image(VikWaypoint *wp, const gchar *image);
 void vik_waypoint_set_image_direction_info(VikWaypoint *wp, gdouble direction, VikWaypointImageDirectionRef direction_ref);
 void vik_waypoint_set_symbol(VikWaypoint *wp, const gchar *symname);
+void vik_waypoint_set_extensions(VikWaypoint *wp, const gchar *value);
 void vik_waypoint_free(VikWaypoint * wp);
 VikWaypoint *vik_waypoint_copy(const VikWaypoint *wp);
 void vik_waypoint_set_comment_no_copy(VikWaypoint *wp, gchar *comment);
