@@ -29,6 +29,7 @@
 
 #include "vikviewport.h"
 #include "vikstatus.h"
+#include "viktrack.h"
 
 G_BEGIN_DECLS
 
@@ -71,6 +72,8 @@ struct _VikLayersPanel * vik_window_layers_panel(VikWindow *vw);
 struct _VikStatusbar * vik_window_get_statusbar(VikWindow *vw);
 const gchar *vik_window_get_filename(VikWindow *vw);
 
+gboolean vik_window_save_file_as ( VikWindow *vw, gpointer val ); // gpointer is a VikAggregateLayer
+
 void vik_window_statusbar_update (VikWindow *vw, const gchar* message, vik_statusbar_type_t vs_type);
 
 void vik_window_set_redraw_trigger(struct _VikLayer *vl);
@@ -81,8 +84,8 @@ gpointer vik_window_get_selected_trw_layer ( VikWindow *vw ); /* return type Vik
 void vik_window_set_selected_trw_layer ( VikWindow *vw, gpointer vtl ); /* input VikTrwLayer */
 GHashTable *vik_window_get_selected_tracks ( VikWindow *vw );
 void vik_window_set_selected_tracks ( VikWindow *vw, GHashTable *ght, gpointer vtl ); /* gpointer is a VikTrwLayer */
-gpointer vik_window_get_selected_track ( VikWindow *vw ); /* return type VikTrack */
-void vik_window_set_selected_track ( VikWindow *vw, gpointer *vt, gpointer vtl ); /* gpointer is a VikTrwLayer */
+VikTrack *vik_window_get_selected_track ( VikWindow *vw );
+void vik_window_set_selected_track ( VikWindow *vw, VikTrack *vt, gpointer vtl ); /* gpointer is a VikTrwLayer */
 GHashTable *vik_window_get_selected_waypoints ( VikWindow *vw );
 void vik_window_set_selected_waypoints ( VikWindow *vw, GHashTable *ght, gpointer vtl ); /* gpointer is a VikTrwLayer */
 gpointer vik_window_get_selected_waypoint ( VikWindow *vw ); /* return type VikWaypoint */
@@ -99,26 +102,38 @@ void vik_window_clear_busy_cursor ( VikWindow *vw );
 void vik_window_set_busy_cursor_widget ( GtkWidget *widget, VikWindow *vw );
 void vik_window_clear_busy_cursor_widget ( GtkWidget *widget, VikWindow *vw );
 
+void vik_window_set_modified ( VikWindow *vw );
+
 typedef struct {
   VikWindow *vw;
   VikViewport *vvp;
-  gpointer vtl; // VikTrwlayer
+  gpointer vtl; // VikTrwLayer
   gboolean holding;
   gboolean moving;
   gboolean is_waypoint; // otherwise a track
   GdkGC *gc;
+  GdkColor color; // For GTK3+ use as no longer in the gc
   int oldx, oldy;
   // Monitor the bounds for the tool with shift modifier
   gboolean bounds_active;
   gint start_x;
   gint start_y;
+#if !GTK_CHECK_VERSION (3,0,0)
   GdkPixmap *pixmap;
+#endif
   // The following are mostly for ruler tool
   gboolean has_oldcoord;
   VikCoord oldcoord;
   gboolean displayed;
 } tool_ed_t;
 
+tool_ed_t* tool_edit_create ( VikWindow *vw, VikViewport *vvp );
+void tool_edit_destroy ( tool_ed_t *te );
+void tool_edit_remove_image ( tool_ed_t *te );
+
+gpointer vik_window_get_active_tool_interface ( VikWindow *vw ); // returns VikToolInterface*
+gpointer vik_window_get_active_tool_data ( VikWindow *vw );
+
 VikWindow *a_vik_window_get_a_window ();
 
 G_END_DECLS
