@@ -8,9 +8,6 @@
 
 int main(int argc, char *argv[])
 {
-#if !GLIB_CHECK_VERSION (2, 36, 0)
-  g_type_init();
-#endif
   // Some stuff must be initialized as it gets auto used
   a_settings_init ();
   a_preferences_init ();
@@ -20,10 +17,12 @@ int main(int argc, char *argv[])
   VikLayer *vl = vik_layer_create (VIK_LAYER_TRW, NULL, FALSE);
   VikTrwLayer *trw = VIK_TRW_LAYER (vl);
 
-  a_gpx_read_file(trw, stdin, NULL);
+  a_gpx_read_file(trw, stdin, NULL, FALSE);
   a_gpx_write_file(trw, stdout, NULL, NULL);
-  // NB no layer_free functions directly visible anymore
-  //  automatically called by layers_panel_finalize cleanup in full Viking program
+
+  g_object_unref ( vl );
+
+  vik_trwlayer_uninit ();
 
   a_layer_defaults_uninit ();
   a_preferences_uninit ();
