@@ -6,10 +6,6 @@
 
 int main(int argc, char *argv[])
 {
-#if !GLIB_CHECK_VERSION(2,36,0)
-  g_type_init ();
-#endif
-
   if ( !argv[1] ) {
     g_printerr ( "Nothing specified\n" );
     return 1;
