@@ -358,6 +358,8 @@ def get_tile_path (tid):
         return "Mapbox-Outdoors"
     elif tile_id == 212:
         return "Bing-Aerial"
+    elif tile_id == 901:
+        return "OpenTopoMap"
     # Default extension Map ids (from data/maps.xml)
     elif tile_id == 29:
         return "CalTopo"
@@ -400,9 +402,9 @@ Here one must specify the output directory name explicitly and set your maps.xml
 
 parser.add_option('-t', '--tileid', dest='tileid',
     action="store",
-    help='''Tile id of Viking map cache to use (19 if not specified as this is Viking's default (MaqQuest))''',
+    help='''Tile id of Viking map cache to use (901 if not specified as this is Viking's default (OpenTopoMap))''',
     type='string',
-    default='19')
+    default='901')
 
 parser.add_option('-n', '--nooptimize', dest='nooptimize',
     action="store_true",
